//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         FilesDlg.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Implementation of Class FilesDlg.
//
//////////////////////////////////////////////////////////////////////////////////////////////

#include "direct.h"
#include "stdafx.h"
#include "diary.h"
#include "FilesDlg.h"
#include "toolkit.h"
#include "setting.h"
#include "diarydlg.h"
#include "monitors.h"
#include "multimonitor.h"
#include "monitordc.h"

// FilesDlg dialog

IMPLEMENT_DYNAMIC(SFilesDlg, CDialog)
SFilesDlg::SFilesDlg(CWnd* pParent /*=NULL*/)
	: CDialog(SFilesDlg::IDD, pParent)
{
	ivDiaryDlg = (SDiaryDlg *)pParent;
}

SFilesDlg::~SFilesDlg()
{
}

void SFilesDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(SFilesDlg, CDialog)
	ON_BN_CLICKED(ID_STORE, OnBnClickedStore)
	ON_BN_CLICKED(ID_RETRIEVE, OnBnClickedRetrieve)
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
	ON_BN_CLICKED(ID_DELETE_FILE, OnBnClickedDeleteFile)
	ON_BN_CLICKED(ID_OPEN, &SFilesDlg::OnBnClickedOpen)
END_MESSAGE_MAP()

// SavePosition does nothing else than retrieve the window position, 
// and store it in the settings object. It is being called from all button and 
// radiobutton handlers, so position can be restored once the dialog is again
// opened...

void SFilesDlg::SavePosition(void)
{
   TK_LOG0("Save window position to settings.dat");
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);

   // cancel save if window centered, and not moved yet...
   if (ivWindowCentered == true &&
	   (ivTop == lWinPlace.rcNormalPosition.top ||
	    ivLeft == lWinPlace.rcNormalPosition.left))
   {
	   return;
   }

   // ok, we save so set the flag to off
   ivWindowCentered = false;
   ivTop = -1;
   ivLeft = -1;

   TK_LOG0("Get the Top position value");
   char *lTop = new char[MAX_STRING];
   _itoa_s(lWinPlace.rcNormalPosition.top, lTop, MAX_STRING - 1, 10);
   SSetting *lTopSSetting = new SSetting(SET_FILES_TOP, lTop);
   this->ivDiaryDlg->GetSettings()->AddSetting(lTopSSetting);
   delete [] lTop;
 
   TK_LOG0("Get the Left position value");
   char *lLeft = new char[MAX_STRING];
   _itoa_s(lWinPlace.rcNormalPosition.left, lLeft, MAX_STRING - 1, 10);
   SSetting *lLeftSSetting = new SSetting(SET_FILES_LEFT, lLeft);
   this->ivDiaryDlg->GetSettings()->AddSetting(lLeftSSetting);
   delete [] lLeft;

   TK_LOG0("Make sure the settings are flushed to disk");
   this->ivDiaryDlg->GetSettings()->WriteFile();
}

BOOL SFilesDlg::OnInitDialog(void)
{
   BOOL lRetCode = CDialog::OnInitDialog();

   TK_LOG0("Read the position settings and restore them");
   SSetting *lSettingTop    = this->ivDiaryDlg
                                 ->GetSettings()->GetSetting(SET_FILES_TOP);
   SSetting *lSettingLeft   = this->ivDiaryDlg
                                 ->GetSettings()->GetSetting(SET_FILES_LEFT);

   if (lSettingTop  != NULL &&
       lSettingLeft != NULL)
   {
      TK_LOG0("Position settings found, positioning window");

      int lTop  = lSettingTop->GetValueAsNumber();
      int lLeft = lSettingLeft->GetValueAsNumber();

	  // make sure it will be on-screen, but keep the original position in config
	  this->SetWindowPos(0, lLeft, lTop, 0, 0, SWP_NOSIZE);
	  if (CMonitors::IsOnScreen( this ) == false)
	  {
		  CMonitor thePrimary = CMonitors::GetPrimaryMonitor();
		  thePrimary.CenterWindowToMonitor(this, true);
		  ivWindowCentered = true;
          WINDOWPLACEMENT lWinPlace;
          this->GetWindowPlacement(&lWinPlace);
          ivTop = lWinPlace.rcNormalPosition.top;
          ivLeft = lWinPlace.rcNormalPosition.left;
	  }
   }

   // this might fail because it already exists...
   _mkdir(mDirName);

   char  *lTitle = new char[MAX_STRING];
   char  *lCurDir = new char[MAX_STRING];
   GetCurrentDirectory(MAX_STRING, lCurDir);
   strcat_s(lCurDir, MAX_STRING - 1, "\\");
   strcat_s(lCurDir, MAX_STRING - 1, mDirName);
   sprintf_s(lTitle, MAX_STRING - 1,  
             "%s - Files", 
             lCurDir);
   TK_LOG1("Set Window title to %s", lTitle);
   SetWindowText(lTitle);

   FillListbox();

   delete [] lTitle;
   delete [] lCurDir;

   return lRetCode;
}

// SFilesDlg message handlers

void SFilesDlg::OnBnClickedStore()
{
    this->SavePosition();

    static char BASED_CODE szFilesFilter[] = "All Files (*.*)|*.*||";

    HRESULT hResult;

    DWORD lDirReturn = 0;
    char  *lPath = new char[MAX_STRING];

    TK_LOG0("Retrieve the path of execution");
    lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);

    CFileDialog dlg(TRUE, NULL, NULL, OFN_FILEMUSTEXIST, szFilesFilter);
    m_nFilterLoad = 0;
    dlg.m_ofn.nFilterIndex = m_nFilterLoad;

    hResult = (int)dlg.DoModal();
    if(hResult != IDOK) 
    {
        SetCurrentDirectory(lPath);
		delete [] lPath;
        return;
    }

    SetCurrentDirectory(lPath);
	delete [] lPath;
	if (dlg.GetPathName() != "")
	{
		AddFileToBox(dlg.GetPathName());
	}
}

void SFilesDlg::OnBnClickedRetrieve()
{
    this->SavePosition();

	CListBox *theBox = (CListBox *)(this->GetDlgItem(IDC_FILELIST));

    int lCurSel = theBox->GetCurSel();
    if (lCurSel == LB_ERR)
    {
       AfxMessageBox("Select a file first");
       return;
    }

    CString lSelString;
    theBox->GetText(lCurSel, lSelString);

    CString lFileName = "";
	char theCurDir[MAX_STRING];
	_getcwd(theCurDir, MAX_STRING - 1);
	lFileName = theCurDir;
	lFileName += "\\";
	lFileName += mDirName;
	lFileName += "\\";
	lFileName += lSelString;

    static char BASED_CODE szFilesFilter[] = "All Files (*.*)|*.*||";

    HRESULT hResult;

    DWORD lDirReturn = 0;
    char  *lPath = new char[MAX_STRING];

    TK_LOG0("Retrieve the path of execution");
    lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);

    CFileDialog dlg(FALSE, NULL, lFileName, 0, szFilesFilter);
    m_nFilterLoad = 0;
    dlg.m_ofn.nFilterIndex = m_nFilterLoad;

    hResult = (int)dlg.DoModal();
    if(hResult != IDOK) 
    {
        SetCurrentDirectory(lPath);
		delete [] lPath;
        return;
    }

    SetCurrentDirectory(lPath);
	delete [] lPath;
	if (dlg.GetPathName() != "")
	{
		if (CopyFile(lFileName, dlg.GetPathName(), FALSE) == FALSE)
		{
            TK_LOG0("Retrieve file failed");
            CString fmt;
            fmt.Format("Retrieve file failed");
            MessageBox(fmt);
            return;
		}
	}
    
	FillListbox();
}

void SFilesDlg::OnBnClickedCancel()
{
    this->SavePosition();

	FillListbox();

	CListBox *theBox = (CListBox *)(this->GetDlgItem(IDC_FILELIST));

	if (theBox->GetCount() == 0)
	{
	    _rmdir(mDirName);
	}

	OnCancel();
}

void SFilesDlg::OnBnClickedDeleteFile()
{
    this->SavePosition();
	CListBox *theBox = (CListBox *)(this->GetDlgItem(IDC_FILELIST));

    int lCurSel = theBox->GetCurSel();
    if (lCurSel == LB_ERR)
    {
       AfxMessageBox("Select a file first");
       return;
    }

    CString lSelString;
    theBox->GetText(lCurSel, lSelString);

    TK_LOG0("File Delete Clicked");
    CString lMsg = "Do you really want to\ndelete the file?";
    if (MessageBox(lMsg, "File Delete", MB_YESNO) == IDYES)
    {
        TK_LOG0("File Delete Confirmed");
        CString lFileName = "";
		char theCurDir[MAX_STRING];
		_getcwd(theCurDir, MAX_STRING - 1);
		lFileName = theCurDir;
		lFileName += "\\";
		lFileName += mDirName;
		lFileName += "\\";
		lFileName += lSelString;

        TK_LOG1("File to delete: %s", lFileName.GetBuffer());

        int lRet = _unlink(lFileName.GetBuffer());
        TK_LOG1("Return code from _unlink = %d", lRet);

        if (lRet == -1)
        {
            TK_LOG0("Delete failed");
            CString fmt;
            fmt.Format("Delete file failed");
            MessageBox(fmt);
            return;
        }
        TK_LOG0("Delete succeeded");
    }
	FillListbox();
}

void SFilesDlg::AddFileToBox(CString theName)
{
	// build destination filespec, based from current dir
	char theCurDir[MAX_STRING];
	_getcwd(theCurDir, MAX_STRING - 1);
	CString theDestination = theCurDir;
	theDestination += "\\";
	theDestination += mDirName;
	theDestination += strrchr((LPCSTR)theName, '\\');

	// copy the file...
	if (CopyFile((LPCTSTR)theName, (LPCTSTR)theDestination, FALSE) == FALSE)
	{
		MessageBox("Could not store file, is the disk full?");
	}

	FillListbox();
}

void SFilesDlg::FillListbox()
{
	CListBox *theBox = (CListBox *)(this->GetDlgItem(IDC_FILELIST));
	theBox->ResetContent();

    // create the directory to store stuff in...
    SDate current = ivDiaryDlg->GetCurrentDate();
    sprintf_s(mDirName, 10, "%04d%02d%02d", current.getYear(), current.getMonth(), current.getDays());

	// build destination filespec, based from current dir
	char *theCurDir = new char[MAX_STRING];
	_getcwd(theCurDir, MAX_STRING - 1);
	CString theDestination = theCurDir;
	theDestination += "\\";
	theDestination += mDirName;
	theDestination += "\\*.*";
    delete [] theCurDir;

	CFileFind Finder;
    BOOL Ret = Finder.FindFile((LPCTSTR)theDestination);
    while (Ret)
    {
        Ret = Finder.FindNextFile();

        //if we have encountered a directory
        //and it's not a . or a ..
        if (!Finder.IsDots())
        {
            theBox->AddString(Finder.GetFileName());
        }
    }
}

int SFilesDlg::CountFiles()
{
	// build destination filespec, based from current dir
    // create the directory to store stuff in...
    SDate current = ivDiaryDlg->GetCurrentDate();
    sprintf_s(mDirName, 10, "%04d%02d%02d", current.getYear(), current.getMonth(), current.getDays());
	char *theCurDir = new char[MAX_STRING];
	_getcwd(theCurDir, MAX_STRING - 1);
	CString theDestination = theCurDir;
	theDestination += "\\";
	theDestination += mDirName;
	theDestination += "\\*.*";

	CFileFind Finder;
    BOOL Ret = Finder.FindFile((LPCTSTR)theDestination);
	int theCount = 0;
    while (Ret)
    {
        Ret = Finder.FindNextFile();

        //if we have encountered a directory
        //and it's not a . or a ..
        if (!Finder.IsDots())
        {
            theCount++;
        }
    }

	delete [] theCurDir;
	return theCount;
}


void SFilesDlg::OnBnClickedOpen()
{
    this->SavePosition();

	CListBox *theBox = (CListBox *)(this->GetDlgItem(IDC_FILELIST));

    int lCurSel = theBox->GetCurSel();
    if (lCurSel == LB_ERR)
    {
       AfxMessageBox("Select a file first");
       return;
    }

    CString lSelString;
    theBox->GetText(lCurSel, lSelString);

    CString lFileName = "";
	char theCurDir[MAX_STRING];
	_getcwd(theCurDir, MAX_STRING - 1);
	lFileName = "\"";
	lFileName += theCurDir;
	lFileName += "\\";
	lFileName += mDirName;
	lFileName += "\\";
	lFileName += lSelString;
	lFileName += "\"";

	system(lFileName);
	
	FillListbox();
}
