//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         ImageDlg.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  One of the more recent additions for Diary, the SImageDlg class displays the
//                image dialog, and handles all interactions with it. This includes loading and
//                deleting images, and switching the image slot visible.
//
//////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <conio.h>
#include <direct.h>
#include <io.h>

#include "stdafx.h"
#include "diarydlg.h"
#include "imagedlg.h"
#include "toolkit.h"
#include "diary.h"
#include "imagedlg.h"
#include "monitors.h"
#include "multimonitor.h"
#include "monitordc.h"
#include "ResizeDlg.h"
#include "BitmapEx.h"
#include "Todolist.h"

// SImageDlg dialog

IMPLEMENT_DYNAMIC(SImageDlg, CResizeDlg)
SImageDlg::SImageDlg(CWnd* pParent)
          :CResizeDlg(SImageDlg::IDD, pParent),
		   ivManipulator(0)
{
   ivSlotNumber = 0;

   ivDiaryDlg = (SDiaryDlg *)pParent;
}

SImageDlg::~SImageDlg()
{
}

void SImageDlg::CrummyInvalidate()
{
   static int tumbler = -1;
   tumbler = -tumbler;
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);
   lWinPlace.rcNormalPosition.right += tumbler;
   lWinPlace.rcNormalPosition.bottom += tumbler;
   this->SetWindowPlacement(&lWinPlace);
}

void SImageDlg::DoDataExchange(CDataExchange* pDX)
{
	CResizeDlg::DoDataExchange(pDX);
	DDX_Control(pDX, ID_IMGWEEKDAY, ivWeekday);
}

BEGIN_MESSAGE_MAP(SImageDlg, CResizeDlg)
   ON_WM_PAINT()
   ON_WM_WINDOWPOSCHANGED()
   ON_BN_CLICKED(ID_IMG_LOAD, OnBnClickedImgLoad)
   ON_BN_CLICKED(ID_IMG_DELETE, OnBnClickedImgDelete)
   ON_BN_CLICKED(IDC_SLOT0, OnBnClickedSlot0)
   ON_BN_CLICKED(IDC_SLOT1, OnBnClickedSlot1)
   ON_BN_CLICKED(IDC_SLOT2, OnBnClickedSlot2)
   ON_BN_CLICKED(IDC_SLOT3, OnBnClickedSlot3)
   ON_BN_CLICKED(IDC_SLOT4, OnBnClickedSlot4)
   ON_BN_CLICKED(IDC_SLOT5, OnBnClickedSlot5)
   ON_BN_CLICKED(IDC_SLOT6, OnBnClickedSlot6)
   ON_BN_CLICKED(IDC_SLOT7, OnBnClickedSlot7)
   ON_BN_CLICKED(IDC_SLOT8, OnBnClickedSlot8)
   ON_BN_CLICKED(IDC_SLOT9, OnBnClickedSlot9)
   ON_BN_CLICKED(ID_IMG_SLIDESHOW, OnBnClickedImgSlideshow)
   ON_BN_CLICKED(ID_IMG_EXPORT, &SImageDlg::OnBnClickedImgExport)
   ON_BN_CLICKED(IDC_IMGPREV, &SImageDlg::OnBnClickedImgprev)
   ON_BN_CLICKED(IDC_IMGNEXT, &SImageDlg::OnBnClickedImgnext)
END_MESSAGE_MAP()


// SImageDlg message handlers

int  SImageDlg::DoModal(int pSlot)
{
   ASSERT(pSlot >= 0);
   ASSERT(pSlot <= 9);

   this->ivSlotNumber = pSlot;



   return CResizeDlg::DoModal();
}

void SImageDlg::OnBnClickedImgLoad()
{
   CString strFilter;
   CSimpleArray<GUID> aguidFileTypes;
   HRESULT hResult;

   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];

   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);

   hResult = imgOriginal.GetExporterFilterString(strFilter,aguidFileTypes);
   if (FAILED(hResult))
   {
      CString fmt;
      fmt.Format("GetExporterFilter failed");
      MessageBox(fmt);
      SetCurrentDirectory(lPath);
	  delete [] lPath;
      return;
   }

   CFileDialog dlg(TRUE, NULL, NULL, OFN_FILEMUSTEXIST, strFilter);
   m_nFilterLoad = 2;
   dlg.m_ofn.nFilterIndex = m_nFilterLoad;

   hResult = (int)dlg.DoModal();
   if(hResult != IDOK) 
   {
      SetCurrentDirectory(lPath);
	  delete [] lPath;
      return;
   }

   m_nFilterLoad = dlg.m_ofn.nFilterIndex;
   TK_LOG0("This loads an external image, no need to decrypt");
   CString inPath = "";
   inPath = dlg.GetFolderPath();
   inPath += "\\";
   inPath += dlg.GetFileName();
   imgOriginal.Destroy();
   hResult = imgOriginal.Load(inPath);
   if (FAILED(hResult)) 
   {
      CString fmt;
      fmt.Format("Load image failed");
      MessageBox(fmt);
	  SetCurrentDirectory(lPath);
	  delete [] lPath;
      return;
   }

   SetCurrentDirectory(lPath);
   delete [] lPath;

   CString strFileName = this->GetCurrentImageFilename(this->ivSlotNumber);
   hResult = imgOriginal.Save(strFileName);
   if (FAILED(hResult)) 
   {
      CString fmt;
      fmt.Format("Save image failed");
      MessageBox(fmt);
      return;
   }

   TK_LOG0("Encrypt image after saving");
   theApp.EncryptFile(strFileName.GetBuffer());

   this->GetDlgItem(ID_IMG_DELETE)->EnableWindow(true);

   this->SetChecks();
   CrummyInvalidate();
}

// this paint routine does one thing on top of what CResizeDlg::OnPaint() does:
// it paints the image in the image dialog. Funny thing is that we are not 
// really painting it into the window that shows because of the resource, 
// but directly on the dialog, inside the border set by the image frame.

void SImageDlg::OnPaint()
{
   CPaintDC dc(this); // device context for painting
   CResizeDlg::OnPaint();
 
   TK_LOG0("Is there an image to paint?");
   TK_LOG0("Determine the window size first");
   WINDOWPLACEMENT lWinPlace;
   this->GetDlgItem(IDC_PICTURE)->GetWindowPlacement(&lWinPlace);

   int lTop    = lWinPlace.rcNormalPosition.top;
   int lLeft   = lWinPlace.rcNormalPosition.left;
   int lBottom = lWinPlace.rcNormalPosition.bottom;
   int lRight  = lWinPlace.rcNormalPosition.right;

#define H_OFFSET   (14)
#define V_OFFSET   (36)

   TK_LOG0("Yes, so calculate the scaling to make it fit");
   int lMaxWidth = lRight - lLeft - 5; // - 140;
   int lMaxHeight = lBottom - lTop - 5; // - 65;

   // report the pixelsize for easy reference
   CString lPixelSize;
   lPixelSize.Format("%dx%d", lMaxWidth, lMaxHeight);
   this->GetDlgItem(IDC_PIXELSIZE)->SetWindowText(lPixelSize);

   // set a clipping rect
   ivRect.SetRect(H_OFFSET, V_OFFSET, H_OFFSET + lMaxWidth, V_OFFSET + lMaxHeight);
   dc.SetBoundsRect(&ivRect, DCB_DISABLE);

   // clear out the window
   CBrush lBrush;
   lBrush.CreateSysColorBrush(COLOR_APPWORKSPACE);
   dc.FillRect(&ivRect, &lBrush);

   if (!imgOriginal.IsNull()) 
   {
      int lRealWidth = imgOriginal.GetWidth();
      int lRealHeight = imgOriginal.GetHeight();
      double lHorScalefactor = (double)lMaxWidth / (double)lRealWidth;
      double lVertScalefactor = (double)lMaxHeight / (double)lRealHeight;
      double lScalefactor = lHorScalefactor < lVertScalefactor 
                          ? lHorScalefactor : lVertScalefactor;
      int lScaledWidth = (int)(lRealWidth * lScalefactor);
      int lScaledHeight = (int)(lRealHeight * lScalefactor);
      int lHorOffset = H_OFFSET + (lMaxWidth - lScaledWidth) / 2;
      int lVertOffset = V_OFFSET + (lMaxHeight - lScaledHeight) / 2;

      TK_LOG0("and draw the image.");

      // Actually, the StretchBlt function is much too rough to correctly 
      // display the image, a true color pixel resize should be done here...

		  if (ivManipulator == 0)
		  {
			  imgOriginal.Save("temp.bmp");
			  ivManipulator = new CBitmapEx();
			  ivManipulator->Load("temp.bmp");
			  _unlink("temp.bmp");
			  ivManipulator->SetResampleMode(RM_BICUBIC);
    		  ivManipulator->Scale((long)(lScalefactor * 100), (long)(lScalefactor * 100));
		  }

		  ivManipulator->Draw(dc, lHorOffset, lVertOffset);

/*
		  imgOriginal.StretchBlt(dc,
								 lHorOffset,
								 lVertOffset,
								 lScaledWidth,
								 lScaledHeight,
								 SRCCOPY);
*/
   }
}

// OnInitDialog is used to restore the position settings from the settings 
// object, and use them to position the window. Also, the correct slot 
// is selected, based on the slot number passed in with the DoModal() call.

BOOL SImageDlg::OnInitDialog(void)
{
   BOOL lRetCode = CResizeDlg::OnInitDialog();

   AddControl(IDC_IMGNEXT, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_PICTURE, CST_NONE, CST_RESIZE, CST_NONE, CST_RESIZE, 1);
   AddControl(IDOK, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(ID_IMG_LOAD, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(ID_IMG_DELETE, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SLOT0, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SLOT1, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SLOT2, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SLOT3, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SLOT4, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SLOT5, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SLOT6, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SLOT7, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SLOT8, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SLOT9, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK0, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK1, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK2, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK3, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK4, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK5, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK6, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK7, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK8, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_CHECK9, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(ID_IMG_SLIDESHOW, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(ID_IMG_EXPORT, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_PIXELSIZE, CST_NONE, CST_REPOS, CST_NONE, CST_REPOS, 1);


   TK_LOG0("Read the position settings and restore them");
   SSetting *lSettingTop    = this->ivDiaryDlg->GetSettings()->GetSetting(SET_IMAGE_TOP);
   SSetting *lSettingLeft   = this->ivDiaryDlg->GetSettings()->GetSetting(SET_IMAGE_LEFT);
   SSetting *lSettingWidth  = this->ivDiaryDlg->GetSettings()->GetSetting(SET_IMAGE_WIDTH);
   SSetting *lSettingHeight = this->ivDiaryDlg->GetSettings()->GetSetting(SET_IMAGE_HEIGHT);

   if (lSettingTop    != NULL &&
       lSettingLeft   != NULL &&
	   lSettingWidth  != NULL &&
	   lSettingHeight != NULL)
   {
      TK_LOG0("Position settings found, positioning window");

      int lTop  = lSettingTop->GetValueAsNumber();
      int lLeft = lSettingLeft->GetValueAsNumber();
	  int lWidth = lSettingWidth->GetValueAsNumber();
	  int lHeight = lSettingHeight->GetValueAsNumber();

	  // make sure it will be on-screen, but keep the original position in config
	  this->SetWindowPos(0, lLeft, lTop, lWidth, lHeight, 0);
	  if (CMonitors::IsOnScreen( this ) == false)
	  {
		  CMonitor thePrimary = CMonitors::GetPrimaryMonitor();
		  thePrimary.CenterWindowToMonitor(this, true);
		  ivWindowCentered = true;
          WINDOWPLACEMENT lWinPlace;
          this->GetWindowPlacement(&lWinPlace);
          ivTop = lWinPlace.rcNormalPosition.top;
          ivLeft = lWinPlace.rcNormalPosition.left;
	  }
   }

   this->SelectSlot(this->ivSlotNumber);

   CString strFileName = this->GetCurrentImageFilename();

   if (!this->DoesImageFileExistForCurrentDate())
   {
      this->OnBnClickedImgLoad();

      if (!this->DoesImageFileExistForCurrentDate())
      {
         CResizeDlg::OnCancel();
      }
   }

   this->SetChecks();

   UpdateWindow();

   return lRetCode;
}

void SImageDlg::OnBnClickedImgDelete()
{
   TK_LOG0("Image Delete Clicked");
   CString lMsg = "Do you really want to\ndelete the image?";
   if (MessageBox(lMsg, "Image Delete", MB_YESNO) == IDYES)
   {
      TK_LOG0("Image Delete Confirmed");
      CString lFileName = this->GetCurrentImageFilename(this->ivSlotNumber);

      lFileName = lFileName.Left(lFileName.GetLength() - 1);
      lFileName += "x";

      TK_LOG1("File to delete: %s", lFileName.GetBuffer());

      imgOriginal.Destroy();
      Invalidate();
      UpdateWindow();

      int lRet = _unlink(lFileName.GetBuffer());
      TK_LOG1("Return code from _unlink = %d", lRet);

      if (lRet == -1)
      {
         TK_LOG0("Delete failed");
         CString fmt;
         fmt.Format("Delete image failed");
         MessageBox(fmt);
         return;
      }
      TK_LOG0("Delete succeeded");
   }
   this->SetChecks();
   CrummyInvalidate();
   if (!this->DoesImageFileExistForCurrentDate())
   {
      CResizeDlg::OnCancel();
   }
}

// SavePosition does nothing else than retrieve the window position, 
// and store it in the settings object. It is being called from all button and 
// radiobutton handlers, so position can be restored once the dialog is again
// opened...

void SImageDlg::SavePosition(int x, int y, int width, int height)
{
   TK_LOG0("Save window position to settings.dat");
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);

   // cancel save if window centered, and not moved yet...
   if (ivWindowCentered == true &&
	   (ivTop == lWinPlace.rcNormalPosition.top ||
	    ivLeft == lWinPlace.rcNormalPosition.left))
   {
	   return;
   }

   // ok, we save so set the flag to off
   ivWindowCentered = false;
   ivTop = -1;
   ivLeft = -1;

   char *lTop = new char[MAX_STRING];
   if (y == -1)
   {
	   y = lWinPlace.rcNormalPosition.top;
   }
   _itoa_s(y, lTop, MAX_STRING - 1, 10);
   SSetting *lTopSSetting = new SSetting(SET_IMAGE_TOP, lTop);
   this->ivDiaryDlg->GetSettings()->AddSetting(lTopSSetting);
   delete [] lTop;

   char *lLeft = new char[MAX_STRING];
   if (x == -1)
   {
	   x = lWinPlace.rcNormalPosition.left;
   }
   _itoa_s(x, lLeft, MAX_STRING - 1, 10);
   SSetting *lLeftSSetting = new SSetting(SET_IMAGE_LEFT, lLeft);
   this->ivDiaryDlg->GetSettings()->AddSetting(lLeftSSetting);
   delete [] lLeft;

   char *lWidth = new char[MAX_STRING];
   if (width == -1)
   {
       width = lWinPlace.rcNormalPosition.right - 
	           lWinPlace.rcNormalPosition.left;
   }
   _itoa_s(width, lWidth, MAX_STRING - 1, 10);
   SSetting *lWidthSSetting = new SSetting(SET_IMAGE_WIDTH, lWidth);
   this->ivDiaryDlg->GetSettings()->AddSetting(lWidthSSetting);
   delete [] lWidth;

   char *lHeight = new char[MAX_STRING];
   if (height == -1)
   {
       height = lWinPlace.rcNormalPosition.bottom - 
	            lWinPlace.rcNormalPosition.top;
   }
   _itoa_s(height, lHeight, MAX_STRING - 1, 10);
   SSetting *lHeightSSetting = new SSetting(SET_IMAGE_HEIGHT, lHeight);
   this->ivDiaryDlg->GetSettings()->AddSetting(lHeightSSetting);
   delete [] lHeight;

   TK_LOG0("Make sure the settings are flushed to disk");
   this->ivDiaryDlg->GetSettings()->WriteFile();
}

// This basically just cancels the dialog leaving the Status Quo

void SImageDlg::OnOK() 
{
   CResizeDlg::OnOK();
}

// this routine is called if one of the radiobutton handlers kicks in.
// it makes sure the correct image is displayed.

void SImageDlg::SelectSlot(int pSlotNumber)
{
   TK_LOG0("Perform any actions you need on the old slot");

   TK_LOG1("Set the slot number of the dialog to %d", pSlotNumber);
   this->ivSlotNumber = pSlotNumber;

   TK_LOG0("Update the status of the buttons, really only needed first time");
   switch (this->ivSlotNumber)
   {
   case 0:
      this->CheckRadioButton(IDC_SLOT0, IDC_SLOT9, IDC_SLOT0);
      break;
   case 1:
      this->CheckRadioButton(IDC_SLOT0, IDC_SLOT9, IDC_SLOT1);
      break;
   case 2:
      this->CheckRadioButton(IDC_SLOT0, IDC_SLOT9, IDC_SLOT2);
      break;
   case 3:
      this->CheckRadioButton(IDC_SLOT0, IDC_SLOT9, IDC_SLOT3);
      break;
   case 4:
      this->CheckRadioButton(IDC_SLOT0, IDC_SLOT9, IDC_SLOT4);
      break;
   case 5:
      this->CheckRadioButton(IDC_SLOT0, IDC_SLOT9, IDC_SLOT5);
      break;
   case 6:
      this->CheckRadioButton(IDC_SLOT0, IDC_SLOT9, IDC_SLOT6);
      break;
   case 7:
      this->CheckRadioButton(IDC_SLOT0, IDC_SLOT9, IDC_SLOT7);
      break;
   case 8:
      this->CheckRadioButton(IDC_SLOT0, IDC_SLOT9, IDC_SLOT8);
      break;
   case 9:
      this->CheckRadioButton(IDC_SLOT0, IDC_SLOT9, IDC_SLOT9);
      break;
   }

   TK_LOG0("Perform actions to update the dialog for the new slot");
   imgOriginal.Destroy();

   HRESULT hResult;
   if (this->DoesImageFileExistForCurrentDate(this->ivSlotNumber))
   {
      CString lFileName = this->GetCurrentImageFilename(this->ivSlotNumber);

      TK_LOG0("Decrypt internal image before loading");
      theApp.DecryptFile(lFileName.GetBuffer());

      hResult = imgOriginal.Load(lFileName);
      if (FAILED(hResult)) 
      {
         CString fmt;
         fmt.Format("Load image failed");
         MessageBox(fmt);
         TK_LOG0("Remove the unencrypted file again");
         _unlink(lFileName.GetBuffer());
         return;
      }

      TK_LOG0("Remove the unencrypted file again");
      _unlink(lFileName.GetBuffer());
     
      this->GetDlgItem(ID_IMG_DELETE)->EnableWindow(true);
   }
   else
   {
      this->GetDlgItem(ID_IMG_DELETE)->EnableWindow(false);
   }

   char  *lTitle = new char[MAX_STRING];

   sprintf_s(lTitle, MAX_STRING - 1, 
             "%s - Image", 
             this->GetCurrentImageFilename(this->ivSlotNumber, true));
   TK_LOG1("Set Window title to %s", lTitle);
   SetWindowText(lTitle);
   delete [] lTitle;

   delete ivManipulator;
   ivManipulator = 0;

   // UpdateWindow();   // just isn't enough...
   UpdateWeekday();
   CrummyInvalidate();
}

// the next ten routines are called if the corresponding radio button 
// is clicked. They all call SavePosition & SelectSlot for a different 
// slot number. 

void SImageDlg::OnBnClickedSlot0()
{
   CWaitCursor lWaiter;
   this->SelectSlot(0);
}

void SImageDlg::OnBnClickedSlot1()
{
   CWaitCursor lWaiter;
   this->SelectSlot(1);
}

void SImageDlg::OnBnClickedSlot2()
{
   CWaitCursor lWaiter;
   this->SelectSlot(2);
}

void SImageDlg::OnBnClickedSlot3()
{
   CWaitCursor lWaiter;
   this->SelectSlot(3);
}

void SImageDlg::OnBnClickedSlot4()
{
   CWaitCursor lWaiter;
   this->SelectSlot(4);
}

void SImageDlg::OnBnClickedSlot5()
{
   CWaitCursor lWaiter;
   this->SelectSlot(5);
}

void SImageDlg::OnBnClickedSlot6()
{
   CWaitCursor lWaiter;
   this->SelectSlot(6);
}

void SImageDlg::OnBnClickedSlot7()
{
   CWaitCursor lWaiter;
   this->SelectSlot(7);
}

void SImageDlg::OnBnClickedSlot8()
{
   CWaitCursor lWaiter;
   this->SelectSlot(8);
}

void SImageDlg::OnBnClickedSlot9()
{
   CWaitCursor lWaiter;
   this->SelectSlot(9);
}

// this routine is called to update the ten checkboxes according to the 
// existence of image files for the corresponding slots...

void SImageDlg::SetChecks(void)
{
   for (int lSlotNumber = 0; lSlotNumber < 10; lSlotNumber++)
   {
      UINT lCheck;

      TK_LOG0("Determine check value based on existence of file");
      if (this->DoesImageFileExistForCurrentDate(lSlotNumber))
      {
         lCheck = BST_CHECKED;
      }
      else
      {
         lCheck = BST_UNCHECKED;
      }

      TK_LOG0("Update corresponding checkbox");
      switch (lSlotNumber)
      {
      case 0:
         this->CheckDlgButton(IDC_CHECK0, lCheck);
         break;
      case 1:
         this->CheckDlgButton(IDC_CHECK1, lCheck);
         break;   
      case 2:
         this->CheckDlgButton(IDC_CHECK2, lCheck);
         break;   
      case 3:
         this->CheckDlgButton(IDC_CHECK3, lCheck);
         break;   
      case 4:
         this->CheckDlgButton(IDC_CHECK4, lCheck);
         break;   
      case 5:
         this->CheckDlgButton(IDC_CHECK5, lCheck);
         break;   
      case 6:
         this->CheckDlgButton(IDC_CHECK6, lCheck);
         break;   
      case 7:
         this->CheckDlgButton(IDC_CHECK7, lCheck);
         break;   
      case 8:
         this->CheckDlgButton(IDC_CHECK8, lCheck);
         break;   
      case 9:
         this->CheckDlgButton(IDC_CHECK9, lCheck);
         break;   
      default:
         MessageBox("Invalid Slot number in SImageDlg::SetCheck()");
      }
   }
}

void SImageDlg::OnBnClickedImgSlideshow()
{
	for (int i = 0; i < 10; i++)
	{
		if (this->DoesImageFileExistForCurrentDate(i))
		{
			this->SelectSlot(i);
            OnPaint();
		    Sleep(1500);
		}
	}
}

boolean SImageDlg::DoesImageFileExistForCurrentDate(int pSlotNumber)
{
   TK_TRACER("SImageDlg::DoesImageFileExistForCurrentDate(int pSlotNumber)");

   if (pSlotNumber == ANY_SLOT)
   {
      boolean lTotalResult = false;
      for (int i = 0; i < 10; i++)
      {
         TK_LOG0("Check all ten slots anyway, because of old image conversion");
         boolean lResult = this->DoesImageFileExistForCurrentDate(i);
         if (lResult) 
         {
            lTotalResult = true;
         }
      }
      return lTotalResult;
   }
   else
   {
      CString lFileName = this->GetCurrentImageFilename(pSlotNumber);

      FILE *lFile = fopen(lFileName.GetBuffer(), "r");
      if (lFile != NULL)
      {
         fclose(lFile); 
         return true;
      }
   
      TK_LOG0(".JPG image file not found. Check for .GIF file from version 4.1 or 4.2");
      CString lNewFileName = lFileName;
      int lPositionOfPoint = lFileName.ReverseFind('.');
      lFileName = lFileName.Left(lPositionOfPoint);
      lFileName = lFileName + ".gif";
      TK_LOG1("Checking for file %s", lNewFileName.GetBuffer());

      lFile = fopen(lFileName.GetBuffer(), "r");
      if (lFile != NULL)
      {
         fclose(lFile); 

         TK_LOG0("Old Image file found, converting.....");
         CImage lImg;
         lImg.Load(lFileName.GetBuffer());
         lImg.Save(lNewFileName.GetBuffer());

         TK_LOG0("Encrypt converted file afterwards");
         theApp.EncryptFile(lNewFileName.GetBuffer());

         TK_LOG1("Deleting old image file %s", lFileName.GetBuffer());
         _unlink(lFileName.GetBuffer());
         return true;
      }

      CString lEncryptedFileName = lFileName;
      lPositionOfPoint = lFileName.ReverseFind('.');
      lFileName = lFileName.Left(lPositionOfPoint);
      lFileName = lFileName + ".jpx";
      TK_LOG1("Checking for file %s", lEncryptedFileName.GetBuffer());

      lFile = fopen(lFileName.GetBuffer(), "r");
      if (lFile != NULL)
      {
         fclose(lFile);
         return true;
      }
   }

   return false;
}

CString SImageDlg::GetCurrentImageFilename(int pSlotNumber, bool pEncryptedForm)
{
   TK_TRACER("SImageDlg::GetCurrentImageFilename(int pSlotNumber, bool pEncryptedForm)");

   CString lFilename;
   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];

   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);

   char lExtension[5];
   if (pEncryptedForm)
   {
      strcpy_s(lExtension, 4, "jpx");
   }
   else
   {
      strcpy_s(lExtension, 4, "jpg");
   }
   if (pSlotNumber == 0)
   {
      lFilename.Format("%s\\%08d.%s", lPath, this->ivDiaryDlg->GetCurrentDate().asDecimal(), lExtension);
   }
   else
   {
      lFilename.Format("%s\\%08d%01d.%s", lPath, 
                       this->ivDiaryDlg->GetCurrentDate().asDecimal(), pSlotNumber, lExtension);
   }
   TK_LOG1("Image filename: %s", lFilename.GetBuffer());
   delete [] lPath;
   return lFilename;
}

int SImageDlg::CountImageFiles(void)
{
   TK_TRACER("SImageDlg::CountImageFiles(void)");
   int lCount = 0;
   for (int i = 0; i < 10; i++)
   {
      if (DoesImageFileExistForCurrentDate(i))
      {
         lCount++;
      }
   }
   return lCount;
}


void SImageDlg::OnBnClickedImgExport()
{
    TK_LOG0("Image Export Clicked");

	CString lInputFileName = this->GetCurrentImageFilename(this->ivSlotNumber);

	lInputFileName = lInputFileName.Left(lInputFileName.GetLength() - 1);
	lInputFileName += "x";

	TK_LOG1("File to export: %s", lInputFileName.GetBuffer());
   
   CString strFilter;
   CSimpleArray<GUID> aguidFileTypes;
   HRESULT hResult;

   hResult = imgOriginal.GetExporterFilterString(strFilter,aguidFileTypes);
   if (FAILED(hResult))
   {
      CString fmt;
      fmt.Format("GetExporterFilter failed");
      MessageBox(fmt);
      return;
   }

   CFileDialog dlg(TRUE, 
	               NULL, 
				   NULL, 
				   OFN_NOCHANGEDIR|OFN_HIDEREADONLY|OFN_NOREADONLYRETURN, 
				   strFilter);
   m_nFilterLoad = 2;
   dlg.m_ofn.nFilterIndex = m_nFilterLoad;

   hResult = (int)dlg.DoModal();
   if(hResult != IDOK) 
   {
      return;
   }

   m_nFilterLoad = dlg.m_ofn.nFilterIndex;

   // complete the filepath
   CString lNewPath = dlg.GetPathName();
   CString lExt = "." + dlg.GetFileExt();
   if (lNewPath.Find(lExt, 0) == -1)
   {
	   lNewPath += lExt;
   }

   TK_LOG0("Save it straight from the image");
   hResult = imgOriginal.Save(lNewPath);
}

void SImageDlg::OnWindowPosChanged(WINDOWPOS* lpwndpos)
{
	CResizeDlg::OnWindowPosChanged(lpwndpos);

    delete ivManipulator;
    ivManipulator = 0;

	SavePosition(lpwndpos->x, lpwndpos->y, lpwndpos->cx, lpwndpos->cy);
	UpdateWindow();
}

void SImageDlg::OnBnClickedImgprev()
{
    TK_TRACER("SDiaryDlg::OnBnClickedPrev(void)");
	// iterate backward until we find another diary page
	SDate theDate = this->ivDiaryDlg->GetCurrentDate();
	int daysTried = 0;
    do
	{
		theDate.Subtract(0, 0, 1);
		daysTried++;

	} while (DoImagesExistForDate(theDate) == false && daysTried < MAX_DAYS_TO_SCAN);

	if (DoImagesExistForDate(theDate) == false)
	{
		MessageBox("No previous image found in over five years", 
			       "Warning", 
				   MB_OK);
		return;
	}
    TK_LOG0("But save diary page and todolist before doing so");
    this->ivDiaryDlg->ivCurrentTodoList->Save();
    this->ivDiaryDlg->SaveOldPage(true);
	this->ivDiaryDlg->ivNewSelDate = theDate;
    SYSTEMTIME lTime;
	lTime.wYear  = theDate.getYear();
    lTime.wMonth = theDate.getMonth();
	lTime.wDay   = theDate.getDays();
	this->ivDiaryDlg->m_pMonthCalendar.SetCurSel(&lTime);
    this->ivDiaryDlg->LoadNewPage();
    this->ivDiaryDlg->UpdateWeekday();
    this->ivDiaryDlg->UpdateStatus();
	this->SetChecks();
    this->UpdateWeekday();
	this->SelectSlot(GetLowestFilledSlot(theDate));
}

void SImageDlg::OnBnClickedImgnext()
{
    TK_TRACER("SDiaryDlg::OnBnClickedNext(void)");
	// iterate forward until we find another diary page
	SDate theDate = this->ivDiaryDlg->GetCurrentDate();
	int daysTried = 0;
    do
	{
		theDate.Add(0, 0, 1);
		daysTried++;

	} while (DoImagesExistForDate(theDate) == false && daysTried < MAX_DAYS_TO_SCAN);

	if (DoImagesExistForDate(theDate) == false)
	{
		MessageBox("No next image found in over five years", 
			       "Warning", 
				   MB_OK);
		return;
	}
    TK_LOG0("But save diary page and todolist before doing so");
    this->ivDiaryDlg->ivCurrentTodoList->Save();
    this->ivDiaryDlg->SaveOldPage(true);
	this->ivDiaryDlg->ivNewSelDate = theDate;
    SYSTEMTIME lTime;
	lTime.wYear  = theDate.getYear();
    lTime.wMonth = theDate.getMonth();
	lTime.wDay   = theDate.getDays();
	this->ivDiaryDlg->m_pMonthCalendar.SetCurSel(&lTime);
    this->ivDiaryDlg->LoadNewPage();
    this->ivDiaryDlg->UpdateWeekday();
    this->ivDiaryDlg->UpdateStatus();
	this->SetChecks();
    this->UpdateWeekday();
	this->SelectSlot(GetLowestFilledSlot(theDate));
}

bool SImageDlg::DoImagesExistForDate(SDate &theDate)
{
	CString theFilename;
	CString theExtension = ".jpx";
	CString theFullSpec;
	theFilename.Format("%08d", theDate.asDecimal());

	for (int i = 0; i < 10; i++)
	{
		if (i == 0)
		{
			theFullSpec = theFilename + theExtension;
		}
		else
		{
			theFullSpec.Format("%s%d%s", theFilename, i, theExtension);
		}
		if (_access(theFullSpec, 6) != -1)
		{
			return true;
		}
	}

	return false;
}

int SImageDlg::GetLowestFilledSlot(SDate &theDate)
{
	CString theFilename;
	CString theExtension = ".jpx";
	CString theFullSpec;
	theFilename.Format("%08d", theDate.asDecimal());

	for (int i = 0; i < 10; i++)
	{
		if (i == 0)
		{
			theFullSpec = theFilename + theExtension;
		}
		else
		{
			theFullSpec.Format("%s%d%s", theFilename, i, theExtension);
		}
		if (_access(theFullSpec, 6) != -1)
		{
			return i;
		}
	}

	return -1;
}

void SImageDlg::UpdateWeekday(void)
{
   TK_TRACER("SImageDlg::UpdateWeekday()");
   CWaitCursor lWaiter;

   CString lMsg;
   lMsg.Format("%s, %s %d%s, %4d", 
               this->ivDiaryDlg->GetCurrentDate().dayAsLongString(),
               SDate::monthAsLongString((this->ivDiaryDlg->GetCurrentDate().getMonth())), 
               this->ivDiaryDlg->GetCurrentDate().getDays(), 
               SDate::getDateSuffix(this->ivDiaryDlg->GetCurrentDate().getDays()), 
               this->ivDiaryDlg->GetCurrentDate().getYear());
   ivWeekday.SetWindowText(lMsg.GetBuffer());
}
