//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         ImageDlg.h
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Declaration of Class ImageDlg.
//
//////////////////////////////////////////////////////////////////////////////////////////////
#ifndef IMAGEDLG_H_INCLUDED
#define IMAGEDLG_H_INCLUDED

#include <atlimage.h>
#include "diary.h"
#include "ResizeDlg.h"
#include "BitmapEx.h"
#include "afxwin.h"

// SImageDlg dialog

class SDiaryDlg;

//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Class SImageDlg
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  SImageDlg handles the interactions with the Images dialog. It allows for 
//                manipulation of up to ten images per date, fully resizeable of course. 
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//
//////////////////////////////////////////////////////////////////////////////////////////////
class SImageDlg : public CResizeDlg
{
   DECLARE_DYNAMIC(SImageDlg)

public:
   SImageDlg(CWnd* pParent = NULL);   // standard constructor
   virtual ~SImageDlg();

   virtual void OnPaint();
   virtual void OnOK(); 
   void SavePosition(int x, int y, int width, int height);
   virtual int  DoModal(int pSlot = 0);
   boolean DoesImageFileExistForCurrentDate(int pSlotNumber = ANY_SLOT);
   CString GetCurrentImageFilename(int pSlotNumber = 0, bool pEncryptedForm = false);
   int CountImageFiles(void);
   bool DoImagesExistForDate(SDate &theDate);
   int GetLowestFilledSlot(SDate &theDate);
   void CrummyInvalidate();
   void UpdateWeekday(void);

// Dialog Data
   enum { IDD = IDD_IMAGE_DIALOG };

protected:
   virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

   CImage imgOriginal;
   int   m_nFilterLoad;
   SDiaryDlg *ivDiaryDlg;
   int ivSlotNumber;

   DECLARE_MESSAGE_MAP()

private:
   // stuff to correctly reposition if offscreen
   bool ivWindowCentered;
   int  ivTop;
   int  ivLeft;
   CRect ivRect; // to get from OnSize() to OnPaint()
   CBitmapEx *ivManipulator;

public:
   afx_msg void OnWindowPosChanged(WINDOWPOS* lpwndpos);
   afx_msg void OnBnClickedImgLoad();
   virtual BOOL OnInitDialog(void);
   afx_msg void OnBnClickedImgDelete();
   virtual void SelectSlot(int pSlotNumber);
   afx_msg void OnBnClickedSlot0();
   afx_msg void OnBnClickedSlot1();
   afx_msg void OnBnClickedSlot2();
   afx_msg void OnBnClickedSlot3();
   afx_msg void OnBnClickedSlot4();
   afx_msg void OnBnClickedSlot5();
   afx_msg void OnBnClickedSlot6();
   afx_msg void OnBnClickedSlot7();
   afx_msg void OnBnClickedSlot8();
   afx_msg void OnBnClickedSlot9();
   virtual void SetChecks(void);
   afx_msg void OnBnClickedImgSlideshow();
   afx_msg void OnBnClickedImgExport();
   afx_msg void OnBnClickedImgprev();
   afx_msg void OnBnClickedImgnext();
   CStatic ivWeekday;
};

#endif
