//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         ImportDlg.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  This file implements the various methods of the SImportDlg object that is 
//                used to import possibly encrypted files into Diary.
//
//////////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "diary.h"
#include "diarydlg.h"
#include "ImportDlg.h"
#include "toolkit.h"
#include "settingsfile.h"
#include "setting.h"
#include "diarypage.h"
#include "monitors.h"
#include "multimonitor.h"
#include "monitordc.h"

IMPLEMENT_DYNAMIC(SImportDlg, CDialog)

// This constructor sets all attributes of the SImportDlg, except the parent 
// SDiaryDlg. This is set in the call on the DoModal.

SImportDlg::SImportDlg(CWnd* pParent)
           :CDialog(SImportDlg::IDD, pParent)
{
   this->ivDiaryDlg = (SDiaryDlg *)pParent;
   this->ivBefore = false;
   this->ivAfter  = false;
   this->ivMerge  = true;
   this->ivCryptKey = new char[MAX_STRING + 1];
   this->ivCryptKey[0] = '\0';
   this->ivBulkImport = false;
}

// This destructor does nothing for now....

SImportDlg::~SImportDlg()
{
	delete [] ivCryptKey;
}

// Method to transfer data from dialog to variables, and vice versa.

void SImportDlg::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_FILE_PREVIEW, m_Preview);
}

BEGIN_MESSAGE_MAP(SImportDlg, CDialog)
   ON_BN_CLICKED(ID_BROWSE_FILENAME, OnBnClickedBrowseFilename)
   ON_BN_CLICKED(IDC_RADIO2, OnBnClickedRadio2)
   ON_BN_CLICKED(IDC_RADIO3, OnBnClickedRadio3)
   ON_BN_CLICKED(IDC_RADIO4, OnBnClickedRadio4)
   ON_EN_KILLFOCUS(IDC_ENCRYPTION_KEY, OnEnKillfocusEncryptionKey)
   ON_EN_KILLFOCUS(IDC_IMPORT_FILENAME, OnEnKillfocusImportFilename)
   ON_BN_CLICKED(IDC_GO, OnBnClickedGo)
END_MESSAGE_MAP()


// This method is entered when the user clicks on the Browse Files button on the
// SImportDlg. It brings up a file selection box, where the user can select a single
// file. The full filespec of the chosen file is displayed in the filespec field.

void SImportDlg::OnBnClickedBrowseFilename()
{
   this->SavePosition();

   char lFilters[255] = "Text Files (*.txt)|*.txt|Diary Files (*.dry)|*.dry|All Files (*.*)|*.*||";
   HRESULT hResult;

   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];
   char  *lImportPath = new char[MAX_STRING];

   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);

   CFileDialog dlg(TRUE, NULL, NULL, OFN_FILEMUSTEXIST, lFilters);

   hResult = (int)dlg.DoModal();
   if (hResult != IDOK) 
   {
      SetCurrentDirectory(lPath);
	  delete [] lImportPath;
	  delete [] lPath;
      return;
   }

   TK_LOG0("Create the full file spec, and store it in the object and the field");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lImportPath);
   ivFullFileSpec.Format("%s\\%s", lImportPath, dlg.GetFileName());
   this->GetDlgItem(IDC_IMPORT_FILENAME)->SetWindowText(ivFullFileSpec.GetBuffer());
   this->OnEnKillfocusImportFilename();
   this->GetDlgItem(IDC_IMPORT_FILENAME)->SetFocus();

   TK_LOG0("Restore the current directory, after the file selection box closes");
   lDirReturn = SetCurrentDirectory(lPath);
   delete [] lImportPath;
   delete [] lPath;
}

// This method is entered if the user clicks Insert Before on the SImportDlg.

void SImportDlg::OnBnClickedRadio2()
{
   this->SavePosition();
   this->ivBefore = true;
   this->ivAfter  = false;
   this->ivMerge  = false;
}

// This method is entered if the user clicks Insert After on the SImportDlg.

void SImportDlg::OnBnClickedRadio3()
{
   this->SavePosition();
   this->ivBefore = false;
   this->ivAfter  = true;
   this->ivMerge  = false;
}

// This method is entered if the user clicks Timestamp Merge on the SImportDlg.

void SImportDlg::OnBnClickedRadio4()
{
   this->SavePosition();
   this->ivBefore = false;
   this->ivAfter  = false;
   this->ivMerge  = true;
}

// This method gets called when the encryption keyword field loses focus. It sets
// the current encryption, and decrypts the displayed file preview if necessary.

void SImportDlg::OnEnKillfocusEncryptionKey()
{
   this->SavePosition();
   TK_LOG0("Set the new encryption keyword");
   char *lCryptKey = new char[MAX_STRING];
   this->GetDlgItem(IDC_ENCRYPTION_KEY)->GetWindowText(lCryptKey, MAX_STRING);
   this->SetCryptKey(lCryptKey);  
   this->OnEnKillfocusImportFilename();
   delete [] lCryptKey;
}

// The user can manually edit the import filename, for instance to include wildcards.
// This field is called when the filespec field changes, and update the file preview 
// or selection list accordingly. Also, the caption on the action button is changed
// accordingly. It gets called if the filespec field loses the focus.

void SImportDlg::OnEnKillfocusImportFilename()
{
   this->SavePosition();
   TK_LOG0("User updated the Import filename, check for wildcards");
   this->GetDlgItem(IDC_IMPORT_FILENAME)->GetWindowText(ivFullFileSpec);
   if (ivFullFileSpec.Find('*') != -1 ||
       ivFullFileSpec.Find('?') != -1)
   {
      TK_LOG0("Wildcards found, enumerate files");
      this->SetFileListForMultipleFiles();
   }
   else
   {
      TK_LOG0("No wildcards, set preview with current password");
      this->SetPreviewForSingleFile();
   }
}

// This method is used to set the encryption keyword for the current import session.

void SImportDlg::SetCryptKey(char *pKey)
{
   TK_TRACER("SImportDlg::SetCryptKey(char *pKey)");
   strncpy_s(ivCryptKey, MAX_STRING, pKey, MAX_STRING);
   ivCryptKey[MAX_STRING] = '\0';
 
   // actually, this should put a NULL character right behind the ivCryptKey,
   // but instead it puts one at the end of the buffer. If the key is longer than
   // the buffer, this closes the key for good, but if the key is shorter, it does
   // nothing, so leave it like it is, because ANY change in encryption might 
   // break it for the user, and cause compatibility problems....
}

// This method has been overridden to add a little functionality before the dialog 
// is shown. It sets the window title, restores the window position, etc.

BOOL SImportDlg::OnInitDialog(void)
{
   BOOL lRetCode = CDialog::OnInitDialog();

   SetWindowText(this->ivDiaryDlg->getWindowTitle().GetBuffer());

   TK_LOG0("Read the position settings and restore them");
   SSetting *lSettingTop    = this->ivDiaryDlg
                                 ->GetSettings()->GetSetting(SET_IMPORT_TOP);
   SSetting *lSettingLeft   = this->ivDiaryDlg
                                 ->GetSettings()->GetSetting(SET_IMPORT_LEFT);

   if (lSettingTop  != NULL &&
       lSettingLeft != NULL)
   {
      TK_LOG0("Position settings found, positioning window");

      int lTop  = lSettingTop->GetValueAsNumber();
      int lLeft = lSettingLeft->GetValueAsNumber();
	  
	  // make sure it will be on-screen, but keep the original position in config
	  this->SetWindowPos(0, lLeft, lTop, 0, 0, SWP_NOSIZE);
	  if (CMonitors::IsOnScreen( this ) == false)
	  {
		  CMonitor thePrimary = CMonitors::GetPrimaryMonitor();
		  thePrimary.CenterWindowToMonitor(this, true);
		  ivWindowCentered = true;
          WINDOWPLACEMENT lWinPlace;
          this->GetWindowPlacement(&lWinPlace);
          ivTop = lWinPlace.rcNormalPosition.top;
          ivLeft = lWinPlace.rcNormalPosition.left;
	  }
   }

   UpdateRadioButtons();
   this->GetDlgItem(IDC_ENCRYPTION_KEY)->SetWindowText(ivCryptKey);
   this->GetDlgItem(IDC_IMPORT_FILENAME)->SetWindowText(ivFullFileSpec.GetBuffer());
   this->OnEnKillfocusImportFilename();

   return lRetCode;
}

// This gets called by all user control routines, to save the position of the window
// in the SettingsFile object, which is then sent to disk.

void SImportDlg::SavePosition(void)
{
   TK_LOG0("Save window position to settings.dat");
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);

   // cancel save if window centered, and not moved yet...
   if (ivWindowCentered == true &&
	   (ivTop == lWinPlace.rcNormalPosition.top ||
	    ivLeft == lWinPlace.rcNormalPosition.left))
   {
	   return;
   }

   // ok, we save so set the flag to off
   ivWindowCentered = false;
   ivTop = -1;
   ivLeft = -1;

   TK_LOG0("Get the Top position value");
   char *lTop = new char[MAX_STRING];
   _itoa_s(lWinPlace.rcNormalPosition.top, lTop, MAX_STRING - 1, 10);
   SSetting *lTopSSetting = new SSetting(SET_IMPORT_TOP, lTop);
   this->ivDiaryDlg->GetSettings()->AddSetting(lTopSSetting);
   delete [] lTop;
 
   TK_LOG0("Get the Left position value");
   char *lLeft = new char[MAX_STRING];
   _itoa_s(lWinPlace.rcNormalPosition.left, lLeft, MAX_STRING - 1, 10);
   SSetting *lLeftSSetting = new SSetting(SET_IMPORT_LEFT, lLeft);
   this->ivDiaryDlg->GetSettings()->AddSetting(lLeftSSetting);
   delete [] lLeft;

   TK_LOG0("Make sure the settings are flushed to disk");
   this->ivDiaryDlg->GetSettings()->WriteFile();
}

// If the dialog is cancelled, this method gets called.

void SImportDlg::OnCancel()
{
   this->SavePosition();
   CDialog::OnCancel();
}

// This routines is called in the OnInitDialog, and whenever a radio button changes.
// actually, only the first is really necessary, the rest is automatical.

void SImportDlg::UpdateRadioButtons(void)
{
   if (this->ivBefore)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO4, IDC_RADIO2);
   }
   else if (this->ivAfter)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO4, IDC_RADIO3);
   }
   else if (this->ivMerge)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO4, IDC_RADIO4);
   }
}

// This method is called if the full file spec does not contain any wildcards.
// It will attempt to open the requested file, and displays it decrypted by the
// currently filled in encryption keyword.

void SImportDlg::SetPreviewForSingleFile(void)
{
   TK_LOG0("update preview pane title");
   this->GetDlgItem(IDC_PREVIEW_TITLE)->SetWindowText("File Preview");
   TK_LOG0("retrieve file");
   char lBuffer[MAX_BUFFER];
   CString lFileSpec;
   this->GetDlgItem(IDC_IMPORT_FILENAME)->GetWindowText(lFileSpec);
   if (lFileSpec.GetLength() == 0)
   {
      return;
   }
   FILE *lFile = fopen(lFileSpec, "rb");
   if (lFile != NULL)
   {
      TK_LOG0("File is open, read it and decode");

      int lBytesRead = fread(lBuffer, 1, MAX_BUFFER, lFile);
      fclose(lFile);
      if (lBytesRead > 0)
      {
         lBuffer[lBytesRead] = '\0';
         
         TK_LOG0("decode with current password");
         theApp.Decode(lBuffer, lBytesRead, this->ivCryptKey);

         TK_LOG0("Do our own word wrap, because Microsoft doesn't any more...");
         int lIndex = 0;
         int lLength = strlen(lBuffer);
         char *lRunner = lBuffer;
         while (lRunner < lBuffer + lLength)
         {
            TK_LOG0("Try to count off 100 characters, stop at two newlines");
            int i = 0;
            for (i = 0; i < 100; i++, lRunner++)
            {
               if (*lRunner == '\n')
               {
                  TK_LOG0("newline found, break from loop");
                  break; 
               }
            }
            TK_LOG1("Counted off %d characters", i);
            if (lRunner >= lBuffer + lLength)
            {
               TK_LOG0("End of buffer encountered, breaking...");
               break;
            }
            TK_LOG0("Find first whitespace before current position. ");
            while (*lRunner != ' ' && *lRunner != '\n' && *lRunner != '\r')
            {
               lRunner--;
               if (lRunner <= lBuffer)
               {
                  TK_LOG0("Begin of buffer found, breaking...");
                  break;
               }
            }
            if (lRunner <= lBuffer)
            {
               TK_LOG0("Begin of buffer found, breaking...");
               break;
            }
            TK_LOG0("Turn space into a newline character");
            *lRunner = '\n';
            lRunner++;
         }

         TK_LOG0("put in preview pane");
         m_Preview.SetWindowText(lBuffer);
         m_Preview.SetSel(-1, -1);
      }
   }

   TK_LOG0("Set the caption on the Go button");
   this->GetDlgItem(IDC_GO)->SetWindowText("Import Previewed File on current date");
   this->ivBulkImport = false;
}

// This gets called if the full file spec is a wildcard specification, and shows 
// instead in the file preview a list of all files relevant to the file 
// specification.

void SImportDlg::SetFileListForMultipleFiles(void)
{
   TK_LOG0("update preview pane title");
   this->GetDlgItem(IDC_PREVIEW_TITLE)->SetWindowText("File Selection");

   TK_LOG0("enumerate files with a filefind");
   CFileFind finder;
   BOOL bWorking = finder.FindFile(ivFullFileSpec);
   char lBuffer[MAX_BUFFER];
   lBuffer[0] = '\0';
   while (bWorking)
   {   
      bWorking = finder.FindNextFile();

      TK_LOG1("Adding file %s", finder.GetFilePath());
      strcat_s(lBuffer, MAX_BUFFER - 1, finder.GetFilePath());
      strcat_s(lBuffer, MAX_BUFFER - 1, "\n");
   }

   TK_LOG0("put in preview pane");
   m_Preview.SetWindowText(lBuffer);
   m_Preview.SetSel(-1, -1);

   TK_LOG0("Set the caption on the Go button");
   this->GetDlgItem(IDC_GO)->SetWindowText("Import all files to corresponding dates");
   this->ivBulkImport = true;
}

// If this routine is entered, then the user has clicked the Go button. Depending 
// on the setting of ivBulkImport, it will execute one of two paths....
// if bulk import is requested, it will enumerate all relevant files, and import
// them to their corresponding dates as indicated by the filename part, according
// to the current encryption settings.
// If no bulk import is required, then the selected file is imported to the current 
// date.

void SImportDlg::OnBnClickedGo()
{
   CWaitCursor lWaiter;
   TK_LOG0("Import started...");
   if (ivBulkImport == true)
   {
      TK_LOG0("Bulk Import requested, enumerate files and process...");
      TK_LOG0("enumerate files with a filefind");
      CFileFind finder;
      BOOL bWorking = finder.FindFile(ivFullFileSpec);
      char lBuffer[MAX_BUFFER];
      lBuffer[0] = '\0';
      while (bWorking)
      {   
         bWorking = finder.FindNextFile();

         TK_LOG0("File found, determine file date from filespec if possible");
         SDate lDate;
         lDate.Set(finder.GetFileName().GetBuffer(), false);

         TK_LOG0("Otherwise, skip file");
         if (lDate.asDecimal() == 19000101)
         {
            CString lMsg;
            lMsg.Format("File %s skipped, cannot determine\nvalid date to import from filename", 
                        finder.GetFileName().GetBuffer());
            AfxMessageBox(lMsg);
            continue;
         }

         TK_LOG0("Create SDiaryPage for Diary page of found date");
         CString lFilename = this->ivDiaryDlg->getFilenameForDate(lDate);
         SDiaryPage *lPage = new SDiaryPage(lFilename.GetBuffer(),  
                                            lDate,
                                            theApp.GetCryptKey());

         TK_LOG0("Updating status line");
         CString lMsg;
         lMsg.Format("Importing %s...", finder.GetFileName().GetBuffer());
         this->GetDlgItem(IDC_STATUS)->SetWindowText(lMsg.GetBuffer());

         TK_LOG0("Create SDiaryPage for Import page found");
         SDiaryPage *lImport = new SDiaryPage(finder.GetFilePath().GetBuffer(),  
                                              lDate,
                                              this->ivCryptKey);

         TK_LOG0("Merge the two");
         TK_LOG0("both pages present...");
         if (this->ivBefore == true)
         {
            TK_LOG0("Insert import page before current page");
            lPage->InsertBefore(lImport);
         }
         else if (this->ivAfter == true)
         {
            TK_LOG0("Insert import page after current page");
            lPage->InsertAfter(lImport);
         }
         else if (this->ivMerge == true)
         {
            TK_LOG0("Merge import page into current page");
            lPage->Merge(lImport);
         }

         TK_LOG0("Write the page to disk again");
         lPage->Write();

         TK_LOG0("Delete the imported file if the user asked for it");
         if (this->IsDlgButtonChecked(IDC_DELETE_IMPORTED_FILES) == (BOOL)true)
         {
            int lRet = _unlink(finder.GetFilePath().GetBuffer());
            TK_LOG1("Return code from _unlink = %d", lRet);

            if (lRet == -1)
            {
               TK_LOG1("Import file\n%s\ncould not be deleted", 
                       finder.GetFilePath().GetBuffer());
               CString fmt;
               fmt.Format("Import file\n%s\ncould not be deleted", 
                          finder.GetFilePath().GetBuffer());
               MessageBox(fmt);
            }      
         }
      }
      this->GetDlgItem(IDC_STATUS)->SetWindowText("");
   }
   else
   {
      TK_LOG0("Single file import requested, import to current date...");

      TK_LOG0("Check filename for file date if possible");

      char *lFullSpec = new char[MAX_STRING + 1];
      strcpy_s(lFullSpec, MAX_STRING, this->ivFullFileSpec);
      char *lFilename = strrchr(lFullSpec, '\\') + 1;

      SDate lDate;
      lDate.Set(lFilename, false);

      TK_LOG0("If it has, check for equal date");
      if (lDate.asDecimal() != 19000101 &&
          lDate != this->ivDiaryDlg->GetCurrentDate())
      {
         CString lMsg;
         lMsg.Format("File appears to be for a different date\n"
                     "Are you sure you want to import\n%s\n"
                     "on %08d?", 
                    this->ivFullFileSpec.GetBuffer(),
                    this->ivDiaryDlg->GetCurrentDate().asDecimal());
         if (AfxMessageBox(lMsg, MB_YESNO) == IDNO)
         {
			 delete [] lFullSpec;
            return;
         }
      }

	  delete [] lFullSpec;
      TK_LOG0("create an SDiaryPage object for the current date");
      SDiaryPage *lPage = this->ivDiaryDlg->getSDiaryPageForCurrentDate();

      TK_LOG0("create an SDiaryPage object for the file to import");
      SDiaryPage *lImport = new SDiaryPage(this->ivFullFileSpec,  
                                           this->ivDiaryDlg->GetCurrentDate(),
                                           this->ivCryptKey);

      TK_LOG0("both pages present...");
      if (this->ivBefore == true)
      {
         TK_LOG0("Insert import page before current page");
         lPage->InsertBefore(lImport);
      }
      else if (this->ivAfter == true)
      {
         TK_LOG0("Insert import page after current page");
         lPage->InsertAfter(lImport);
      }
      else if (this->ivMerge == true)
      {
         TK_LOG0("Merge import page into current page");
         lPage->Merge(lImport);
      }

      TK_LOG0("Write the page to disk again");
      lPage->Write();
   }
}
