//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         RecodeDlg.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  the recode dialog implements the password change facility of Diary.
//                it presents a dialog where the user can fill in his or her new password
//                (twice), and re-encrypts all diary files accordingly...
//
//////////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "diary.h"
#include "RecodeDlg.h"
#include "toolkit.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

SRecodeDlg::SRecodeDlg(CWnd* pParent)
           :CDialog(SRecodeDlg::IDD, pParent)
{
   //{{AFX_DATA_INIT(SRecodeDlg)
   //}}AFX_DATA_INIT
}

void SRecodeDlg::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   //{{AFX_DATA_MAP(SRecodeDlg)
   DDX_Control(pDX, IDC_RECODE_STATUS, ivRecodeStatus);
   DDX_Control(pDX, IDC_NEWKEY_REPEAT, ivNewCryptKeyRepeat);
   DDX_Control(pDX, IDC_NEWKEY, ivNewCryptKey);
   //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(SRecodeDlg, CDialog)
   //{{AFX_MSG_MAP(SRecodeDlg)
   //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// SRecodeDlg message handlers

void SRecodeDlg::OnOK() 
{
   CWaitCursor lWaiter;

   TK_TRACER("SRecodeDlg::OnOK()");
   TK_LOG0("Retrieve old encryption key from the App object");
   CString lOldCryptKey = theApp.GetCryptKey();

   TK_LOG0("Make sure new encryption keywords match....");
   CString lNewCryptKey;
   CString lNewCryptKeyRepeat;
   ivNewCryptKey.GetWindowText(lNewCryptKey);
   ivNewCryptKeyRepeat.GetWindowText(lNewCryptKeyRepeat);
   if (lNewCryptKey != lNewCryptKeyRepeat)
   {
      AfxMessageBox("New keywords do not match");
      return;
   }

   TK_LOG0("Enumerate all .dry files in the working directory");
   CFileFind finder;
   BOOL bWorking = finder.FindFile("*.dry");
   while (bWorking)
   {   
      bWorking = finder.FindNextFile();
      
      TK_LOG0("and recode each file found");
      CString lMsg;
      lMsg.Format("Recoding %s", finder.GetFileName()); 
      ivRecodeStatus.SetWindowText(lMsg.GetBuffer());
      RecodeDataFile(finder.GetFileName(), lOldCryptKey, lNewCryptKey);
   }

   bWorking = finder.FindFile("*.jpx");
   while (bWorking)
   {   
      bWorking = finder.FindNextFile();

      CString lMsg;
      lMsg.Format("Recoding image file %s", finder.GetFileName().GetBuffer());
      ivRecodeStatus.SetWindowText(lMsg.GetBuffer());

      theApp.RecodeFile(finder.GetFilePath().GetBuffer(), lOldCryptKey.GetBuffer(), lNewCryptKey.GetBuffer());
   }
   CDialog::OnOK();
}

void SRecodeDlg::RecodeDataFile(CString pFileName, 
                                CString pOldCryptKey, 
                                CString pNewCryptKey)
{
   TK_TRACER("SRecodeDlg::RecodeDataFile(CString pFileName, CString pOldCryptKey, CString pNewCryptKey)");
   TK_PRMSTRING("pFileName", pFileName);
   TK_PRMSTRING("pOldCryptKey", pOldCryptKey);
   TK_PRMSTRING("pNewCryptKey", pNewCryptKey);

   TK_LOG0("set the App to the old keyword");
   theApp.SetCryptKey(pOldCryptKey.GetBuffer(0));

   TK_LOG0("read and decode the datafile");
   char lBuffer[MAX_BUFFER + 1];
   FILE *lFile = fopen(pFileName, "rb");
   if (lFile != NULL)
   {
      int lBytesRead = 0;
      lBytesRead = fread(lBuffer, 1, MAX_BUFFER, lFile);
      lBuffer[lBytesRead] = '\0';
      theApp.Decode(lBuffer, lBytesRead);
      fclose(lFile);

      TK_LOG0("set the App to the new keyword");
      theApp.SetCryptKey(pNewCryptKey.GetBuffer(0));

      TK_LOG0("encode and write the datafile");
      lFile = fopen(pFileName, "wb");
      theApp.Encode(lBuffer, lBytesRead);
      fwrite(lBuffer, 1, lBytesRead, lFile);
      fclose(lFile);
   }
}
