//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         SShortcut.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Implementation of Class SShortcut.
//
//////////////////////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "DiaryDlg.h"
#include "SShortcut.h"

SShortcut::SShortcut(SDiaryDlg *pDiaryDlg, 
					 SLabelDlg *pLabelDlg, 
					 int pSlotNumber)
          :ivDiaryDlg(pDiaryDlg),
		   ivShortcutDlg(pLabelDlg), 
		   ivSlotNumber(pSlotNumber),
		   ivLabelSetting(0),
	       ivTimeSetting(0),
	       ivInsertSetting(0)
{
}

SShortcut::~SShortcut()
{
}

void SShortcut::ReadFromSettings()
{
	// set the corresponding settings objects if not done yet...
	if (ivLabelSetting == 0)
	{
		CString theSetting;
		theSetting.Format("SET_LABEL_TEXT_%02d", ivSlotNumber);
		ivLabelSetting = this->ivDiaryDlg->GetSettings()->GetSetting(theSetting.GetBuffer(0));
		if (ivLabelSetting == 0)
		{
			ivLabelSetting = new SSetting(theSetting.GetBuffer(0), "");
		}
	}

	if (ivTimeSetting == 0)
	{
		CString theSetting;
		theSetting.Format("SET_LABEL_TIMESTAMP_%02d", ivSlotNumber);
		ivTimeSetting = this->ivDiaryDlg->GetSettings()->GetSetting(theSetting.GetBuffer(0));
		if (ivTimeSetting == 0)
		{
			ivTimeSetting = new SSetting(theSetting.GetBuffer(0), "false");
		}
	}
	
	if(ivInsertSetting == 0)
	{
		CString theSetting;
		theSetting.Format("SET_LABEL_WORDS_%02d", ivSlotNumber);
		ivInsertSetting = this->ivDiaryDlg->GetSettings()->GetSetting(theSetting.GetBuffer(0));
		if (ivInsertSetting == 0)
		{
			ivInsertSetting = new SSetting(theSetting.GetBuffer(0), "");
		}
	}
}

void SShortcut::WriteToSettings(bool WriteFile)
{
	ivDiaryDlg->GetSettings()->AddSetting(ivLabelSetting);
	ivDiaryDlg->GetSettings()->AddSetting(ivTimeSetting);
	ivDiaryDlg->GetSettings()->AddSetting(ivInsertSetting);

	if (WriteFile)
	{
        ivDiaryDlg->GetSettings()->WriteFile();
	}
}

void SShortcut::Revert()
{
	if (ivLabelSetting == 0)
	{
		ReadFromSettings();
	}

	ivLabelText = ivLabelSetting->GetValue();
	CString theBool = ivTimeSetting->GetValue();
	if (theBool == "true")
	{
		ivTimestamp = true;
	}
	else
	{
		ivTimestamp = false;
	}
	ivInsertText = ivInsertSetting->GetValue();
}

void SShortcut::Apply()
{
	if (ivLabelSetting == 0)
	{
		ReadFromSettings();
	}

	ivLabelSetting->SetValue(ivLabelText);
	CString theBool = ivTimestamp?"true":"false";
	ivTimeSetting->SetValue(theBool);
	ivInsertSetting->SetValue(ivInsertText);
}

void SShortcut::ReadFromDialog()
{
	CEdit *theLabel = (CEdit *)(ivShortcutDlg->GetDlgItem(ivLabelTextControl));
	theLabel->GetWindowText(ivLabelText);

	BOOL theCheckboxIsChecked = ivShortcutDlg->IsDlgButtonChecked(ivTimestampControl);
	if (theCheckboxIsChecked == TRUE)
	{
		ivTimestamp = true;
	}
	else
	{
		ivTimestamp = false;
	}

	CEdit *theWords = (CEdit *)(ivShortcutDlg->GetDlgItem(ivInsertTextControl));
	theWords->GetWindowText(ivInsertText);
}

void SShortcut::SetOnDialog()
{
	CEdit *theLabel = (CEdit *)(ivShortcutDlg->GetDlgItem(ivLabelTextControl));
	theLabel->SetWindowText(ivLabelText);

	ivShortcutDlg->CheckDlgButton(ivTimestampControl, ivTimestamp);
	
	CEdit *theWords = (CEdit *)(ivShortcutDlg->GetDlgItem(ivInsertTextControl));
	theWords->SetWindowText(ivInsertText);
}

void SShortcut::Couple(int pLabelTextControl, int pTimestampControl, int pInsertTextControl)
{
	ASSERT(pLabelTextControl != 0);
	ASSERT(pTimestampControl != 0);
	ASSERT(pInsertTextControl != 0);

	ivLabelTextControl = pLabelTextControl;
    ivTimestampControl = pTimestampControl;
	ivInsertTextControl = pInsertTextControl;

	CEdit *theLabel = (CEdit *)(ivShortcutDlg->GetDlgItem(ivLabelTextControl));
	theLabel->SetLimitText(12);
	CEdit *theText = (CEdit *)(ivShortcutDlg->GetDlgItem(ivInsertTextControl));
	theText->SetLimitText(MAX_STRING - 1);

	// since Couple is always called from OnInitDialog,
	// we can call ReadFromSettings and SetOnDialog right away...
	SettingsToDialog();
}

void SShortcut::SettingsToDialog()
{
	ReadFromSettings();
	Revert();
	SetOnDialog();
}

void SShortcut::DialogToSettings(bool WriteFile)
{
	ReadFromDialog();
	Apply();
	WriteToSettings(WriteFile);
}

void SShortcut::SetButton(int theID)
{
	CWnd *theButton = ivDiaryDlg->GetDlgItem(theID);
	if (ivLabelText.IsEmpty())
	{
		theButton->SetWindowText("...");
		theButton->EnableWindow(FALSE);
	}
	else
	{
		theButton->SetWindowText(ivLabelText);
		theButton->EnableWindow(TRUE);
	}
}

CString SShortcut::GetLabelText()
{
	return ivLabelText;
}

CString SShortcut::GetInsertText()
{
	CString theInsertText = ivInsertText;
	if (theInsertText.IsEmpty())
	{
		theInsertText = GetLabelText();
	}
	else
	{
	    theInsertText.Replace("|", "\n");
	}
	return theInsertText;
}

bool SShortcut::NeedsTimestamp()
{
	return ivTimestamp;
}

void SShortcut::Clear()
{
	ivLabelText = "";
	ivTimestamp = false;
	ivInsertText = "";
	SetOnDialog();
}