//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         SearchDlg.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Implementation of Class SearchDlg.
//
//////////////////////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "diary.h"
#include "SearchDlg.h"
#include "toolkit.h"
#include "searchdlg.h"
#include "setting.h"
#include "settingsfile.h"
#include "diary.h"
#include "diarydlg.h"
#include "searchdlg.h"
#include "monitors.h"
#include "multimonitor.h"
#include "monitordc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// SSearchDlg dialog


SSearchDlg::SSearchDlg(CWnd* pParent /*=NULL*/)
   : CResizeDlg(SSearchDlg::IDD, pParent)
{
   //{{AFX_DATA_INIT(SSearchDlg)
   //}}AFX_DATA_INIT
   ivSelDate.wYear          = 0; 
   ivSelDate.wMonth         = 0; 
   ivSelDate.wDayOfWeek     = 0; 
   ivSelDate.wDay           = 0; 
   ivSelDate.wHour          = 0; 
   ivSelDate.wMinute        = 0; 
   ivSelDate.wSecond        = 0; 
   ivSelDate.wMilliseconds  = 0; 
   ivAlways = true;
   ivYearOffset = 0;
   ivMonthOffset = 0;
   ivDaysOffset = 0;
   ivSelection = -1;

   ivDiaryDlg = (SDiaryDlg *)pParent;
}


void SSearchDlg::DoDataExchange(CDataExchange* pDX)
{
	CResizeDlg::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SSearchDlg)
	DDX_Control(pDX, IDC_STATUS, m_Status);
	DDX_Control(pDX, IDC_SEARCH_RESULTS, m_SearchResults);
	DDX_Control(pDX, IDC_SEARCH_STRING, m_SearchString);
	//}}AFX_DATA_MAP
	DDX_Control(pDX, IDC_FIND_STRING, m_FindString);
}


BEGIN_MESSAGE_MAP(SSearchDlg, CResizeDlg)
   //{{AFX_MSG_MAP(SSearchDlg)
   ON_WM_WINDOWPOSCHANGED()
   ON_BN_CLICKED(IDSELECT, OnSelect)
   //}}AFX_MSG_MAP
   ON_BN_CLICKED(ID_EXPORT, OnBnClickedExport)
   ON_LBN_SELCHANGE(IDC_SEARCH_RESULTS, OnLbnSelchangeSearchResults)
   ON_BN_CLICKED(IDC_RADIO2, OnBnClickedRadio2)
   ON_BN_CLICKED(IDC_RADIO3, OnBnClickedRadio3)
   ON_BN_CLICKED(IDC_RADIO4, OnBnClickedRadio4)
   ON_BN_CLICKED(IDC_RADIO5, OnBnClickedRadio5)
   ON_BN_CLICKED(IDC_RADIO6, OnBnClickedRadio6)
   ON_BN_CLICKED(IDC_RADIO7, OnBnClickedRadio7)
   ON_BN_CLICKED(IDC_RADIO8, OnBnClickedRadio8)
   ON_BN_CLICKED(IDC_RADIO9, OnBnClickedRadio9)
   ON_BN_CLICKED(IDC_RADIO11, OnBnClickedRadio11)
   ON_BN_CLICKED(IDC_RADIO10, OnBnClickedRadio10)
   ON_BN_CLICKED(IDC_RADIO12, OnBnClickedRadio12)
   ON_BN_CLICKED(IDC_RADIO13, OnBnClickedRadio13)
   ON_LBN_DBLCLK(IDC_SEARCH_RESULTS, OnLbnDblclkSearchResults)
   ON_BN_CLICKED(IDFIND, &SSearchDlg::OnBnClickedFind)
END_MESSAGE_MAP()

// This method sets the search parameter, and searches all .dry
// files in the current directory for occurences of that string. 
// decryption is used if an encryption keyword has been entered. 

void SSearchDlg::OnOK() 
{
   CWaitCursor lWaiter; 

   // determine the size of the context....
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);
   int lContextSize = (lWinPlace.rcNormalPosition.right - 
                       lWinPlace.rcNormalPosition.left ) / 13;

   SDate lCurrDate = this->ivDiaryDlg->GetCurrentDate();
   SDate lLimitDate = lCurrDate;
   if (ivAlways)
   {
      lLimitDate.Set(1900, 1, 1);
   }
   else
   {
      lLimitDate.Subtract(ivYearOffset, ivMonthOffset, ivDaysOffset);
   }

   TK_TRACER("SSearchDlg::OnOK()");
   m_SearchString.GetWindowText(ivSearchString);

   if (ivSearchString == "")
   {
      AfxMessageBox("Can't search for nothing...");
      return;
   }
   ivSearchString.MakeLower();

   m_SearchResults.ResetContent();

   CFileFind finder;
   BOOL bWorking = finder.FindFile("*.dry");
   while (bWorking)
   {   
      bWorking = finder.FindNextFile();

      SDate lScanDate;
      lScanDate.Set(finder.GetFileName().GetBuffer());
      
      if (ivAlways || finder.GetFileName() == "todolist.dry" || lScanDate > lLimitDate)
      {
         CString lMsg;
         lMsg.Format("Scanning %s", finder.GetFileName());
         m_Status.SetWindowText(lMsg);
         TK_LOG1("Opening file %s", finder.GetFileName().GetBuffer());
         FILE *lFile = fopen(finder.GetFileName(), "rb");
         if (lFile != NULL)
         {
            char lBuffer[MAX_BUFFER + 1];
            int lBytesRead = 0;
            lBytesRead = fread(lBuffer, 1, MAX_BUFFER, lFile);
            TK_LOG1("Opening file %s", finder.GetFileName().GetBuffer());
            fclose(lFile);
            lBuffer[lBytesRead] = '\0';
            theApp.Decode(lBuffer, lBytesRead);
            for (int i = 0; i < lBytesRead; i++)
            {
               if (lBuffer[i] == '\n' ||
                   lBuffer[i] == '\r')
               {
                  lBuffer[i] = ' ';
               }
               lBuffer[i] = tolower(lBuffer[i]);
            }
            TK_LOG0("Take the page buffer in all lowercase");
            char *lSearchWindow = lBuffer;
            while (strstr(lSearchWindow, ivSearchString) != NULL)
            {
               TK_LOG0("For each occurrence of the Searchstring");
               char *lTextString = new char[MAX_STRING + 1];
               TK_LOG0("Set a pointer at the start of the found string");
               char *lContext = strstr(lSearchWindow, ivSearchString);
               int lCharPos = lContext - lBuffer;
               TK_LOG0("Update the search from pointer to point after the found string");
               lSearchWindow = lContext + ivSearchString.GetLength();
               TK_LOG0("Try to go back some characters");
               lContext -= lContextSize;
               if (lContext < lBuffer)
               {
                  TK_LOG0("if underflow, set to start of page");
                  lContext = lBuffer;
               }
               TK_LOG0("If not at start of page, find next space");
               while ((lContext != lBuffer) && (lContext[0] != ' '))
               {
                  lContext++;
               }
               TK_LOG0("Copy the string to display it in the list");
               strncpy_s(lTextString, MAX_STRING, lContext, 2 * lContextSize + ivSearchString.GetLength());
               lTextString[2 * lContextSize + ivSearchString.GetLength()] = '\0';

#ifdef FIND_LAST_WORDBREAK_TO_END_STRING
               TK_LOG0("Turn the last space into a '\0', thus ending the string");
               char *lLastSpace = strrchr(lTextString, ' ');
               if (lLastSpace != NULL)
               {
                  lLastSpace[1] = '\0';
               }
#endif 

               CString lMsg;
               if (lContext == lBuffer)
               {
                  lMsg.Format("%s  %06d   %s", finder.GetFileName(), lCharPos, lTextString);
               }
               else
               {
                  lMsg.Format("%s  %06d   ...%s...", finder.GetFileName(), lCharPos, lTextString);
               }
               m_SearchResults.AddString(lMsg);
			   delete [] lTextString;
            }
         }
      }
   }

   m_Status.SetWindowText("");
}

void SSearchDlg::OnCancel() 
{
   CWaitCursor lWaiter; 
   
   TK_TRACER("SSearchDlg::OnCancel()");
   CResizeDlg::OnCancel();
}

// This method gets the selected search result from the listbox, 
// and sets the SDiaryDlg to that date. 

void SSearchDlg::OnSelect() 
{
   CWaitCursor lWaiter; 
   
   TK_TRACER("SSearchDlg::OnSelect()");

   // get the currently selected text
   int lCurSel = m_SearchResults.GetCurSel();
   if (lCurSel == LB_ERR)
   {
      AfxMessageBox("Select an occurrence first");
      return;
   }

   CString lSelString;
   m_SearchResults.GetText(lCurSel, lSelString);

   // extract the date from it
   char *lRawDate = new char[MAX_STRING + 1];
   strcpy_s(lRawDate, MAX_STRING, lSelString);
   int lYear;
   int lMonth;
   int lDay;
   sscanf_s(lRawDate, "%04d%02d%02d.dry  %06d", &lYear, &lMonth, &lDay, &ivCharPos);
   delete [] lRawDate;
   ivSelDate.wYear  = lYear;
   ivSelDate.wMonth = lMonth;
   ivSelDate.wDay   = lDay;

   CResizeDlg::OnOK();
}


BOOL SSearchDlg::OnInitDialog() 
{
   CWaitCursor lWaiter; 
   
   TK_TRACER("SSearchDlg::OnInitDialog()");
   CResizeDlg::OnInitDialog();

   AddControl(IDC_RADIO2, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_RADIO3, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_RADIO4, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_RADIO5, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_RADIO6, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_RADIO7, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_RADIO8, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_RADIO9, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_RADIO10, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_RADIO11, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_RADIO12, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_RADIO13, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);

   AddControl(IDC_GROUPBOX, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 0);
   AddControl(IDCANCEL, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(ID_EXPORT, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDSELECT, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDFIND, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDOK, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_FIND_STRING, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_SEARCH_STRING, CST_REPOS, CST_REPOS, CST_NONE, CST_NONE, 1);
   
   AddControl(IDC_SEARCH_RESULTS, CST_NONE, CST_RESIZE, CST_NONE, CST_RESIZE, 1);

   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];
   char  *lTitle = new char[MAX_STRING];

   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);
   sprintf_s(lTitle, MAX_STRING - 1, "%s - Search", lPath);
   TK_LOG1("Set Window title to %s", lTitle);
   SetWindowText(lTitle);

   TK_LOG0("Read the position settings and restore them");
   SSetting *lSettingTop    = this->ivDiaryDlg->GetSettings()->GetSetting(SET_SEARCH_TOP);
   SSetting *lSettingLeft   = this->ivDiaryDlg->GetSettings()->GetSetting(SET_SEARCH_LEFT);
   SSetting *lSettingWidth  = this->ivDiaryDlg->GetSettings()->GetSetting(SET_SEARCH_WIDTH);
   SSetting *lSettingHeight = this->ivDiaryDlg->GetSettings()->GetSetting(SET_SEARCH_HEIGHT);

   if (lSettingTop    != NULL &&
       lSettingLeft   != NULL &&
	   lSettingWidth  != NULL &&
	   lSettingHeight != NULL)
   {
      TK_LOG0("Position settings found, positioning window");

      int lTop = lSettingTop->GetValueAsNumber();
      int lLeft = lSettingLeft->GetValueAsNumber();
	  int lWidth = lSettingWidth->GetValueAsNumber();
	  int lHeight = lSettingHeight->GetValueAsNumber();

	  // make sure it will be on-screen, but keep the original position in config
	  this->SetWindowPos(0, lLeft, lTop, lWidth, lHeight, 0);
	  if (CMonitors::IsOnScreen( this ) == false)
	  {
		  CMonitor thePrimary = CMonitors::GetPrimaryMonitor();
		  thePrimary.CenterWindowToMonitor(this, true);
		  ivWindowCentered = true;
          WINDOWPLACEMENT lWinPlace;
          this->GetWindowPlacement(&lWinPlace);
          ivTop = lWinPlace.rcNormalPosition.top;
          ivLeft = lWinPlace.rcNormalPosition.left;
	  }
   }

   if (ivSearchString != "")
   {
      m_SearchString.SetWindowText(ivSearchString);
      OnOK();
   }

   if (ivSelection != -1)
   {
	   m_SearchResults.SetCurSel(ivSelection);
   }

   UpdateLimitButtons();

   delete [] lPath;
   delete [] lTitle;
      
   return TRUE;  // return TRUE unless you set the focus to a control
                 // EXCEPTION: OCX Property Pages should return FALSE
}


void SSearchDlg::OnBnClickedExport()
{
   CWaitCursor lWaiter; 
   
   FILE *lFile = fopen("search.txt", "w");
   if (lFile != NULL)
   {
      char *lSelString = new char[MAX_STRING]; 
      for (int i=0;i < m_SearchResults.GetCount();i++)
      {         
       if (m_SearchResults.GetText(i, lSelString) > 0)
         {
            fprintf(lFile, "%s\n", lSelString);
         }
      }
     fclose(lFile);
	 delete [] lSelString;
   }
   else
   {
      MessageBox("Export file could not be created");
   }
}  // CBitMap

void SSearchDlg::SavePosition(int x, int y, int width, int height)
{
   TK_LOG0("Save window position to settings.dat");
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);

   // cancel save if window centered, and not moved yet...
   if (ivWindowCentered == true &&
	   (ivTop == lWinPlace.rcNormalPosition.top ||
	    ivLeft == lWinPlace.rcNormalPosition.left))
   {
	   return;
   }

   // ok, we save so set the flag to off
   ivWindowCentered = false;
   ivTop = -1;
   ivLeft = -1;

   char *lTop = new char[MAX_STRING];
   if (y == -1)
   {
	   y = lWinPlace.rcNormalPosition.top;
   }
   _itoa_s(y, lTop, MAX_STRING - 1, 10);
   SSetting *lTopSSetting = new SSetting(SET_SEARCH_TOP, lTop);
   this->ivDiaryDlg->GetSettings()->AddSetting(lTopSSetting);
   delete [] lTop;

   char *lLeft = new char[MAX_STRING];
   if (x == -1)
   {
	   x = lWinPlace.rcNormalPosition.left;
   }
   _itoa_s(x, lLeft, MAX_STRING - 1, 10);
   SSetting *lLeftSSetting = new SSetting(SET_SEARCH_LEFT, lLeft);
   this->ivDiaryDlg->GetSettings()->AddSetting(lLeftSSetting);
   delete [] lLeft;

   char *lWidth = new char[MAX_STRING];
   if (width == -1)
   {
       width = lWinPlace.rcNormalPosition.right - 
	           lWinPlace.rcNormalPosition.left;
   }
   _itoa_s(width, lWidth, MAX_STRING - 1, 10);
   SSetting *lWidthSSetting = new SSetting(SET_SEARCH_WIDTH, lWidth);
   this->ivDiaryDlg->GetSettings()->AddSetting(lWidthSSetting);
   delete [] lWidth;

   char *lHeight = new char[MAX_STRING];
   if (height == -1)
   {
       height = lWinPlace.rcNormalPosition.bottom - 
	            lWinPlace.rcNormalPosition.top;
   }
   _itoa_s(height, lHeight, MAX_STRING - 1, 10);
   SSetting *lHeightSSetting = new SSetting(SET_SEARCH_HEIGHT, lHeight);
   this->ivDiaryDlg->GetSettings()->AddSetting(lHeightSSetting);
   delete [] lHeight;

   this->ivDiaryDlg->GetSettings()->WriteFile();
}

void SSearchDlg::OnLbnSelchangeSearchResults()
{
   CWaitCursor lWaiter; 
   int lCurSel = m_SearchResults.GetCurSel();
   if (lCurSel != LB_ERR)
   {
	   ivSelection = lCurSel;
   }
   this->GetDlgItem(IDSELECT)->SetFocus();
}

void SSearchDlg::OnBnClickedRadio2()
{
   CWaitCursor lWaiter; 
   
   TK_LOG0("Anytime button clicked...");
   ivAlways = true;
   ivYearOffset = 0;
   ivMonthOffset = 0;
   ivDaysOffset = 0;
}

void SSearchDlg::OnBnClickedRadio3()
{
   CWaitCursor lWaiter; 
   
   TK_LOG0("1 Year button clicked...");
   ivAlways = false;
   ivYearOffset = 1;
   ivMonthOffset = 0;
   ivDaysOffset = 0;
}

void SSearchDlg::OnBnClickedRadio4()
{
   CWaitCursor lWaiter; 
   
   TK_LOG0("6 Months button clicked...");
   ivAlways = false;
   ivYearOffset = 0;
   ivMonthOffset = 6;
   ivDaysOffset = 0;
}

void SSearchDlg::OnBnClickedRadio5()
{
   CWaitCursor lWaiter; 
   
   TK_LOG0("3 Months button clicked...");
   ivAlways = false;
   ivYearOffset = 0;
   ivMonthOffset = 3;
   ivDaysOffset = 0;
}

void SSearchDlg::OnBnClickedRadio6()
{
   CWaitCursor lWaiter; 
   
   TK_LOG0("2 Months button clicked...");
   ivAlways = false;
   ivYearOffset = 0;
   ivMonthOffset = 2;
   ivDaysOffset = 0;
}

void SSearchDlg::OnBnClickedRadio7()
{
   CWaitCursor lWaiter; 
   
   TK_LOG0("1 Month button clicked...");
   ivAlways = false;
   ivYearOffset = 0;
   ivMonthOffset = 1;
   ivDaysOffset = 0;
}

void SSearchDlg::OnBnClickedRadio8()
{
   CWaitCursor lWaiter; 
   
   TK_LOG0("3 Weeks button clicked...");
   ivAlways = false;
   ivYearOffset = 0;
   ivMonthOffset = 0;
   ivDaysOffset = 21;
}

void SSearchDlg::OnBnClickedRadio9()
{
   CWaitCursor lWaiter; 
   
   TK_LOG0("2 Weeks button clicked...");
   ivAlways = false;
   ivYearOffset = 0;
   ivMonthOffset = 0;
   ivDaysOffset = 14;
}

void SSearchDlg::OnBnClickedRadio10()
{
   CWaitCursor lWaiter; 
   
   TK_LOG0("1 Week button clicked...");
   ivAlways = false;
   ivYearOffset = 0;
   ivMonthOffset = 0;
   ivDaysOffset = 7;
}

void SSearchDlg::OnBnClickedRadio11()
{
   CWaitCursor lWaiter; 
   
   TK_LOG0("4 Days button clicked...");
   ivAlways = false;
   ivYearOffset = 0;
   ivMonthOffset = 0;
   ivDaysOffset = 4;
}


void SSearchDlg::OnBnClickedRadio12()
{
   CWaitCursor lWaiter; 
   
   TK_LOG0("2 Days button clicked...");
   ivAlways = false;
   ivYearOffset = 0;
   ivMonthOffset = 0;
   ivDaysOffset = 2;
}

void SSearchDlg::OnBnClickedRadio13()
{
   CWaitCursor lWaiter; 
   
   TK_LOG0("1 Day button clicked...");
   ivAlways = false;
   ivYearOffset = 0;
   ivMonthOffset = 0;
   ivDaysOffset = 1;
}

void SSearchDlg::UpdateLimitButtons(void)
{
   if (ivAlways)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO13, IDC_RADIO2);
   }
   else if (ivYearOffset == 1)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO13, IDC_RADIO3);
   }
   else if (ivMonthOffset == 6)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO13, IDC_RADIO4);
   }
   else if (ivMonthOffset == 3)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO13, IDC_RADIO5);
   }
   else if (ivMonthOffset == 2)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO13, IDC_RADIO6);
   }
   else if (ivMonthOffset == 1)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO13, IDC_RADIO7);
   }
   else if (ivDaysOffset == 21)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO13, IDC_RADIO8);
   }   
   else if (ivDaysOffset == 14)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO13, IDC_RADIO9);
   }
   else if (ivDaysOffset == 7)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO13, IDC_RADIO10);
   }
   else if (ivDaysOffset == 4)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO13, IDC_RADIO11);
   }
   else if (ivDaysOffset == 2)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO13, IDC_RADIO12);
   }
   else if (ivDaysOffset == 1)
   {
      this->CheckRadioButton(IDC_RADIO2, IDC_RADIO13, IDC_RADIO13);
   }
}


void SSearchDlg::OnLbnDblclkSearchResults()
{
	OnSelect();
}

void SSearchDlg::OnBnClickedFind()
{
   CWaitCursor lWaiter; 
   
   TK_TRACER("SSearchDlg::OnBnClickedFind()");

   // get the currently selected text
   int lCurSel = m_SearchResults.GetCurSel();
   if (lCurSel == LB_ERR)
   {
      lCurSel = -1;
   }

   // find the next hit...
   char *lSelString = new char[MAX_STRING]; 
   bool lFound = false;
   for (int i = ++lCurSel; i < m_SearchResults.GetCount(); i++)
   {      
	   if (m_SearchResults.GetText(i, lSelString) > 0)
       {
           //check if find string is in there....
		   CString lFindString = "";
           m_FindString.GetWindowText(lFindString);
           CString theLine = lSelString;
		   if (theLine.Find(lFindString) != -1)
		   {
	           m_SearchResults.SetCurSel(i);
			   lFound = true;
			   break;
		   }
       }
   }
   if (lFound == false)
   {
	   MessageBox("No further hits...", "Info", MB_ICONINFORMATION|MB_OK);
   }
   delete [] lSelString;
}

void SSearchDlg::OnWindowPosChanged(WINDOWPOS* lpwndpos)
{
	CResizeDlg::OnWindowPosChanged(lpwndpos);

	SavePosition(lpwndpos->x, lpwndpos->y, lpwndpos->cx, lpwndpos->cy);
}
