//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         Todolist.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Implementation of Class Todolist.
//
//////////////////////////////////////////////////////////////////////////////////////////////

#include "TodoList.h"
#include "toolkit.h"

STodoList::STodoList(int pSlot, 
					 SDiaryDlg *theDiaryDlg, 
					 SLabelDlg *pLabelDlg )
{
	ivName = "";
	ivFullText = "";
	ivNameControl = 0;
	ivDiaryDlg = theDiaryDlg;
	ivSlotNumber = pSlot;
	ivNameSetting = 0;
	ivLabelDlg = pLabelDlg;
}

STodoList::~STodoList()
{
}

void STodoList::SetName(CString &pName)
{
	ivName = pName;
	if (ivName == "")
	{
		ivName = "...";
	}
	if (ivNameControl != 0)
	{
		this->ivDiaryDlg->GetDlgItem(ivNameControl)->SetWindowTextA(ivName.c_str());
	}
}

void STodoList::Couple(int pLabelTextControl)
{
	ASSERT(pLabelTextControl != 0);

	ivNameControl = pLabelTextControl;

	CEdit *theLabel = (CEdit *)(this->ivDiaryDlg->GetDlgItem(ivNameControl));
	theLabel->SetLimitText(12);

	// since Couple is always called from OnInitDialog,
	// we can call ReadFromSettings and SetOnDialog right away...
	//SettingsToDialog();

}

void STodoList::ReadFromSettings()
{
	// set the corresponding settings objects if not done yet...
	if (ivNameSetting == 0)
	{
		CString theSetting;
		theSetting.Format("SET_TODOLIST_NAME_%02d", ivSlotNumber);
		ivNameSetting = this->ivDiaryDlg->GetSettings()->GetSetting(theSetting.GetBuffer(0));
		if (ivNameSetting == 0)
		{
			ivNameSetting = new SSetting(theSetting.GetBuffer(0), "");
		}
	}
}

void STodoList::WriteToSettings(bool WriteFile)
{
	ivDiaryDlg->GetSettings()->AddSetting(ivNameSetting);

	if (WriteFile)
	{
        ivDiaryDlg->GetSettings()->WriteFile();
	}
}

void STodoList::Revert()
{
	if (ivNameSetting == 0)
	{
		ReadFromSettings();
	}

	ivName = ivNameSetting->GetValue();
}

void STodoList::Apply()
{
	if (ivNameSetting == 0)
	{
		ReadFromSettings();
	}

	ivNameSetting->SetValue(ivName.c_str());
}

void STodoList::ReadFromDialog()
{
	CEdit *theLabel = (CEdit *)(this->ivLabelDlg->GetDlgItem(ivNameControl));
	CString theText;
	theLabel->GetWindowText(theText);
	ivName = theText;
}

void STodoList::SetOnDialog()
{
	CEdit *theLabel = (CEdit *)(this->ivLabelDlg->GetDlgItem(ivNameControl));
	theLabel->SetWindowText(ivName.c_str());
}

void STodoList::SettingsToDialog()
{
	ReadFromSettings();
	Revert();
	SetOnDialog();
}

void STodoList::DialogToSettings(bool WriteFile)
{
	ReadFromDialog();
	Apply();
	WriteToSettings(WriteFile);
}

std::string STodoList::GetName(bool ellipsis)
{
	if (ivName == "")
	{
		if (ellipsis)
		{
			return "...";
		}
		else
		{
		    return "todolist";
		}
	}
	return ivName;
}

bool STodoList::Save()
{
   TK_TRACER("SDiaryDlg::SaveTodoList(void)");
   CWaitCursor lWaiter;

   if (ivDiaryDlg->isListDirty())
   {
      TK_LOG0("something to save, so lets do it");
      CString lFileName;
      lFileName.Format(GetName(false).c_str());
	  lFileName += ".dry";

      TK_LOG0("Open the todo file");
      FILE *lFile = fopen(lFileName.GetBuffer(0), "wb");

      if (lFile == NULL)
      {
         CString lMsg;
         lMsg.Format("File %s could not be opened for writing, \ncontinue with intended action?", 
                     lFileName.GetBuffer());
         if (AfxMessageBox(lMsg, MB_YESNO) == IDNO)
		 {
			 return false;
		 }
		 else
		 {
			 return true;
		 }
      }

      TK_LOG0("file now open, proceed");
      long lMin = 0;
      long lMax = 0;
      char lBuffer[MAX_BUFFER + 1];
      ivDiaryDlg->m_TodoList.SetSel(0, -1);
      ivDiaryDlg->m_TodoList.GetSel(lMin, lMax);
      CString theStr = ivDiaryDlg->m_TodoList.GetSelText();
	  strcpy_s(lBuffer, MAX_BUFFER, theStr);
      ivDiaryDlg->m_TodoList.SetSel(-1, -1);
      theApp.Encode(lBuffer, lMax - lMin);
      fwrite(lBuffer, 1, lMax - lMin, lFile);

      fclose(lFile);
   }
   return true; 
}

void STodoList::Load()
{
   TK_TRACER("STodoList::Load(void)");
   CWaitCursor lWaiter;

   ivDiaryDlg->m_TodoList.SetWindowText("");

   CString lFileName;
   lFileName.Format(GetName(false).c_str());
   lFileName += ".dry";

   TK_LOG0("Open the todo file for reading");
   FILE *lFile = fopen(lFileName, "rb");

   if (lFile != NULL)
   {
      TK_LOG0("File open, read and decrypt if needed"); 
      char lBuffer[MAX_BUFFER + 1];

      int lBytesRead = fread(lBuffer, 1, MAX_BUFFER, lFile);
      if (lBytesRead > 0)
      {
         lBuffer[lBytesRead] = '\0';
         theApp.Decode(lBuffer, lBytesRead);
         if (theApp.IsProperlyDecrypted(lBuffer, lBytesRead))
         {
            ivDiaryDlg->m_TodoList.SetWindowText(lBuffer);
            ivDiaryDlg->HighlightSearchStrings(&(ivDiaryDlg->m_TodoList));
            ivDiaryDlg->m_TodoList.SetSel(-1, -1);
            ivDiaryDlg->m_TodoList.EnableWindow(true);
            ivDiaryDlg->clearListDirty();
         }
         else
         {
            ivDiaryDlg->m_TodoList.SetWindowText("");
            ivDiaryDlg->m_TodoList.EnableWindow(false);
         }
      }
      else
      {
         ivDiaryDlg->m_TodoList.SetWindowText("");
         ivDiaryDlg->m_TodoList.EnableWindow(true);
      }
   }
   else
   {
      ivDiaryDlg->m_TodoList.SetWindowText("");
      ivDiaryDlg->m_TodoList.EnableWindow(true);
   }
}