//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         diary.h
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  this header file holds the application-wide version number, and the prototype
//                of the routine that returns it. Any file wanting to use this version number
//                should include this header file. The same is true for any application wanting
//                to instantiate a SDiaryApp object. Currently, I know of only one application
//                that does, but the file also holds the labels for the settings object, which
//                might be better off in settingsfile.h...
//
//////////////////////////////////////////////////////////////////////////////////////////////

#ifndef DIARY_H_INCLUDED
#define DIARY_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
   #error include 'stdafx.h' before including this file for PCH
#endif
#include "common.h"
#include "resource.h"      // main symbols

// this should be the only place where the version number is stored in the
// whole application. This can be used throughout the app by calling the 
// global function GetVersionNumber()

#define VERSION_NUMBER       "5.01"

char *GetVersionNumber(void);

// if an image slot identification means 'any slot', this is the number...
#define ANY_SLOT          (-1)

// This defines the application object for the Diary application.
// see DiaryApp.cpp for the implementation of the methods.

//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Class SDiaryApp
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  the Application class for the Diary application. It provides certain 
//                services for the various dialogs in this project. 
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//
//////////////////////////////////////////////////////////////////////////////////////////////
class SDiaryApp : public CWinApp
{
private:
   char *ivCryptKey;
   CCommandLineInfo ivCmdLineInfo;

public:
   SDiaryApp();
   virtual ~SDiaryApp(void);

   void Encode(char *pBuffer, int pLength, char *pCryptKey = NULL);
   void Decode(char *pBuffer, int pLength, char *pCryptKey = NULL);
   bool IsProperlyDecrypted(char *pBuffer, int pLength);
   bool EncryptFile(char *pFileName, char *pCryptKey = NULL);
   bool DecryptFile(char *pFileName, char *pCryptKey = NULL);
   bool RecodeFile(char *pFileName, char *pDecryptKey, char *pEncryptKey);

   void SetCryptKey(char *pKey);
   char *GetCryptKey()                      { return ivCryptKey;      };

   CCommandLineInfo *GetCmdLineInfo(void)   { return &ivCmdLineInfo;  };

   // ClassWizard generated stuff
   //{{AFX_VIRTUAL(SDiaryApp)
public:
   virtual BOOL InitInstance();
   //}}AFX_VIRTUAL

   //{{AFX_MSG(SDiaryApp)
      // NOTE - the ClassWizard will add and remove member functions here.
      //    DO NOT EDIT what you see in these blocks of generated code !
   //}}AFX_MSG
   DECLARE_MESSAGE_MAP()
};

extern SDiaryApp theApp;

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before 
// the previous line.

#endif // !defined(AFX_DIARY_H__56E16263_1095_11D6_924F_000102213AEE__INCLUDED_)

