//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         diaryDlg.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Implementation of Class diaryDlg.
//
//////////////////////////////////////////////////////////////////////////////////////////////

// system-wide includes
#include "stdafx.h"
#include <time.h>
#include <sys/timeb.h>
#include <atlimage.h>

// application-specific includes
#include "diary.h"
#include "diaryDlg.h"
#include "RecodeDlg.h"
#include "toolkit.h"
#include "sdate.h"
#include "monitors.h"
#include "multimonitor.h"
#include "monitordc.h"
#include "todolist.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define BOLDDAY(ds,iDay) if (iDay>0 && iDay<32)(ds)|=(0x00000001 << (iDay-1));

// In revolt to the one class one file rule, the CAboutDlg is declared with 
// the SDiaryDlg, because it is a short class, that requires virtually nothing.
// also, it is private to the SDiaryDlg class.

class CAboutDlg : public CDialog
{
public:
   CAboutDlg();

   //{{AFX_DATA(CAboutDlg)
   enum { IDD = IDD_ABOUTBOX };
   //}}AFX_DATA

   // ClassWizard generated virtual function overrides
   //{{AFX_VIRTUAL(CAboutDlg)
protected:
   virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
   //}}AFX_VIRTUAL

   //{{AFX_MSG(CAboutDlg)
   //}}AFX_MSG
   DECLARE_MESSAGE_MAP()

public:
   virtual BOOL OnInitDialog(void);
};

// the constructor of CAboutDlg...

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
   TK_TRACER("CAboutDlg::CAboutDlg(void)");
   //{{AFX_DATA_INIT(CAboutDlg)
   //}}AFX_DATA_INIT
}

// useless, since no data is exchanged between the dialogs...

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
   TK_TRACER("CAboutDlg::DoDataExchange(void)");
   CDialog::DoDataExchange(pDX);
   //{{AFX_DATA_MAP(CAboutDlg)
   //}}AFX_DATA_MAP
}

// we do however change the version line before showing the dialog...

BOOL CAboutDlg::OnInitDialog(void)
{
   TK_TRACER("CAboutDlg::OnInitDialog(void)");

   BOOL lResult = CDialog::OnInitDialog();

   TK_LOG0("Set the current version number on the first line of the dialog");
   CString lString;
   lString.Format("Diary %s", ::GetVersionNumber());
   this->GetDlgItem(IDC_DIARY_VERSION)->SetWindowText(lString.GetBuffer());

   return lResult;
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
   //{{AFX_MSG_MAP(CAboutDlg)
      // No message handlers
   //}}AFX_MSG_MAP
END_MESSAGE_MAP()

// SDiaryDlg constructor basically loads the icon for the app, and initialises
// certain instance variables for the dialog. 

SDiaryDlg::SDiaryDlg(CWnd* pParent)
          :CResizeDlg(SDiaryDlg::IDD, pParent),
		   ivSearchDlg(0),
           ivImageDlg(0),
           ivImportDlg(0),
           ivExportDlg(0),
           ivFilesDlg(0),
           ivShortcutDlg(0),
		   ivCurrentTodoList(0)
{
   ivConfigRead = false;

   TK_TRACER("SDiaryDlg::SDiaryDlg(CWnd* pParent)");
   TK_PRMLHEX("pParent", pParent);
   //{{AFX_DATA_INIT(SDiaryDlg)
   //}}AFX_DATA_INIT

   TK_LOG0("Note that LoadIcon doesn't require subsequent DestroyIcon in Win32");
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

   TK_LOG0("Reset the page & list dirty flags");
   ivPageDirty = false;
   ivListDirty = false;

   TK_LOG0("Enable showing of the image dialog");
   ivShowImages = true;
   ivDiaryPage = NULL;
}

SDiaryDlg::~SDiaryDlg(void)
{
   TK_TRACER("SDiaryDlg::~SDiaryDlg()");
   if (this->ivDiaryPage != NULL)
   {
      TK_LOG0("Delete the SDiaryPage object, because we're dying ourselves");
      delete this->ivDiaryPage;
   }

   if (ivSearchDlg != NULL)
   {
      delete ivSearchDlg;
	  ivSearchDlg = NULL;
   }   

   if (ivImportDlg != NULL)
   {
      delete ivImportDlg;
	  ivImportDlg = NULL;
   }   

   if (ivExportDlg != NULL)
   {
      delete ivExportDlg;
	  ivExportDlg = NULL;
   }   

   if (ivImageDlg != NULL)
   {
      delete ivImageDlg;
	  ivImageDlg = NULL;
   }   

   if (ivFilesDlg != NULL)
   {
      delete ivFilesDlg;
	  ivFilesDlg = NULL;
   }   

   if (ivShortcutDlg != NULL)
   {
      delete ivShortcutDlg;
	  ivShortcutDlg = NULL;
   }   
}

// This is actually used to get the controls to exchange data...

void SDiaryDlg::DoDataExchange(CDataExchange* pDX)
{
   TK_TRACER("SDiaryDlg::DoDataExchange(CDataExchange* pDX)");
   TK_PRMLHEX("pDX", pDX);
   CResizeDlg::DoDataExchange(pDX);

   //{{AFX_DATA_MAP(SDiaryDlg)
   DDX_Control(pDX, IDC_ENCRYPTION_KEY, m_EncryptionKey);
   DDX_Control(pDX, IDC_RICHEDIT1, m_DiaryPage);
   DDX_Control(pDX, IDC_RICHEDIT2, m_TodoList);
   DDX_Control(pDX, IDC_MONTHCALENDAR1, m_pMonthCalendar);
   //}}AFX_DATA_MAP
   DDX_Control(pDX, ID_WEEKDAY, ivWeekday);
}

BEGIN_MESSAGE_MAP(SDiaryDlg, CResizeDlg)
   //{{AFX_MSG_MAP(SDiaryDlg)
   ON_WM_SYSCOMMAND()
   ON_WM_WINDOWPOSCHANGED()
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   ON_NOTIFY(MCN_GETDAYSTATE, IDC_MONTHCALENDAR1, OnGetdaystateMonthcalendar1)
   ON_NOTIFY(MCN_SELCHANGE, IDC_MONTHCALENDAR1, OnSelchangeMonthcalendar1)
   ON_EN_CHANGE(IDC_RICHEDIT1, OnChangeRichedit1)
   ON_EN_CHANGE(IDC_RICHEDIT2, OnChangeRichedit2)
   ON_BN_CLICKED(IDC_INSERT_TIME, OnInsertTime)
   ON_BN_CLICKED(IDC_SEARCH, OnSearch)
   ON_BN_CLICKED(IDC_EXPORT, OnExport)
   ON_BN_CLICKED(IDC_IMAGE, OnBnClickedImage)
   ON_NOTIFY(EN_SELCHANGE, IDC_RICHEDIT1, OnEnSelchangeRichedit1)
   //}}AFX_MSG_MAP
   ON_BN_CLICKED(IDC_IMPORT, OnBnClickedImport)
   ON_BN_CLICKED(IDC_PREV, OnBnClickedPrev)
   ON_BN_CLICKED(IDC_NEXT, OnBnClickedNext)
   ON_BN_CLICKED(IDC_FILES, OnBnClickedFiles)
   ON_BN_CLICKED(IDC_LBL_CONFIGURE, OnBnClickedLblConfigure)
   ON_BN_CLICKED(IDC_LBL1, OnBnClickedLbl1)
   ON_BN_CLICKED(IDC_LBL2, OnBnClickedLbl2)
   ON_BN_CLICKED(IDC_LBL3, OnBnClickedLbl3)
   ON_BN_CLICKED(IDC_LBL4, OnBnClickedLbl4)
   ON_BN_CLICKED(IDC_LBL5, OnBnClickedLbl5)
   ON_BN_CLICKED(IDC_LBL6, OnBnClickedLbl6)
   ON_BN_CLICKED(IDC_LBL7, OnBnClickedLbl7)
   ON_BN_CLICKED(IDC_LBL8, OnBnClickedLbl8)
   ON_BN_CLICKED(IDC_LBL9, OnBnClickedLbl9)
   ON_BN_CLICKED(IDC_LBL10, OnBnClickedLbl10)
   ON_BN_CLICKED(IDC_LBL11, OnBnClickedLbl11)
   ON_BN_CLICKED(IDC_LBL12, OnBnClickedLbl12)
   ON_BN_CLICKED(IDC_LBL13, OnBnClickedLbl13)
   ON_BN_CLICKED(IDC_LBL14, OnBnClickedLbl14)
   ON_BN_CLICKED(IDC_LBL15, OnBnClickedLbl15)
   ON_BN_CLICKED(IDC_LBL16, OnBnClickedLbl16)
   ON_BN_CLICKED(IDC_LBL17, OnBnClickedLbl17)
   ON_BN_CLICKED(IDC_LBL18, OnBnClickedLbl18)
   ON_BN_CLICKED(IDC_PRINT, &SDiaryDlg::OnBnClickedPrint)
   ON_BN_CLICKED(IDC_TODO0, &SDiaryDlg::OnBnClickedTodo0)
   ON_BN_CLICKED(IDC_TODO1, &SDiaryDlg::OnBnClickedTodo1)
   ON_BN_CLICKED(IDC_TODO2, &SDiaryDlg::OnBnClickedTodo2)
   ON_BN_CLICKED(IDC_TODO3, &SDiaryDlg::OnBnClickedTodo3)
   ON_BN_CLICKED(IDC_TODO4, &SDiaryDlg::OnBnClickedTodo4)
   ON_BN_CLICKED(IDC_TODO5, &SDiaryDlg::OnBnClickedTodo5)
   ON_BN_CLICKED(IDC_TODO6, &SDiaryDlg::OnBnClickedTodo6)
   ON_BN_CLICKED(IDC_TODO7, &SDiaryDlg::OnBnClickedTodo7)
   ON_BN_CLICKED(IDC_CHECK1, &SDiaryDlg::OnBnClickedCheck1)
   ON_EN_CHANGE(IDC_NUMDAYS, &SDiaryDlg::OnEnChangeNumdays)
END_MESSAGE_MAP()

// SDiaryDlg::OnInitDialog adds the about box to the system menu, and 
// further initialises the CRichEditCtrls. It also determines the first 
// date to show, and loads the diary page and the todo list. 

BOOL SDiaryDlg::OnInitDialog()
{
   TK_TRACER("SDiaryDlg::OnInitDialog(void)");

   CWaitCursor lWaiter;

   CResizeDlg::OnInitDialog();

   AddControl(IDC_RICHEDIT1, CST_NONE, CST_RESIZE, CST_NONE, CST_RESIZE, 1);
   AddControl(IDC_RICHEDIT2, CST_NONE, CST_NONE, CST_NONE, CST_RESIZE, 1);
   AddControl(IDC_NEXT, CST_NONE, CST_REPOS, CST_NONE, CST_NONE, 1);
   AddControl(IDC_LBL_CONFIGURE, CST_NONE, CST_NONE, CST_NONE, CST_REPOS, 1);
   AddControl(IDC_GROUPBOX, CST_NONE, CST_NONE, CST_NONE, CST_REPOS, 0);
   AddControl(IDC_TODO0, CST_NONE, CST_NONE, CST_NONE, CST_REPOS, 1);
   AddControl(IDC_TODO1, CST_NONE, CST_NONE, CST_NONE, CST_REPOS, 1);
   AddControl(IDC_TODO2, CST_NONE, CST_NONE, CST_NONE, CST_REPOS, 1);
   AddControl(IDC_TODO3, CST_NONE, CST_NONE, CST_NONE, CST_REPOS, 1);
   AddControl(IDC_TODO4, CST_NONE, CST_NONE, CST_NONE, CST_REPOS, 1);
   AddControl(IDC_TODO5, CST_NONE, CST_NONE, CST_NONE, CST_REPOS, 1);
   AddControl(IDC_TODO6, CST_NONE, CST_NONE, CST_NONE, CST_REPOS, 1);
   AddControl(IDC_TODO7, CST_NONE, CST_NONE, CST_NONE, CST_REPOS, 1);

   SetWindowText(getWindowTitle().GetBuffer());

   TK_LOG0("Initialize the SSettings object thats part of the SDiaryDlg");
   this->GetSettings()->SetFileName("settings.dat");
   this->GetSettings()->Dump();

   TK_LOG0("Read the position settings and restore them");
   SSetting *lSettingTop    = this->GetSettings()->GetSetting(SET_TOP);
   SSetting *lSettingLeft   = this->GetSettings()->GetSetting(SET_LEFT);
   SSetting *lSettingWidth  = this->GetSettings()->GetSetting(SET_WIDTH);
   SSetting *lSettingHeight = this->GetSettings()->GetSetting(SET_HEIGHT);

   if (lSettingTop    != NULL &&
       lSettingLeft   != NULL &&
	   lSettingWidth  != NULL &&
	   lSettingHeight != NULL)
   {
      TK_LOG0("Position settings found, positioning window");
      int lTop  = lSettingTop->GetValueAsNumber();
      int lLeft = lSettingLeft->GetValueAsNumber();
	  int lWidth = lSettingWidth->GetValueAsNumber();
	  int lHeight = lSettingHeight->GetValueAsNumber();

	  // make sure it will be on-screen, but keep the original position in config
	  this->SetWindowPos(0, lLeft, lTop, lWidth, lHeight, 0);
	  if (CMonitors::IsOnScreen( this ) == false)
	  {
		  CMonitor thePrimary = CMonitors::GetPrimaryMonitor();
		  thePrimary.CenterWindowToMonitor(this, true);
		  ivWindowCentered = true;
          WINDOWPLACEMENT lWinPlace;
          this->GetWindowPlacement(&lWinPlace);
          ivTop = lWinPlace.rcNormalPosition.top;
          ivLeft = lWinPlace.rcNormalPosition.left;
	  }
   }

   ivConfigRead = true;

   TK_LOG0("Lock the other controls, so we can't get to work");
   this->EnableAllControls(FALSE);
   
   TK_LOG0("Add other menu items to system menu");
   TK_LOG0("IDM_values must be in the system command range");
   ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
   ASSERT(IDM_ABOUTBOX < 0xF000);
   ASSERT((IDM_RECODE & 0xFFF0) == IDM_RECODE);
   ASSERT(IDM_RECODE < 0xF000);
   ASSERT((IDM_8POINTS & 0xFFF0) == IDM_8POINTS);
   ASSERT(IDM_8POINTS < 0xF000);
   ASSERT((IDM_10POINTS & 0xFFF0) == IDM_10POINTS);
   ASSERT(IDM_10POINTS < 0xF000);
   ASSERT((IDM_12POINTS & 0xFFF0) == IDM_12POINTS);
   ASSERT(IDM_12POINTS < 0xF000);
   ASSERT((IDM_14POINTS & 0xFFF0) == IDM_14POINTS);
   ASSERT(IDM_14POINTS < 0xF000);
   ASSERT((IDM_16POINTS & 0xFFF0) == IDM_16POINTS);
   ASSERT(IDM_16POINTS < 0xF000);

   TK_LOG0("Building the System Menu");
   CMenu* pSysMenu = GetSystemMenu(FALSE);
   if (pSysMenu != NULL)
   {
      TK_LOG0("First the Recode Data Files entry");
      CString strRecodeMenu;
      strRecodeMenu.LoadString(IDS_RECODE);
      if (!strRecodeMenu.IsEmpty())
      { 
          pSysMenu->AppendMenu(MF_SEPARATOR);
          pSysMenu->AppendMenu(MF_STRING, IDM_RECODE, strRecodeMenu);
      }   

      TK_LOG0("Add the font size entries to the menu");
      CString str8PointsMenu;
      CString str10PointsMenu;
      CString str12PointsMenu;
      CString str14PointsMenu;
      CString str18PointsMenu;

      str8PointsMenu.LoadString(IDS_8POINTS);
      str10PointsMenu.LoadString(IDS_10POINTS);
      str12PointsMenu.LoadString(IDS_12POINTS);
      str14PointsMenu.LoadString(IDS_14POINTS);
      str18PointsMenu.LoadString(IDS_16POINTS);

      if (!str8PointsMenu.IsEmpty())
      { 
          pSysMenu->AppendMenu(MF_SEPARATOR);
          pSysMenu->AppendMenu(MF_STRING, IDM_8POINTS, str8PointsMenu);
          if (!str10PointsMenu.IsEmpty())
          {
             pSysMenu->AppendMenu(MF_STRING, IDM_10POINTS, str10PointsMenu);
          }
          if (!str12PointsMenu.IsEmpty())
          {
             pSysMenu->AppendMenu(MF_STRING, IDM_12POINTS, str12PointsMenu);
          }
          if (!str14PointsMenu.IsEmpty())
          {
             pSysMenu->AppendMenu(MF_STRING, IDM_14POINTS, str14PointsMenu);
          }
          if (!str18PointsMenu.IsEmpty())
          {
             pSysMenu->AppendMenu(MF_STRING, IDM_16POINTS, str18PointsMenu);
          }
      }   

      TK_LOG0("Add the about entry to the menu");
      CString strAboutMenu;
      strAboutMenu.LoadString(IDS_ABOUTBOX);
      if (!strAboutMenu.IsEmpty())
      { 
          pSysMenu->AppendMenu(MF_SEPARATOR);
          pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
      }   
   }

   // set the number of days to print to one...
   SSetting *lSettingNumDays   = this->GetSettings()->GetSetting(SET_NUMDAYS);
   if (lSettingNumDays != NULL)
   {
       this->GetDlgItem(IDC_NUMDAYS)->SetWindowTextA(lSettingNumDays->GetValue());
   }
   else
   {
       this->GetDlgItem(IDC_NUMDAYS)->SetWindowText("1");
   }

   this->CheckDlgButton(IDC_TODO0, TRUE);
   ivSelectedTodoList = 0;
   ivOldSelectedTodoList = 0;

   // Set the icon for this dialog.  The framework does this automatically
   // when the application's main window is not a dialog
   SetIcon(m_hIcon, TRUE);       // Set big icon
   SetIcon(m_hIcon, FALSE);      // Set small icon
   
   TK_LOG0("Instantiate the subdialogs");
   ivSearchDlg   = new SSearchDlg(this);      
   ivImageDlg    = new SImageDlg(this);
   ivImportDlg   = new SImportDlg(this);
   ivExportDlg   = new SExportDlg(this);
   ivFilesDlg    = new SFilesDlg(this);
   ivShortcutDlg = new SLabelDlg(this);
   SetLabelButtons();
   ivCurrentTodoList = ivShortcutDlg->GetDefaultTodoList();

   TK_LOG0("Retrieve the current date");
   SYSTEMTIME lTime;
   m_pMonthCalendar.GetCurSel(&lTime);

   TK_LOG0("get the passed in date, if applicable");
   GetTimeFromPassedInFile(&lTime);
   m_pMonthCalendar.SetCurSel(&lTime);

   TK_LOG0("Retrieve the current fontsize and set it");
   SSetting *lFontSizeSSetting = this->ivSettings.GetSetting(SET_FONT_SIZE);
   if (lFontSizeSSetting == NULL)
   {
      lFontSizeSSetting = new SSetting(SET_FONT_SIZE, "12");
      this->ivSettings.AddSetting(lFontSizeSSetting);
   }
   this->SetFont(lFontSizeSSetting->GetValueAsNumber());

   TK_LOG0("set the new date, and load diary page and todo list");
   this->ivNewSelDate.Set(lTime.wYear, lTime.wMonth, lTime.wDay);

   this->ivCurrentTodoList->Load();
   this->LoadNewPage();

   TK_LOG0("Lock the Rich Edit Controls until password has been set");
   this->m_DiaryPage.EnableWindow(false);
   this->m_TodoList.EnableWindow(false);

   CString lString;
   lString.Format("%s", ::GetVersionNumber());
   this->GetDlgItem(IDC_VERSIONNUMBER)->SetWindowText(lString.GetBuffer());

   TK_LOG0("Make Smart Import the default");
   this->CheckDlgButton(IDC_SMART, 1);

   // to get the rich edit controls to send EN_CHANGE events
   // and the diary page to send EN_SELCHANGE events, we need
   // to tell them we want these notifications

   TK_LOG0("Set the RichEditCtrls to send ENM_CHANGE & ENM_SELCHANGE messages");
   long lEventMask = m_DiaryPage.GetEventMask();
   lEventMask |= ENM_CHANGE;
   lEventMask |= ENM_SELCHANGE;
   m_DiaryPage.SetEventMask(lEventMask);

   lEventMask = m_TodoList.GetEventMask();
   lEventMask |= ENM_CHANGE;
   m_TodoList.SetEventMask(lEventMask);

   this->UpdateStatus();
   this->UpdateWeekday();

   return TRUE;  // return TRUE  unless you set the focus to a control
} 

// this method sets the fontsize to a given number between 8 and 18.
// in fact it can be set to any number between 8 and 18, but the menu
// restricts it to values that really differ...

void SDiaryDlg::SetFont(int pFontSize)
{
   TK_TRACER("SDiaryDlg::SetFont(int pFontSize)");
   CWaitCursor lWaiter;

   TK_LOG0("Limit fontsize to 8..18");
   if (pFontSize < 8)
   {
      pFontSize = 8;
   }
   if (pFontSize > 18)
   {
      pFontSize = 18;
   }

   TK_LOG0("Store fontsize setting in settings.dat");
   char *lValue = new char[MAX_STRING];
   _itoa_s(pFontSize, lValue, MAX_STRING - 1, 10);
   SSetting *lNewSSettingObject = new SSetting(SET_FONT_SIZE, lValue);
   this->ivSettings.AddSetting(lNewSSettingObject);
   this->ivSettings.WriteFile();

   TK_LOG0("Set fontsize on both RichEditCtrls");
   CHARFORMAT lDefaultCharFormat;

   lDefaultCharFormat.cbSize = sizeof(CHARFORMAT); 
   lDefaultCharFormat.dwMask = CFM_CHARSET | CFM_SIZE; 
   lDefaultCharFormat.dwEffects = 0; 
   lDefaultCharFormat.yHeight = pFontSize * 20; 
   lDefaultCharFormat.yOffset = 0; 
   lDefaultCharFormat.crTextColor = 0x00000000; 
   lDefaultCharFormat.bCharSet = ANSI_CHARSET; 
   lDefaultCharFormat.bPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE; 
   strcpy_s(lDefaultCharFormat.szFaceName, 10, "Arial"); 

   TK_LOG0("But save diary page and todolist before doing so");
   this->ivCurrentTodoList->Save();
   this->SaveOldPage(true);
   
   m_DiaryPage.SetDefaultCharFormat(lDefaultCharFormat);
   m_TodoList.SetDefaultCharFormat(lDefaultCharFormat);

   TK_LOG0("And of course restore them afterwards");
   this->ivCurrentTodoList->Load();
   this->LoadNewPage();
   
   delete [] lValue;
}

// This method gets called when the user selects a menu entry from the 
// system menu. If it isn't one of the special ones, we pass it on to
// the CResizeDlg object.  

void SDiaryDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
   TK_TRACER("SDiaryDlg::OnSysCommand(UINT nID, LPARAM lParam)");
   CWaitCursor lWaiter;

   if ((nID & 0xFFF0) == IDM_ABOUTBOX)
   {
      TK_LOG0("About Diary selected, show About box");
      CAboutDlg dlgAbout;
      dlgAbout.DoModal();
   }
   else if ((nID & 0xFFF0) == IDM_8POINTS)
   {
      TK_LOG0("Font Size 8 selected, set it");
      this->SetFont(8);
   }
   else if ((nID & 0xFFF0) == IDM_10POINTS)
   {
      TK_LOG0("Font Size 10 selected, set it");
      this->SetFont(10);
   }
   else if ((nID & 0xFFF0) == IDM_12POINTS)
   {
      TK_LOG0("Font Size 12 selected, set it");
      this->SetFont(12);
   }
   else if ((nID & 0xFFF0) == IDM_14POINTS)
   {
      TK_LOG0("Font Size 14 selected, set it");
      this->SetFont(14);
   }
   else if ((nID & 0xFFF0) == IDM_16POINTS)
   {
      TK_LOG0("Font Size 18 selected, set it");
      this->SetFont(18);
   }
   else if ((nID & 0xFFF0) == IDM_RECODE)
   {
      TK_LOG0("Recode data files selected, execute dialog");
      this->ivCurrentTodoList->Save();
      this->SaveOldPage(true);

      SRecodeDlg dlgRecode;
      dlgRecode.DoModal();

      this->ivCurrentTodoList->Load();
      this->LoadNewPage();
   }
   else
   {
      TK_LOG0("Other syscommand selected, relay to CResizeDlg");
      CResizeDlg::OnSysCommand(nID, lParam);
   }
}

// If you add a minimize button to your dialog, you will need the code below
// to draw the icon.  For MFC applications using the document/view model,
// this is automatically done for you by the framework.

void SDiaryDlg::OnPaint() 
{
   TK_TRACER("SDiaryDlg::OnPaint()");
   CWaitCursor lWaiter;

   if (IsIconic())
   {
      TK_LOG0("Repaint Icon");
      CPaintDC dc(this); // device context for painting

      SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

      TK_LOG0("Center icon in client rectangle");
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      TK_LOG0("Draw the icon");
      dc.DrawIcon(x, y, m_hIcon);
   }
   else
   {
      TK_LOG0("Repaint Dialog, through CResizeDlg::OnPaint");
      CResizeDlg::OnPaint();
   }
}

// The system calls this to obtain the cursor to display while the user drags
// the minimized window.

HCURSOR SDiaryDlg::OnQueryDragIcon()
{
   TK_TRACER("SDiaryDlg::OnQueryDragIcon()");
   return (HCURSOR) m_hIcon;
}

// this method handles the daystate messages that the CCalendarCtrl sends to 
// the application. it basically boldens those dates for which a data file exists. 

void SDiaryDlg::OnGetdaystateMonthcalendar1(NMHDR* pNMHDR, LRESULT* pResult) 
{
   TK_TRACER("SDiaryDlg::OnGetdaystateMonthcalendar1(NMHDR* pNMHDR, LRESULT* pResult)");
   CWaitCursor lWaiter;

   NMDAYSTATE* pDayState = (NMDAYSTATE *)pNMHDR;

   MONTHDAYSTATE mdState[3];
   INT i, iMax;

   iMax = pDayState->cDayState;

   for (i = 0; i < iMax; i++)
   {
      mdState[i] = GetDayStatesForMonth(this->ivNewSelDate.getYear(), this->ivNewSelDate.getMonth(), i);
   } 

   pDayState->prgDayState = mdState;
}

// this method gets called if the user selects a different date from the 
// calendar control. These messages sometimes get sent even if the date 
// didn't change, so be careful to skip those messages. 

void SDiaryDlg::OnSelchangeMonthcalendar1(NMHDR* pNMHDR, LRESULT* pResult) 
{
   TK_TRACER("OnSelchangeMonthcalendar1(NMHDR* pNMHDR, LRESULT* pResult)");
   SYSTEMTIME lTime;

   CWaitCursor lWaiter;

   TK_LOG0("Is this message coming from the right Hwnd?");
   if (m_pMonthCalendar.m_hWnd == pNMHDR->hwndFrom)
   {
      TK_LOG0("Get the newly selected date");
      if (m_pMonthCalendar.GetCurSel(&lTime))
      {
         TK_LOG0("And calculate the new date in format CCYYMMDD");
         SDate lNewDate(lTime.wYear, lTime.wMonth, lTime.wDay);
		 SetNewDate(lNewDate);
      }
   }

   TK_LOG0("Pages updated, recalc Image count again...");
   this->UpdateStatus();
   this->UpdateWeekday();

   *pResult = 0;
}

// this method saves the contents of the diary page edit control, based 
// on the value of pFinal. If set, then ivNewSelectionDate is used, otherwise
// ivOldSelectionData is used. Saving includes encryption, if an encryption 
// keyword has been set. 

bool SDiaryDlg::SaveOldPage(bool pFinal)
{
   TK_TRACER("SDiaryDlg::SaveOldPage(bool pFinal)");
   if (ivPageDirty)
   {
      TK_LOG0("something to save, so lets do it");
      CString lFileName;
      lFileName.Format("%08d.dry", this->ivOldSelDate.asDecimal());
      if (pFinal)
      {
         lFileName.Format("%08d.dry", this->ivNewSelDate.asDecimal());
      }

      TK_LOG0("Open the file for writing");
      FILE *lFile = fopen(lFileName.GetBuffer(0), "wb");

      if (lFile == NULL)
      {
         CString lMsg;
         lMsg.Format("File %s could not be opened for writing.\ncontinue intended action?", lFileName);
         if (AfxMessageBox(lMsg, MB_YESNO) == IDNO)
		 {
			 return false;
		 }
		 else
		 {
			 return true;
		 }
      }

      TK_LOG0("file now open, proceed");
      long lMin = 0;
      long lMax = 0;
      char lBuffer[MAX_BUFFER + 1];
      m_DiaryPage.SetSel(0, -1);
      m_DiaryPage.GetSel(lMin, lMax);

      if (lMin == 0 && lMax == 2)
      { 
         // nothing to save, delete datafile
         // for some funny reason, this seems to not work. 
         // _unlink keeps returning -1.....
         fclose(lFile);
         lFile = NULL;

         CString lFilename;
         DWORD lDirReturn = 0;
         char  *lPath = new char[MAX_STRING];

         TK_LOG0("Retrieve the path of execution");
         lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);

         lFilename.Format("%s\\%08d.dry", lPath, this->ivOldSelDate.asDecimal());

         TK_LOG1("Attempting to delete %s", lFilename.GetBuffer());
         int lRet = -1;
         long lCount = 0;
         while ((lRet = _unlink(lFilename.GetBuffer(0))) == -1 && lCount < 10)
         {
            lCount++;
         }

#ifdef TIME_TO_INVESTIGATE_THE_UNLINK_PROBLEM
         TK_LOG1("Return code from _unlink = %d", lRet);
         if (lRet == -1)
         {
            switch (errno)
            {
            case EACCES:
               MessageBox("Read-only file, cannot delete");
               break;
            case ENOENT:
               MessageBox("Path specifies a directory, or file not found");
               break;
            }
         }
#endif

         m_DiaryPage.SetWindowText("");
		 delete [] lPath;
         return true;
      }

      TK_LOG0("Encode the page, and save it to the file");
      CString theStr = m_DiaryPage.GetSelText();
	  strcpy_s(lBuffer, MAX_BUFFER, theStr);
      m_DiaryPage.SetSel(-1, -1);
      theApp.Encode(lBuffer, lMax - lMin);
      fwrite(lBuffer, 1, lMax - lMin, lFile);

      fclose(lFile);

      m_DiaryPage.SetWindowText("");
   }
   return true;
}

// this method loads a new page into the diary page edit control. 
// ivNewSelectionDate is used to determine the correct data file. 
// if an encryption keyword has been entered, decryption will occur. 

void SDiaryDlg::LoadNewPage(void)
{
   TK_TRACER("SDiaryDlg::LoadNewPage(void)");
   CWaitCursor lWaiter;

   m_DiaryPage.SetWindowText("");

   TK_LOG0("Build the correct filename, and open the file");
   CString lFileName;
   lFileName.Format("%08d.dry", this->ivNewSelDate.asDecimal());

   FILE *lFile = fopen(lFileName, "rb");

   if (lFile != NULL)
   {
      TK_LOG0("File is open, read it and decode");
      char lBuffer[MAX_BUFFER + 1];

      int lBytesRead = fread(lBuffer, 1, MAX_BUFFER, lFile);
      if (lBytesRead > 0)
      {
         lBuffer[lBytesRead] = '\0';
         theApp.Decode(lBuffer, lBytesRead);
         if (theApp.IsProperlyDecrypted(lBuffer, lBytesRead))
         {
            m_DiaryPage.SetWindowText(lBuffer);
            HighlightSearchStrings(&m_DiaryPage);
            HighlightHyperlinks(&m_DiaryPage, true);
            m_DiaryPage.SetSel(-1, -1);
            HighlightSearchStrings(&m_TodoList);
            m_DiaryPage.EnableWindow(true);
         }
         else
         {
            m_DiaryPage.SetWindowText("");
            m_DiaryPage.EnableWindow(false);
         }
      }
      else
      {
         m_DiaryPage.SetWindowText("");
         m_DiaryPage.EnableWindow(true);
      }
   }
   else
   {
      m_DiaryPage.SetWindowText("");
      m_DiaryPage.EnableWindow(true);
   }

   ivPageDirty = false;
   this->UpdateWeekday();
}

// This method gets called if the user presses enter while the 
// encryption keyword edit control is focused. It basically saves
// any unsaved stuff, sets the keyword, and loads the relevant 
// pages again. 

void SDiaryDlg::OnOK() 
{
   TK_TRACER("SDiaryDlg::OnOK()");
   CWaitCursor lWaiter;

   TK_LOG0("Unlock the Rich Edit Controls because password has been set");
   this->m_DiaryPage.EnableWindow(true);
   this->m_TodoList.EnableWindow(true);
   
   TK_LOG0("Unlock the other controls, so we can get to work");
   this->EnableAllControls(TRUE);

   TK_LOG0("Save todolist and diary page");
   this->ivCurrentTodoList->Save();
   this->SaveOldPage(true);

   TK_LOG0("Set the new encryption keyword");
   char *lCryptKey = new char[MAX_STRING];
   m_EncryptionKey.GetWindowText(lCryptKey, MAX_STRING);
   theApp.SetCryptKey(lCryptKey);
   
   TK_LOG0("...and load the diary page and the todolist again");
   this->ivCurrentTodoList->Load();
   this->LoadNewPage();

   TK_LOG0("Last of all, set the encryption field to empty");
   m_EncryptionKey.SetWindowText("");
   m_DiaryPage.SetFocus();

   TK_LOG0("This is the right point to encrypt any hitherto unencrypted images");
      
   CFileFind finder;
   BOOL bWorking = finder.FindFile("*.jpg");
   bool lAsked = false;
   while (bWorking)
   {   
      if (lAsked == false)
      {
         if (AfxMessageBox("Did you fill in your keyword correctly?", MB_YESNO) == IDNO)
         {
            break;
         }
         lAsked = true;
      }

      bWorking = finder.FindNextFile();

      CString lMsg;
      lMsg.Format("encrypting image file %s", finder.GetFileName().GetBuffer());
      ivWeekday.SetWindowText(lMsg.GetBuffer());

      theApp.EncryptFile(finder.GetFilePath().GetBuffer());
      
   }
   ivWeekday.SetWindowText("");
   this->UpdateWeekday();
   delete [] lCryptKey;
}

// this gets called if the user clicks the close button. It saves any 
// unsaved information, before exiting the application. 

void SDiaryDlg::OnCancel() 
{
   TK_TRACER("SDiaryDlg::OnCancel()");
   CWaitCursor lWaiter;

   TK_LOG0("Save the todolist and the diary page, and exit");
   bool todoSaved = this->ivCurrentTodoList->Save();
   bool pageSaved = this->SaveOldPage(true);
   
   if (pageSaved && todoSaved)
   {
       CResizeDlg::OnCancel();
   }
}

// the next two methods make sure that the dirty flags for both 
// CRichEditCtrls are set properly, and the cursor stays visible
// while editing. 

void SDiaryDlg::OnChangeRichedit1() 
{
   TK_TRACER("SDiaryDlg::OnChangeRichedit1()");
   long lStartOfSelection = 0;
   long lStartOfPage = 0;
   
   TK_LOG0("Get position of cursor, and first visible line");
   lStartOfSelection = m_DiaryPage.LineFromChar(-1);
   lStartOfPage      = m_DiaryPage.GetFirstVisibleLine();

   // Now that there are different sized fonts possible in the diary 
   // page, we work out with the maximum number of lines for different font sizes.

   TK_LOG0("If needed, correct first visible line to keep cursor on screen");
   if (lStartOfSelection - lStartOfPage < 2 ||
       lStartOfSelection - lStartOfPage > this->GetNumberOfLinesForDiaryWindow() - 2)
   {
      m_DiaryPage.LineScroll(lStartOfSelection 
                           - lStartOfPage 
                           - this->GetNumberOfLinesForDiaryWindow() / 2);
   }
 
   this->ivShowImages = true;
   HighlightHyperlinks(&m_DiaryPage, true);

   // set the modification flag
   ivPageDirty = true;
}

void SDiaryDlg::OnChangeRichedit2() 
{
   TK_TRACER("SDiaryDlg::OnChangeRichedit2()");
   long lStartOfSelection = 0;
   long lStartOfPage = 0;
   
   TK_LOG0("Get position of cursor, and first visible line");
   lStartOfSelection = m_TodoList.LineFromChar(-1);
   lStartOfPage      = m_TodoList.GetFirstVisibleLine();
  
   // Now that there are different sized fonts possible in the todo
   // list, and the window itself is smaller, these limits are incorrect. 
   // Must work out something that is correct for any fontsize....

   TK_LOG0("If needed, correct first visible line to keep cursor on screen");
   if (lStartOfSelection - lStartOfPage < 0 ||
       lStartOfSelection - lStartOfPage > this->GetNumberOfLinesForTodoWindow() - 2)
   {
      m_TodoList.LineScroll(lStartOfSelection 
                          - lStartOfPage 
                          - this->GetNumberOfLinesForTodoWindow() / 2);
   }

   TK_LOG0("set the modification flag");   
   ivListDirty = true;
}

// This method determines the daystates for a single month

MONTHDAYSTATE SDiaryDlg::GetDayStatesForMonth(int pYear,
											  int pMonth, 
                                              int pMonthOffset)
{
   TK_TRACER("SDiaryDlg::GetDayStatesForMonth(NMDAYSTATE *pDayState, int pMonthOffset)");

   CWaitCursor lWaiter;
   MONTHDAYSTATE lDayState = 0;
   int lMonth = pMonth - 1 + pMonthOffset;
   int lYear  = pYear;

   if (lMonth < 1)
   {
	   lMonth = 1;
   }

   if (lMonth > 12)
   {
      lMonth -= 12;
      lYear  += 1;
   }

   TK_LOG0("for all days in the month");
   for (int lDay = 1; lDay <= 31; lDay++)
   {
      TK_LOG0("Check for existence of a diary file");
      CString lFileName;
      lFileName.Format("%04d%02d%02d.dry", 
                       lYear, 
                       lMonth,
                       lDay);
      FILE *lFile = fopen(lFileName.GetBuffer(0), "r");
      if (lFile != NULL)
      {
         TK_LOG0("and set the corresponding date to bold");
         BOLDDAY(lDayState, lDay);
         fclose(lFile);
      }
   }

   return lDayState;
}

// This method inserts the current time in the diary page edit control

void SDiaryDlg::OnInsertTime() 
{
   TK_TRACER("SDiaryDlg::OnInsertTime()");
   CWaitCursor lWaiter;

   long lMin = 0;
   long lMax = 0;

   TK_LOG0("set the selection to a single point of insertion");
   m_DiaryPage.GetSel(lMin, lMax);
   m_DiaryPage.SetSel(lMin, lMin);

   time_t          lTime;
   struct tm       lToday;
   static char     lTimeStamp[21];

   TK_LOG0("Use time structure to build a customized time string.");
   time(&lTime);
   localtime_s( &lToday, &lTime );
   strftime(lTimeStamp, 20, "%H:%M  ", &lToday);

   TK_LOG0("check date to be on yesterday, if so advance day");
   SDate thePage = this->GetCurrentDate();
   SDate theTime = SDate(lToday.tm_year + 1900, lToday.tm_mon + 1, lToday.tm_mday);
   if (thePage != theTime)
   {
	   // if we did a timestamp, that implies today...
	   SetNewDate(theTime);
   }

   TK_LOG0("insert the time string into the CRichEditCtrl of the diary page");
   m_DiaryPage.ReplaceSel(lTimeStamp);
   m_DiaryPage.SetFocus();
}

// This method starts the search dialog

void SDiaryDlg::OnSearch() 
{
   TK_TRACER("SDiaryDlg::OnSearch()");
   SYSTEMTIME lTime;
   int lCharPos = 0;

   CWaitCursor lWaiter;

   TK_LOG0("Save todolist and diary page");
   this->ivCurrentTodoList->Save();
   this->SaveOldPage(true);

   TK_LOG0("Start the search dialog");
   ASSERT(ivSearchDlg != NULL);
   if (ivSearchDlg->DoModal() == IDOK)
   {
      TK_LOG0("If OK pressed, get the selected date and position the diary likewise");
      lTime = ivSearchDlg->GetSelectedDate();
      m_pMonthCalendar.SetCurSel(&lTime);
      long lNewDate = lTime.wYear  * 10000 +
                      lTime.wMonth *   100 +
                      lTime.wDay;

      TK_LOG0("Save the position of the sought string");
      lCharPos = ivSearchDlg->GetCharPos();

      this->ivOldSelDate = this->ivNewSelDate;
      this->ivNewSelDate.Set(lNewDate);
   }

   TK_LOG0("Load the new page");
   this->LoadNewPage();

   if (lCharPos > 0)
   {
      TK_LOG0("Position the page five lines above the correct line");
      int lLine = m_DiaryPage.LineFromChar(lCharPos);
      m_DiaryPage.LineScroll(-100000);
      m_DiaryPage.LineScroll(lLine - 5);
   }
}

// This method highlights in red all parts of the CRichEditCtrl text
// that match the search string. 

void SDiaryDlg::HighlightSearchStrings(CRichEditCtrl *pCtrl)
{
   TK_TRACER("SDiaryDlg::HighlightSearchStrings(CRichEditCtrl *pCtrl)");
   CWaitCursor lWaiter;

   // no point in highlighting if the search dialog isn't there yet...
   if (ivSearchDlg == NULL)
   {
	   return;
   }

   CString lSearchString = ivSearchDlg->GetSearchString();

   CHARFORMAT lCf;

   lCf.cbSize           = sizeof(CHARFORMAT); 
   lCf.dwMask           = CFM_COLOR; 
   lCf.dwEffects        = 0; 
   lCf.yHeight          = 0; 
   lCf.yOffset          = 0; 
   lCf.crTextColor      = 0x00000000; 
   lCf.bCharSet         = 0; 
   lCf.bPitchAndFamily  = 0; 
   lCf.szFaceName[0]    = '\0'; 

   pCtrl->SetSel(0, -1);
   pCtrl->SetSelectionCharFormat(lCf);

   lCf.crTextColor      = 0x000000FF;      

   FINDTEXTEX lFtt;

   lFtt.chrg.cpMin     =  0;
   lFtt.chrg.cpMax     = -1;
   lFtt.lpstrText      =  lSearchString.GetBuffer(0);
   lFtt.chrgText.cpMin =  0;
   lFtt.chrgText.cpMax =  0;

   TK_LOG0("While searchstring found.....");
   while (pCtrl->FindText(0, &lFtt) != -1)
   {
      TK_LOG0("... turn it to RED");
      pCtrl->SetSel(lFtt.chrgText);
      pCtrl->SetSelectionCharFormat(lCf);

      lFtt.chrg.cpMin = lFtt.chrgText.cpMax;
   }
}

void SDiaryDlg::HighlightHyperlinks(CRichEditCtrl *pCtrl, bool pFullPage)
{
	// make sure we don't re-enter while it's being executed...
	static bool stillRunning = false;
	if (stillRunning)
	{
		return;
	}
	stillRunning = true;

   TK_TRACER("SDiaryDlg::HighlightHyperlinks(CRichEditCtrl *pCtrl)");
   CWaitCursor lWaiter;

   // we don't want to accidentally pop up the image dialog when we
   // set selections to add the underline.....
   this->ivShowImages = false;
   this->ivKillSelChangeProcessing = true;
   
   // Go for the image hyperlinks first...
   CHARFORMAT lCf;

   lCf.cbSize           = sizeof(CHARFORMAT); 
   lCf.dwMask           = CFM_UNDERLINE; 
   lCf.dwEffects        = CFE_UNDERLINE; 
   lCf.yHeight          = 0; 
   lCf.yOffset          = 0; 
   lCf.crTextColor      = 0;      
   lCf.bCharSet         = 0; 
   lCf.bPitchAndFamily  = 0; 
   lCf.szFaceName[0]    = '\0'; 

   FINDTEXTEX lFtt;
   CHARRANGE  lLeftAt;

   if (pFullPage == true)
   {
      lFtt.chrg.cpMin     =  0;
      lFtt.chrg.cpMax     = -1;
	  //m_DiaryPage.SetSel(0, -1);
      m_DiaryPage.GetSel(lLeftAt);
   }
   else
   {
      m_DiaryPage.GetSel(lFtt.chrg);
      lLeftAt.cpMin = lFtt.chrg.cpMin;
      lLeftAt.cpMax = lFtt.chrg.cpMax;

      if (lFtt.chrg.cpMin != lFtt.chrg.cpMax)
      {
		  return;
      }
      lFtt.chrg.cpMin     -= 8;
      lFtt.chrg.cpMax     -= 2;
   }
   lFtt.lpstrText      =  "Slot";
   lFtt.chrgText.cpMin =  lFtt.chrg.cpMin;
   lFtt.chrgText.cpMax =  lFtt.chrg.cpMax;

   TK_LOG0("While Slot searchstring found.....");
   while (pCtrl->FindText(0, &lFtt) != -1)
   {
      char lBuffer[MAX_BUFFER];

      TK_LOG0("... underline it!");
      lFtt.chrgText.cpMax++;
      pCtrl->SetSel(lFtt.chrgText);
      CString theStr = pCtrl->GetSelText();
	  strcpy_s(lBuffer, MAX_BUFFER, theStr);
      lBuffer[5] = '\0';

      if (lBuffer[4] == '0' ||
          lBuffer[4] == '1' ||
          lBuffer[4] == '2' ||
          lBuffer[4] == '3' ||
          lBuffer[4] == '4' ||
          lBuffer[4] == '5' ||
          lBuffer[4] == '6' ||
          lBuffer[4] == '7' ||
          lBuffer[4] == '8' ||
          lBuffer[4] == '9')
      {
         TK_LOG0("Only if an image really exists in the slot....");
         if (ivImageDlg->DoesImageFileExistForCurrentDate(atoi(&lBuffer[4])))
         {
            TK_LOG0("...do we perform the underline");
            pCtrl->SetSelectionCharFormat(lCf);
         }
         else
         {
            TK_LOG0("File may have been deleted, cancel any underline there may be....");
            lCf.dwEffects        = 0; 
            pCtrl->SetSelectionCharFormat(lCf);
            lCf.dwEffects        = CFE_UNDERLINE; 
         }
      } 

      lFtt.chrgText.cpMin = lFtt.chrgText.cpMax;
      pCtrl->SetSel(lFtt.chrgText);
      lFtt.chrg.cpMin = lFtt.chrgText.cpMax - 1;
      if (pFullPage == false)
      {
         break;
      }
   }

   m_DiaryPage.SetSel(lLeftAt);

   // Now the image dialog may be shown again.....
   this->ivKillSelChangeProcessing = false;
   this->ivShowImages = true;

   stillRunning = false;
}

// This method adapt the passed in SYSTEMTIME structure to contain the 
// time corresponding to the passed in filename. 

void SDiaryDlg::GetTimeFromPassedInFile(SYSTEMTIME *pTime)
{
   TK_TRACER("SDiaryDlg::GetTimeFromPassedInFile(SYSTEMTIME *pTime)");
   CCommandLineInfo *lInfo = theApp.GetCmdLineInfo();

   CString lFileName = lInfo->m_strFileName;

   if (lFileName.GetLength() == 0)
   {
      return;
   }

   char *lFullPath = new char[MAX_STRING];
   char *lFloater;
   strcpy_s(lFullPath, MAX_STRING - 1, lFileName);
   lFloater = strrchr(lFullPath, '\\');
   lFloater++;
   if (atoi(lFloater) == 0)
   {
      return;
   }
   sscanf_s(lFloater, "%04d%02d%02d", 
            &(pTime->wYear),
            &(pTime->wMonth), 
            &(pTime->wDay));

   delete [] lFullPath;
}

/*
void SDiaryDlg::GetPassedInConfigFile(std::string &theFilespec)
{
	TK_TRACER("SDiaryDlg::GetPassedInConfigFile(std::string &theFilespec)");
    CCommandLineInfo *lInfo = theApp.GetCmdLineInfo();

    CString lFileName = lInfo->m_strFileName;

    if (lFileName.GetLength() == 0)
    {
		theFileSpec = "settings.dat");
        return;
    }

}
*/

void SDiaryDlg::OnExport() 
{
   TK_TRACER("SDiaryDlg::OnExport(void)");
   CWaitCursor lWaiter;

   TK_LOG0("Determine if we need the SExportDlg, or that a simple export will do");
   if (this->IsDlgButtonChecked(IDC_SMART) != 0)
   { 
      TK_LOG0("Save todolist and diary page");
      this->ivCurrentTodoList->Save();
      this->SaveOldPage(true);      
      
      TK_LOG0("Smart Export requested, complying...");
      ASSERT(ivExportDlg != NULL);
      this->ivExportDlg->DoModal();

      TK_LOG0("Load the new page");
      this->LoadNewPage();
      return;
   }

   CString lFileName = "export.txt";
   int lYear  = this->ivNewSelDate.getYear();
   int lMonth = this->ivNewSelDate.getMonth();
   int lDay   = this->ivNewSelDate.getDays();
   CString lDate;

   lDate.Format("%04d/%02d/%02d", lYear, lMonth, lDay);

   FILE *lFile = fopen(lFileName.GetBuffer(0), "a");

   if (lFile == NULL)
   {
      CString lMsg;
      lMsg.Format("File %s could not be opened for writing", lFileName);
      AfxMessageBox(lMsg);
      return;
   }

   fprintf(lFile, "\n\n        %s\n\n", lDate.GetBuffer(0));
   fprintf(lFile, "\n\n");

   long lMin = 0;
   long lMax = 0;
   char lBuffer[MAX_BUFFER + 1];
   m_DiaryPage.SetSel(0, -1);
   m_DiaryPage.GetSel(lMin, lMax);
   CString theStr = m_DiaryPage.GetSelText();
   strcpy_s(lBuffer, MAX_BUFFER, theStr);
   m_DiaryPage.SetSel(-1, -1);
      
   char * lStart  = lBuffer;
   char * lEnd    = lBuffer;
   char * lLength = lBuffer + strlen(lBuffer);

   // vervang de newlines door eos
   while (lStart < lLength)
   {
      if (*lStart == '\n')
      {
         *lStart = '\0';
      }
      lStart++;
   }

   // doorloop lBuffer
   lStart = lBuffer;
   while (lEnd < lLength)
   {
      // schuif een regel op
      lEnd += 80;
      if (lEnd > lLength)
      {
         // over het einde heen, compenseer tot einde string
         lEnd = lLength;
      }
      else
      {
         // knip in een woord, schuif op naar links
         while (*lEnd != ' ')
         {
            lEnd--;
         }

         // breek de regel af tussen de woorden
         *lEnd = '\0';
      }

      // print de regel tot de eerste eos
      fprintf(lFile, "            %s\n", lStart);

      // corrigeer indexen
      if (strlen(lStart) < (unsigned int)(lEnd - lStart - 1))
      { 
         *lEnd = ' ';
         lStart = lStart + strlen((const char *)lStart) + 1;
         lEnd = lStart;
      }
      else
      {
         lStart = lEnd + 1;
         lEnd = lStart;
      }
   }

   fclose(lFile);
}

void SDiaryDlg::OnBnClickedImage()
{
   TK_TRACER("SDiaryDlg::OnBnClickedImage()");
   CWaitCursor lWaiter;

#ifdef THIS_MAY_NOT_BE_NEEDED
   TK_LOG0("Save todolist and diary page");
   this->ivCurrentTodoList->Save();
   this->SaveOldPage(true);
#endif

   TK_LOG0("Start the image dialog");
   ASSERT(ivImageDlg != NULL);
   ivImageDlg->DoModal();

   this->UpdateStatus();
   HighlightHyperlinks(&m_DiaryPage, true);
}

void SDiaryDlg::UpdateStatus(void)
{
   TK_TRACER("SDiaryDlg::UpdateStatus()");
   CWaitCursor lWaiter;

   TK_LOG0("Update the caption on the image button if needed");
   ASSERT(ivImageDlg != NULL);
   int lCount = ivImageDlg->CountImageFiles();
   CString lMsg;
   switch (lCount)
   {
   case 0:
      lMsg = "no Images";
      break;
   case 1: 
      lMsg = "1 Image";
      break;
   default:
      lMsg.Format("%d Images", lCount);
      break;
   }

   this->GetDlgItem(IDC_IMG_PRESENT)->SetWindowText(lMsg.GetBuffer());

   TK_LOG0("Update the caption on the files button if needed");
   ASSERT(ivFilesDlg != NULL);
   lCount = ivFilesDlg->CountFiles();
   switch (lCount)
   {
   case 0:
      lMsg = "no Files";
      break;
   case 1: 
      lMsg = "1 File";
      break;
   default:
      lMsg.Format("%d Files", lCount);
      break;
   }

   this->GetDlgItem(IDC_FILES_PRESENT)->SetWindowText(lMsg.GetBuffer());
}

int SDiaryDlg::GetNumberOfLinesForDiaryWindow(void)
{
   TK_TRACER("SDiaryDlg::GetNumberOfLinesForDiaryWindow()");
   CString lFontSize = this->ivSettings.GetSetting(SET_FONT_SIZE)->GetValue();

   if (lFontSize == "8")
   {
      return 50;
   }
   if (lFontSize == "10")
   {
      return 41;
   }
   if (lFontSize == "12")
   {
      return 33;
   }
   if (lFontSize == "14")
   {
      return 27;
   }
   return 22;
}

int SDiaryDlg::GetNumberOfLinesForTodoWindow(void)
{
   TK_TRACER("SDiaryDlg::GetNumberOfLinesForTodoWindow()");
   CString lFontSize = this->ivSettings.GetSetting(SET_FONT_SIZE)->GetValue();

   if (lFontSize == "8")
   {
      return 29;
   }
   if (lFontSize == "10")
   {
      return 24;
   }
   if (lFontSize == "12")
   {
      return 19;
   }
   if (lFontSize == "14")
   {
      return 16;
   }
   return 13;
}

void SDiaryDlg::OnEnSelchangeRichedit1(NMHDR *pNMHDR, LRESULT *pResult)
{
	TK_TRACER("SDiaryDlg::OnEnSelchangeRichedit1()");
    CWaitCursor lWaiter;

    SELCHANGE *pSelChange = reinterpret_cast<SELCHANGE *>(pNMHDR);

    if (this->ivKillSelChangeProcessing == true)
    {
        return;
    }

    this->ivKillChangeProcessing = true;

    int lMax = pSelChange->chrg.cpMax;
    int lMin = pSelChange->chrg.cpMin;

    int lDiff = lMax - lMin;
    if (lDiff >= 5 && lDiff <= 25)
    {
        char lBuffer[MAX_BUFFER];

        TK_LOG0("Get the selected stuff, which may be more than we bargained for...");
        this->ivKillChangeProcessing = true;
        CString theStr = m_DiaryPage.GetSelText();
		strcpy_s(lBuffer, MAX_BUFFER, theStr);
        this->ivKillChangeProcessing = false;

        if (lBuffer[0] == 'S' &&
            lBuffer[1] == 'l' &&
            lBuffer[2] == 'o' &&
            lBuffer[3] == 't')
        {
            TK_LOG0("Sneak in a reset of the selection without anybody knowing...");
            this->ivKillSelChangeProcessing = true;
            m_DiaryPage.SetSel(lMin, lMin + 5);
            CString theStr = m_DiaryPage.GetSelText();
            strcpy_s(lBuffer, MAX_BUFFER, theStr);
		    this->ivKillSelChangeProcessing = false;

			char lSlotNo[2];

			lSlotNo[0] = lBuffer[4];
			lSlotNo[1] = '\0';

			int lSlot = atoi(lSlotNo);
	         
			if ((lSlotNo[0] == '0' ||
			     lSlotNo[0] == '1' ||
				 lSlotNo[0] == '2' ||
				 lSlotNo[0] == '3' ||
				 lSlotNo[0] == '4' ||
				 lSlotNo[0] == '5' ||
				 lSlotNo[0] == '6' ||
				 lSlotNo[0] == '7' ||
				 lSlotNo[0] == '8' ||
				 lSlotNo[0] == '9') &&
				 this->ivShowImages == true && 
				 ivImageDlg->DoesImageFileExistForCurrentDate(lSlot))
			 {
                ASSERT(ivImageDlg != NULL);
			    ivImageDlg->DoModal(lSlot);
                this->UpdateStatus();
				ivShowImages = false;
				HighlightHyperlinks(&m_DiaryPage, true);
			 }

			 // destroy the selection
			 this->ivKillChangeProcessing = true;
			 m_DiaryPage.SetSel(lMax, lMax);
			 this->ivKillChangeProcessing = false;
		  }
	   }
	 
	   this->ivKillChangeProcessing = false;
	   this->ivShowImages = true;
	   *pResult = 0;
}

void SDiaryDlg::UpdateWeekday(void)
{
   TK_TRACER("SDiaryDlg::UpdateWeekday()");
   CWaitCursor lWaiter;

   CString lMsg;
   lMsg.Format("%s, %s %d%s, %4d", 
               this->ivNewSelDate.dayAsLongString(),
               SDate::monthAsLongString((this->ivNewSelDate.getMonth())), 
               this->ivNewSelDate.getDays(), 
               SDate::getDateSuffix(this->ivNewSelDate.getDays()), 
               this->ivNewSelDate.getYear());
   ivWeekday.SetWindowText(lMsg.GetBuffer());

   // update the bold dates in the calendar control...
   NMDAYSTATE* pDayState = (NMDAYSTATE *)0;
   MONTHDAYSTATE lpDayStateArray[3] = {0, 0, 0};
   lpDayStateArray[0] = GetDayStatesForMonth(this->ivNewSelDate.getYear(), this->ivNewSelDate.getMonth(), 0);
   lpDayStateArray[1] = GetDayStatesForMonth(this->ivNewSelDate.getYear(), this->ivNewSelDate.getMonth(), 1);
   lpDayStateArray[2] = GetDayStatesForMonth(this->ivNewSelDate.getYear(), this->ivNewSelDate.getMonth(), 2);
   MonthCal_SetDayState(m_pMonthCalendar.GetSafeHwnd(),
                        3,
                        lpDayStateArray);

}

void SDiaryDlg::OnBnClickedImport()
{
   TK_TRACER("SDiaryDlg::OnBnClickedImport()");
   CWaitCursor lWaiter;

   TK_LOG0("Determine if we need the SImportDlg, or that a simple import will do");
   if (this->IsDlgButtonChecked(IDC_SMART) != 0)
   { 
      TK_LOG0("Save todolist and diary page");
      this->ivCurrentTodoList->Save();
      this->SaveOldPage(true);      
      
      TK_LOG0("Smart Import requested, complying...");
      ASSERT(ivImportDlg != NULL);
      this->ivImportDlg->DoModal();

      TK_LOG0("Load the new page");
      this->LoadNewPage();
      return;
   }

   TK_LOG0("Simple Import wanted, complying...");
   char lFilters[255] = "Text Files (*.txt)|*.txt|Diary Files (*.dry)|*.dry|All Files (*.*)|*.*||";
   HRESULT hResult;

   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];
   char  *lImportPath = new char[MAX_STRING];

   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);

   CFileDialog dlg(TRUE, NULL, NULL, OFN_FILEMUSTEXIST, lFilters);

   hResult = (int)dlg.DoModal();
   if(hResult != IDOK) 
   {
      SetCurrentDirectory(lPath);
	  delete [] lImportPath;
	  delete [] lPath;
      return;
   }

   CString lFullFileSpec;
   lDirReturn = GetCurrentDirectory(MAX_STRING, lImportPath);

   lFullFileSpec.Format("%s\\%s", lImportPath, dlg.GetFileName());

   TK_LOG1("Opening file %s", lFullFileSpec.GetBuffer());
   FILE *lFile = fopen(lFullFileSpec.GetBuffer(), "rb");
   if (lFile != NULL)
   {
      TK_LOG0("File open, reading it and appending to current page");
      long lMin = 0;
      long lMax = 0;
      char lBuffer[MAX_BUFFER];
      char lAppendBuffer[MAX_BUFFER];
      m_DiaryPage.GetSel(lMin, lMax);
      m_DiaryPage.SetSel(0, -1);
      CString theStr = m_DiaryPage.GetSelText();
	  strcpy_s(lBuffer, MAX_BUFFER, theStr);
      m_DiaryPage.SetSel(lMin, lMax);

      int lBytesRead = fread(lAppendBuffer, 1, MAX_BUFFER, lFile);
      if (lBytesRead > 0)
      {
         TK_LOG0("Import Text read, closing off buffer, and appending");
         lAppendBuffer[lBytesRead] = '\0';
         for (int i = strlen(lBuffer); i >= 0; i--)
         {
            if (lBuffer[i] == '\n')
            {
               lBuffer[i] = '\0';
            }
            else
            {
               break;
            }
         }
         strcat_s(lBuffer, MAX_BUFFER - 1, "\n\n");
         strcat_s(lBuffer, MAX_BUFFER - 1, lAppendBuffer);
         m_DiaryPage.SetWindowText(lBuffer);
         HighlightHyperlinks(&m_DiaryPage, true);
         m_DiaryPage.SetSel(lMin, lMax);
      }
      fclose(lFile);
   }
   else
   {
      CString lMsg;
      lMsg.Format("File %s could not be opened for reading", lFullFileSpec.GetBuffer());
      MessageBox(lMsg.GetBuffer());
      TK_LOG0(lMsg.GetBuffer());
   }

   SetCurrentDirectory(lPath);
   HighlightHyperlinks(&m_DiaryPage, true);

   delete [] lImportPath;
   delete [] lPath;
}

void SDiaryDlg::SavePosition(int x, int y, int width, int height)
{
   TK_TRACER("SDiaryDlg::SavePosition(void)");
   CWaitCursor lWaiter;

   TK_LOG0("Save window position to settings.dat");
   WINDOWPLACEMENT lWinPlace;
   this->GetWindowPlacement(&lWinPlace);

   // cancel save if window centered, and not moved yet...
   if (ivWindowCentered == true &&
	   (ivTop == lWinPlace.rcNormalPosition.top ||
	    ivLeft == lWinPlace.rcNormalPosition.left))
   {
	   return;
   }

   // ok, we save so set the flag to off
   ivWindowCentered = false;
   ivTop = -1;
   ivLeft = -1;

   char *lTop = new char[MAX_STRING];
   if (y == -1)
   {
	   y = lWinPlace.rcNormalPosition.top;
   }
   _itoa_s(y, lTop, MAX_STRING - 1, 10);
   SSetting *lTopSSetting = new SSetting(SET_TOP, lTop);
   this->ivSettings.AddSetting(lTopSSetting);
   delete [] lTop;

   char *lLeft = new char[MAX_STRING];
   if (x == -1)
   {
	   x = lWinPlace.rcNormalPosition.left;
   }
   _itoa_s(x, lLeft, MAX_STRING - 1, 10);
   SSetting *lLeftSSetting = new SSetting(SET_LEFT, lLeft);
   this->ivSettings.AddSetting(lLeftSSetting);
   delete [] lLeft;

   char *lWidth = new char[MAX_STRING];
   if (width == -1)
   {
       width = lWinPlace.rcNormalPosition.right - 
	           lWinPlace.rcNormalPosition.left;
   }
   _itoa_s(width, lWidth, MAX_STRING - 1, 10);
   SSetting *lWidthSSetting = new SSetting(SET_WIDTH, lWidth);
   this->ivSettings.AddSetting(lWidthSSetting);
   delete [] lWidth;

   char *lHeight = new char[MAX_STRING];
   if (height == -1)
   {
       height = lWinPlace.rcNormalPosition.bottom - 
	            lWinPlace.rcNormalPosition.top;
   }
   _itoa_s(height, lHeight, MAX_STRING - 1, 10);
   SSetting *lHeightSSetting = new SSetting(SET_HEIGHT, lHeight);
   this->ivSettings.AddSetting(lHeightSSetting);
   delete [] lHeight;

   this->ivSettings.WriteFile();
}

SDiaryPage *SDiaryDlg::getSDiaryPageForCurrentDate(void)
{
   return getSDiaryPageForDate(this->ivNewSelDate);
}

SDiaryPage *SDiaryDlg::getSDiaryPageForDate(SDate pDate)
{
   TK_TRACER("SDiaryDlg::GetSDiaryPageForCurrentDate(void)");
   CWaitCursor lWaiter;
   
   if (this->ivDiaryPage != NULL)
   {
      TK_LOG0("Delete the previous object, before creating a new one");
      delete this->ivDiaryPage;
   }

   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];
   CString lCurDir; 
   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);
   lCurDir.Format("%s\\%08d.dry", lPath, pDate.asDecimal());
   delete [] lPath;

   this->ivDiaryPage = new SDiaryPage(lCurDir,
                                      pDate, 
                                      theApp.GetCryptKey());
   return this->ivDiaryPage; 
}

CString SDiaryDlg::getFilenameForDate(SDate pDate)
{
   TK_TRACER("SDiaryDlg::getFilenameForDate(SDate pDate)");
   CString lReturn;
   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];
   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);
   lReturn.Format("%s\\%08d.dry", 
                  lPath, 
                  pDate.asDecimal());
   delete [] lPath;

   return lReturn;               
}

CString SDiaryDlg::getWindowTitle(void)
{
   TK_TRACER("SDiaryDlg::getWindowTitle(void)");
   DWORD lDirReturn = 0;
   char  *lPath = new char[MAX_STRING];
   CString lReturn;

   TK_LOG0("Retrieve the path of execution");
   lDirReturn = GetCurrentDirectory(MAX_STRING, lPath);
   lReturn.Format("%s", lPath);
   delete [] lPath;

   return lReturn; 
}


void SDiaryDlg::OnBnClickedPrev()
{
    TK_TRACER("SDiaryDlg::OnBnClickedPrev(void)");
	// iterate backward until we find another diary page
	SDate theDate = this->ivNewSelDate;
	int daysTried = 0;
    do
	{
		theDate.Subtract(0, 0, 1);
		daysTried++;

	} while (TestPage(theDate) == false && daysTried < MAX_DAYS_TO_SCAN);
	if (TestPage(theDate) == false)
	{
		MessageBox("No previous entry found in over five years", 
			       "Warning", 
				   MB_OK);
		return;
	}
    TK_LOG0("But save diary page and todolist before doing so");
    this->ivCurrentTodoList->Save();
    this->SaveOldPage(true);
	this->ivNewSelDate = theDate;
    SYSTEMTIME lTime;
	lTime.wYear  = theDate.getYear();
    lTime.wMonth = theDate.getMonth();
	lTime.wDay   = theDate.getDays();
	m_pMonthCalendar.SetCurSel(&lTime);
    this->LoadNewPage();
    this->UpdateWeekday();
    this->UpdateStatus();
}

void SDiaryDlg::OnBnClickedNext()
{
    TK_TRACER("SDiaryDlg::OnBnClickedNext(void)");
	// iterate forward until we find another diary page
	SDate theDate = this->ivNewSelDate;
	int daysTried = 0;
    do
	{
		theDate.Add(0, 0, 1);
		daysTried++;

	} while (TestPage(theDate) == false && daysTried < MAX_DAYS_TO_SCAN);
	if (TestPage(theDate) == false)
	{
		MessageBox("No next entry found in over five years", 
			       "Warning", 
				   MB_OK);
		return;
	}
    TK_LOG0("But save diary page and todolist before doing so");
    this->ivCurrentTodoList->Save();
    this->SaveOldPage(true);
	this->ivNewSelDate = theDate;
    SYSTEMTIME lTime;
	lTime.wYear  = theDate.getYear();
    lTime.wMonth = theDate.getMonth();
	lTime.wDay   = theDate.getDays();
	m_pMonthCalendar.SetCurSel(&lTime);
    this->LoadNewPage();
    this->UpdateWeekday();
    this->UpdateStatus();
}

bool SDiaryDlg::TestPage(SDate pSelDate)
{
   TK_TRACER("SDiaryDlg::TestPage(SDate pSelDate)");
   TK_LOG0("Build the correct filename, and open the file");
   CString lFileName;
   lFileName.Format("%08d.dry", pSelDate.asDecimal());

   FILE *lFile = fopen(lFileName, "rb");

   if (lFile != NULL)
   {
      fclose(lFile);
	  return true;
   }
   return false;
}


void SDiaryDlg::OnBnClickedFiles()
{
    TK_TRACER("SDiaryDlg::OnBnClickedFiles()");
    CWaitCursor lWaiter;

    //this->SavePosition();	
   
    TK_LOG0("Files Dialog requested, complying...");
    ASSERT(ivFilesDlg != NULL);
	this->ivFilesDlg->DoModal();
    this->UpdateStatus();
}

void SDiaryDlg::OnBnClickedLblConfigure()
{
    TK_TRACER("SDiaryDlg::OnBnClickedConfig()");
    CWaitCursor lWaiter;

    //this->SavePosition();	
   
    TK_LOG0("Configure Dialog requested, complying...");
	if (this->ivShortcutDlg->DoModal() == IDOK)
	{
		SetLabelButtons();
		GetDlgItem(IDC_TODO1)->SetWindowTextA(ivShortcutDlg->ivTodoList1->GetName(true).c_str());
		GetDlgItem(IDC_TODO2)->SetWindowTextA(ivShortcutDlg->ivTodoList2->GetName(true).c_str());
		GetDlgItem(IDC_TODO3)->SetWindowTextA(ivShortcutDlg->ivTodoList3->GetName(true).c_str());
		GetDlgItem(IDC_TODO4)->SetWindowTextA(ivShortcutDlg->ivTodoList4->GetName(true).c_str());
		GetDlgItem(IDC_TODO5)->SetWindowTextA(ivShortcutDlg->ivTodoList5->GetName(true).c_str());
		GetDlgItem(IDC_TODO6)->SetWindowTextA(ivShortcutDlg->ivTodoList6->GetName(true).c_str());
		GetDlgItem(IDC_TODO7)->SetWindowTextA(ivShortcutDlg->ivTodoList7->GetName(true).c_str());
	}
}

void SDiaryDlg::SetLabelButtons()
{
	ivShortcutDlg->SetDiaryDlg(this);
	ivShortcutDlg->GetLabel(1)->SetButton(IDC_LBL1);
	ivShortcutDlg->GetLabel(2)->SetButton(IDC_LBL2);
	ivShortcutDlg->GetLabel(3)->SetButton(IDC_LBL3);
	ivShortcutDlg->GetLabel(4)->SetButton(IDC_LBL4);
	ivShortcutDlg->GetLabel(5)->SetButton(IDC_LBL5);
	ivShortcutDlg->GetLabel(6)->SetButton(IDC_LBL6);
	ivShortcutDlg->GetLabel(7)->SetButton(IDC_LBL7);
	ivShortcutDlg->GetLabel(8)->SetButton(IDC_LBL8);
	ivShortcutDlg->GetLabel(9)->SetButton(IDC_LBL9);
	ivShortcutDlg->GetLabel(10)->SetButton(IDC_LBL10);
	ivShortcutDlg->GetLabel(11)->SetButton(IDC_LBL11);
	ivShortcutDlg->GetLabel(12)->SetButton(IDC_LBL12);
	ivShortcutDlg->GetLabel(13)->SetButton(IDC_LBL13);
	ivShortcutDlg->GetLabel(14)->SetButton(IDC_LBL14);
	ivShortcutDlg->GetLabel(15)->SetButton(IDC_LBL15);
	ivShortcutDlg->GetLabel(16)->SetButton(IDC_LBL16);
	ivShortcutDlg->GetLabel(17)->SetButton(IDC_LBL17);
	ivShortcutDlg->GetLabel(18)->SetButton(IDC_LBL18);
}

void SDiaryDlg::EnableAllControls(BOOL theStatus)
{
   this->GetDlgItem(IDC_MONTHCALENDAR1)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_SMART)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_SEARCH)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_IMAGE)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_FILES)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_IMPORT)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_EXPORT)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_INSERT_TIME)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_PREV)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_NEXT)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_NUMDAYS)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_PRINT)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_TODO0)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_TODO1)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_TODO2)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_TODO3)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_TODO4)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_TODO5)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_TODO6)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_TODO7)->EnableWindow(theStatus);

   this->GetDlgItem(IDC_LBL1)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL2)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL3)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL4)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL5)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL6)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL7)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL8)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL9)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL10)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL11)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL12)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL13)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL14)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL15)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL16)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL17)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL18)->EnableWindow(theStatus);
   this->GetDlgItem(IDC_LBL_CONFIGURE)->EnableWindow(theStatus);

   if (theStatus == TRUE)
   {
	    this->SetLabelButtons();
		GetDlgItem(IDC_TODO1)->SetWindowTextA(ivShortcutDlg->ivTodoList1->GetName(true).c_str());
		GetDlgItem(IDC_TODO2)->SetWindowTextA(ivShortcutDlg->ivTodoList2->GetName(true).c_str());
		GetDlgItem(IDC_TODO3)->SetWindowTextA(ivShortcutDlg->ivTodoList3->GetName(true).c_str());
		GetDlgItem(IDC_TODO4)->SetWindowTextA(ivShortcutDlg->ivTodoList4->GetName(true).c_str());
		GetDlgItem(IDC_TODO5)->SetWindowTextA(ivShortcutDlg->ivTodoList5->GetName(true).c_str());
		GetDlgItem(IDC_TODO6)->SetWindowTextA(ivShortcutDlg->ivTodoList6->GetName(true).c_str());
		GetDlgItem(IDC_TODO7)->SetWindowTextA(ivShortcutDlg->ivTodoList7->GetName(true).c_str());
   }
}

void SDiaryDlg::InsertString(CString &theInsertString) 
{
   TK_TRACER("SDiaryDlg::InsertString)");
   CWaitCursor lWaiter;

   long lMin = 0;
   long lMax = 0;

   TK_LOG0("set the selection to a single point of insertion");
   m_DiaryPage.GetSel(lMin, lMax);
   m_DiaryPage.SetSel(lMin, lMin);

   TK_LOG0("insert the string into the CRichEditCtrl of the diary page");
   int theSize = theInsertString.GetLength() - 1;
   if (theInsertString[theSize] != '\n')
   {
       theInsertString += " ";
   }
   m_DiaryPage.ReplaceSel(theInsertString.GetBuffer());
   m_DiaryPage.SetFocus();
}

void SDiaryDlg::OnBnClickedLbl1()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(1);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl2()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(2);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl3()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(3);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl4()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(4);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl5()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(5);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl6()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(6);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl7()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(7);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl8()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(8);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl9()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(9);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl10()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(10);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl11()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(11);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl12()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(12);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl13()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(13);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl14()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(14);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl15()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(15);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl16()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(16);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl17()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(17);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::OnBnClickedLbl18()
{
	// retrieve the corresponding Shortcut object
	SShortcut *theShortcut = ivShortcutDlg->GetLabel(18);

	// if timestamp is set, call Add timestamp
	if (theShortcut->NeedsTimestamp())
	{
		OnInsertTime();
	}

	// retrieve the string to insert, and insert it
	InsertString(theShortcut->GetInsertText());
}

void SDiaryDlg::SetNewDate(SDate theNewDate)
{
     // the CMonthCalCtrl sends spurious SelChange messages, for the 
     // already selected date. This if is supposed to catch these....

     TK_LOG0("If new date doesn't match previous date...");
     if (theNewDate != this->ivNewSelDate)
     {
        TK_LOG0("Switch pages, saving the old and loading the new");
        this->ivOldSelDate = this->ivNewSelDate;
        this->ivNewSelDate = theNewDate;
        SYSTEMTIME lTime;
		lTime.wYear  = theNewDate.getYear();
		lTime.wMonth = theNewDate.getMonth();
		lTime.wDay   = theNewDate.getDays();
		m_pMonthCalendar.SetCurSel(&lTime);
        this->SaveOldPage();
        this->LoadNewPage();
        this->ivCurrentTodoList->Save();
        this->UpdateWeekday();
        this->UpdateStatus();
     }
}
void SDiaryDlg::OnBnClickedPrint()
{
    TK_LOG0("But save diary page and todolist before doing so");
    this->ivCurrentTodoList->Save();
    this->SaveOldPage(true);
    this->LoadNewPage();

	// get number of pages
	int NumDays = 0;
	char theString[21];
	this->GetDlgItem(IDC_NUMDAYS)->GetWindowTextA(theString, 20);
	NumDays = atoi(theString);
	if (NumDays == 0)
	{
		NumDays = 1;
	}

	// and print...
	this->ivExportDlg->PrintEntries(NumDays);
}

void SDiaryDlg::OnBnClickedTodo0()
{
	ActivateTodoList(ivShortcutDlg->ivTodoList0);
}

void SDiaryDlg::OnBnClickedTodo1()
{
	ActivateTodoList(ivShortcutDlg->ivTodoList1);
}

void SDiaryDlg::OnBnClickedTodo2()
{
	ActivateTodoList(ivShortcutDlg->ivTodoList2);
}

void SDiaryDlg::OnBnClickedTodo3()
{
	ActivateTodoList(ivShortcutDlg->ivTodoList3);
}

void SDiaryDlg::OnBnClickedTodo4()
{
	ActivateTodoList(ivShortcutDlg->ivTodoList4);
}

void SDiaryDlg::OnBnClickedTodo5()
{
	ActivateTodoList(ivShortcutDlg->ivTodoList5);
}

void SDiaryDlg::OnBnClickedTodo6()
{
	ActivateTodoList(ivShortcutDlg->ivTodoList6);
}

void SDiaryDlg::OnBnClickedTodo7()
{
	ActivateTodoList(ivShortcutDlg->ivTodoList7);
}

void SDiaryDlg::OnBnClickedCheck1()
{
	// TODO: Add your control notification handler code here
}

void SDiaryDlg::ActivateTodoList(STodoList *pTodoList)
{
	if (ivCurrentTodoList != 0)
	{
		// save old todolist
		ivCurrentTodoList->Save();
	}
	if (pTodoList != ivCurrentTodoList)
	{
		// save the new one
		ivCurrentTodoList = pTodoList;

		// and load it...
		ivCurrentTodoList->Load();
	}
}

void SDiaryDlg::OnEnChangeNumdays()
{
   char *lNumDays = new char[MAX_STRING];
   this->GetDlgItem(IDC_NUMDAYS)->GetWindowTextA(lNumDays, MAX_STRING - 1);
   SSetting *lNumDaysSSetting = new SSetting(SET_NUMDAYS, lNumDays);
   this->ivSettings.AddSetting(lNumDaysSSetting);
   delete [] lNumDays;

   this->ivSettings.WriteFile();
}

void SDiaryDlg::OnWindowPosChanged(WINDOWPOS* lpwndpos)
{
	CResizeDlg::OnWindowPosChanged(lpwndpos);

	SavePosition(lpwndpos->x, lpwndpos->y, lpwndpos->cx, lpwndpos->cy);
}

bool SDiaryDlg::wasConfigRead()
{
	return ivConfigRead;
}