//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         diaryDlg.h
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Declaration of Class diaryDlg.
//
//////////////////////////////////////////////////////////////////////////////////////////////
#ifndef DIARYDLG_H_INCLUDED
#define DIARYDLG_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "diary.h"
#include "diarypage.h"
#include "searchDlg.h"
#include "imagedlg.h"
#include "settingsfile.h"
#include "sdate.h"
#include "afxwin.h"
#include "importdlg.h"
#include "exportdlg.h"
#include "filesdlg.h"
#include "SLabelDlg.h"

#include "ResizeDlg.h"

class STodoList;

//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Class SDiaryDlg
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  this dialog handles the main user interface of the Diary application. It 
//                handles the calendar, the rich edit controls, and the various other controls
//                placed on the main dialog. This dialog is fully resizeable. 
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//
//////////////////////////////////////////////////////////////////////////////////////////////
class SDiaryDlg : public CResizeDlg
{
	friend class SImageDlg;

// Construction
public:
   SDiaryDlg(CWnd* pParent = NULL);   // standard constructor

   bool SaveOldPage(bool pFinal = false);
   void LoadNewPage(void);

   SSettingsFile *GetSettings(void)   { return &this->ivSettings; };

   void HighlightSearchStrings(CRichEditCtrl *pCtrl);
   void HighlightHyperlinks(CRichEditCtrl *pCtrl, bool pFullPage = false);
   void GetTimeFromPassedInFile(SYSTEMTIME *pTime);
   SDate GetCurrentDate(void)         { return this->ivNewSelDate; };
   CString getFilenameForDate(SDate pDate);
   CString getWindowTitle(void);
   void EnableAllControls(BOOL theStatus);
   void UpdateWeekday(void);
   void SavePosition(int x = -1, int y = -1, int width = -1, int height = -1);
   SDiaryPage *getSDiaryPageForCurrentDate(void);
   SDiaryPage *getSDiaryPageForDate(SDate pDate);
   bool SDiaryDlg::TestPage(SDate theSelDate);
   MONTHDAYSTATE GetDayStatesForMonth(int pYear, int pMonth, int pMonthOffset);
   void SetLabelButtons();
   void InsertString(CString &theInsertString);
   void SetNewDate(SDate theNewDate);
   bool isListDirty() { return ivListDirty; };
   void clearListDirty() { ivListDirty = false; };
   bool wasConfigRead();

private:
   bool ivPageDirty;
   bool ivListDirty;
   bool ivShowImages;
   bool ivKillChangeProcessing;
   bool ivKillSelChangeProcessing;

   // stuff to correctly reposition if offscreen
   bool ivWindowCentered;
   int  ivTop;
   int  ivLeft;

   SSearchDlg *ivSearchDlg;      
   SImageDlg  *ivImageDlg;
   SImportDlg *ivImportDlg;
   SExportDlg *ivExportDlg;
   SFilesDlg  *ivFilesDlg;
   SLabelDlg  *ivShortcutDlg;

   SDiaryPage *ivDiaryPage;

   SSettingsFile ivSettings;

// Dialog Data
   //{{AFX_DATA(SDiaryDlg)
   enum { IDD = IDD_DIARY_DIALOG };
   CEdit           m_EncryptionKey;
   CRichEditCtrl   m_DiaryPage;
   CMonthCalCtrl   m_pMonthCalendar;
   //}}AFX_DATA

   // ClassWizard generated virtual function overrides
   //{{AFX_VIRTUAL(SDiaryDlg)
protected:
   virtual void DoDataExchange(CDataExchange* pDX);   // DDX/DDV support
   //}}AFX_VIRTUAL

// Implementation
protected:
   HICON m_hIcon;
   SDate ivOldSelDate;
   SDate ivNewSelDate;

   void SetFont(int pFontSize);
   // Generated message map functions
   //{{AFX_MSG(SDiaryDlg)
   virtual BOOL OnInitDialog();
   afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
   afx_msg void OnPaint();
   afx_msg HCURSOR OnQueryDragIcon();
   afx_msg void OnGetdaystateMonthcalendar1(NMHDR* pNMHDR, LRESULT* pResult);
   afx_msg void OnSelchangeMonthcalendar1(NMHDR* pNMHDR, LRESULT* pResult);
   virtual void OnOK();
   virtual void OnCancel();
   afx_msg void OnWindowPosChanged(WINDOWPOS* lpwndpos);
   afx_msg void OnChangeRichedit1();
   afx_msg void OnChangeRichedit2();
   afx_msg void OnInsertTime();
   afx_msg void OnSearch();
   afx_msg void OnExport();
   //}}AFX_MSG
   DECLARE_MESSAGE_MAP()

public:
   CRichEditCtrl   m_TodoList;
   afx_msg void OnBnClickedImage();
   void UpdateStatus(void);
   int GetNumberOfLinesForDiaryWindow(void);
   int GetNumberOfLinesForTodoWindow(void);
   afx_msg void OnEnSelchangeRichedit1(NMHDR *pNMHDR, LRESULT *pResult);
   CDC ivPrintDC;
   void ActivateTodoList(STodoList *pTodoList);

private:
   CStatic ivWeekday;
   int ivSelectedTodoList;
   int ivOldSelectedTodoList;
   STodoList *ivCurrentTodoList;
   bool ivConfigRead;

public:
   afx_msg void OnBnClickedImport();
   virtual ~SDiaryDlg(void);
   afx_msg void OnBnClickedButton1();
   afx_msg void OnBnClickedPrev();
   afx_msg void OnBnClickedNext();
   afx_msg void OnBnClickedFiles();
   afx_msg void OnBnClickedLblConfigure();
   afx_msg void OnBnClickedLbl1();
   afx_msg void OnBnClickedLbl2();
   afx_msg void OnBnClickedLbl3();
   afx_msg void OnBnClickedLbl4();
   afx_msg void OnBnClickedLbl5();
   afx_msg void OnBnClickedLbl6();
   afx_msg void OnBnClickedLbl7();
   afx_msg void OnBnClickedLbl8();
   afx_msg void OnBnClickedLbl9();
   afx_msg void OnBnClickedLbl10();
   afx_msg void OnBnClickedLbl11();
   afx_msg void OnBnClickedLbl12();
   afx_msg void OnBnClickedLbl13();
   afx_msg void OnBnClickedLbl14();
   afx_msg void OnBnClickedLbl15();
   afx_msg void OnBnClickedLbl16();
   afx_msg void OnBnClickedLbl17();
   afx_msg void OnBnClickedLbl18();
   afx_msg void OnBnClickedPrint();
   afx_msg void OnBnClickedTodo0();
   afx_msg void OnBnClickedTodo1();
   afx_msg void OnBnClickedTodo2();
   afx_msg void OnBnClickedTodo3();
   afx_msg void OnBnClickedTodo4();
   afx_msg void OnBnClickedTodo5();
   afx_msg void OnBnClickedTodo6();
   afx_msg void OnBnClickedTodo7();
   afx_msg void OnBnClickedCheck1();
   afx_msg void OnEnChangeNumdays();
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DIARYDLG_H__56E16265_1095_11D6_924F_000102213AEE__INCLUDED_)
