//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         diaryPage.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Implementation of Class diaryPage.
//
//////////////////////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "common.h"
#include "toolkit.h"
#include "diary.h"
#include "diarypage.h"

SDiaryPage::SDiaryPage(void)
{
   TK_TRACER("SDiaryPage::SDiaryPage(void)");
   this->ivFileSpec = new char[MAX_STRING + 1];
   this->ivFileSpec[0] = '\0';
   this->ivCryptKey = new char[MAX_STRING + 1]; 
   TK_LOG0("Error, SDiaryPage constructed with default constructor");
}

SDiaryPage::SDiaryPage(CString pFileSpec, SDate pDate, char * pCryptKey)
{
   TK_TRACER("SDiaryPage::SDiaryPage(CString pFileSpec, SDate pDate, char *pCryptKey)");
   this->ivFileSpec = new char[MAX_STRING + 1];
   strcpy_s(this->ivFileSpec, MAX_STRING, pFileSpec);
   this->ivDate     = pDate;
   this->ivCryptKey = new char[MAX_STRING + 1]; 
   strcpy_s(this->ivCryptKey, MAX_STRING, pCryptKey );

   this->LoadParagraphs();
}

SDiaryPage::~SDiaryPage(void)
{
   TK_TRACER("SDiaryPage::~SDiaryPage(void)");
   delete [] ivFileSpec;
   delete [] ivCryptKey;
}


void SDiaryPage::LoadParagraphs(void)
{
   TK_TRACER("SDiaryPage::LoadParagraphs(void)");

   CWaitCursor lWaiter; 
   FILE *lFile = fopen(this->GetFileSpec(), "rb");
   if (lFile != NULL)
   {
      TK_LOG0("File is open, read it and decode");
      char lBuffer[MAX_BUFFER + 1];
      lBuffer[0] = '\0';
      int lBytesRead = fread(lBuffer, 1, MAX_BUFFER, lFile);
      if (lBytesRead > 0)
      {
         lBuffer[lBytesRead] = '\0';
         theApp.Decode(lBuffer, lBytesRead, this->ivCryptKey);

         char *lRunner = lBuffer;
         TK_LOG0("File read, creating paragraphs");
         SDiaryParagraph *lNewParagraph = SDiaryParagraph::CreateFromBuffer(lRunner);
         int lSeqNo = 0;
         int lHours = 0;
         int lMinutes = 0;
         while(lNewParagraph != NULL && lRunner <= lBuffer + strlen(lBuffer))
         {
            if (lNewParagraph->hasExplicitTimeStamp() == true)
            {
               lSeqNo = 0;
               lNewParagraph->SetSeqNo(lSeqNo++);
               lHours = lNewParagraph->getHours();
               lMinutes = lNewParagraph->getMinutes();
            }
            else 
            {
               lNewParagraph->SetSeqNo(lSeqNo++);
               lNewParagraph->SetTimeStamp(lHours, lMinutes);
            }
            this->Merge(lNewParagraph);
            //lNewParagraph->DisplayDump();
            lNewParagraph = SDiaryParagraph::CreateFromBuffer(lRunner);
         }
         
         TK_LOG0("Processing done, closing file");
         fclose(lFile);
      }
   }
}

char * SDiaryPage::GetFileSpec(void)
{
   TK_TRACER("SDiaryPage::GetFileSpec(void)");
   return this->ivFileSpec;
}

void SDiaryPage::AddParagraphAtEnd(SDiaryParagraph *pNewParagraph)
{
   TK_TRACER("SDiaryPage::AddParagraphAtEnd(SDiaryParagraph *pNewParagraph)");

   if (pNewParagraph != NULL)
   {
      this->ivParagraphList.AddTail(pNewParagraph);
   }
}

void SDiaryPage::InsertBefore(SDiaryPage *pImport)
{
   TK_TRACER("SDiaryPage::InsertBefore()");

   TK_LOG0("Iterate through the paragraphs of the Import page in reverse order");
   POSITION lPos = NULL;
   SDiaryParagraph *lParagraph = pImport->GetLastParagraph(lPos);
   while (lParagraph != NULL)
   {
      TK_LOG0("Insert each before the ones already there");
      this->ivParagraphList.AddHead(lParagraph);
      lParagraph = pImport->GetPreviousParagraph(lPos);
   }
}

void SDiaryPage::InsertAfter(SDiaryPage *pImport)
{
   TK_TRACER("SDiaryPage::InsertAfter()");

   TK_LOG0("Iterate through the paragraphs of the Import page");
   POSITION lPos = NULL;
   SDiaryParagraph *lParagraph = pImport->GetFirstParagraph(lPos);
   while (lParagraph != NULL)
   {
      TK_LOG0("Insert each after the ones already there");
      this->ivParagraphList.AddTail(lParagraph);
      lParagraph = pImport->GetNextParagraph(lPos);
   }
}

void SDiaryPage::Merge(SDiaryPage *pImport)
{
   TK_TRACER("SDiaryPage::Merge(SDiaryPage *pImport)");

   TK_LOG0("Now pull out all paragraphs to be imported");
   POSITION lPos = NULL;
   SDiaryParagraph *lNextParagraph = pImport->GetFirstParagraph(lPos);
   while (lNextParagraph != NULL)
   {
      this->Merge(lNextParagraph);
      pImport->UnlinkFirstParagraph();
      lNextParagraph = pImport->GetFirstParagraph(lPos);
   }
}

void SDiaryPage::Write(void)
{
   TK_TRACER("SDiaryPage::Write(void)");

   TK_LOG0("Prepare a buffer to write the page in");
   char lBuffer[MAX_BUFFER];
   lBuffer[0] = '\0';
   POSITION lPos = NULL;
   SDiaryParagraph *lParagraph = this->GetFirstParagraph(lPos);
   while (lParagraph != NULL)
   {
      lParagraph->Write(lBuffer);
      lParagraph = this->GetNextParagraph(lPos);
   }

   FILE *lFile = fopen(this->GetFileSpec(), "wb");
   if (lFile != NULL)
   {
      int lLength = strlen(lBuffer);
      theApp.Encode(lBuffer, lLength, this->ivCryptKey);
      if (fwrite(lBuffer, 1, lLength, lFile) != lLength)
      {
         CString lMsg;
         lMsg.Format("File %s could not be (entirely) written", this->GetFileSpec());
         AfxMessageBox(lMsg);
      }

      fclose(lFile);
   }
   else
   {
      CString lMsg;
      lMsg.Format("File %s could not be opened for writing", this->GetFileSpec());
      AfxMessageBox(lMsg);
   }    
}

void SDiaryPage::Merge(SDiaryParagraph *pNextParagraph)
{
   TK_TRACER("SDiaryPage::Merge(SDiaryParagraph *)");

   TK_LOG0("if list is empty, append at end");
   if (this->ivParagraphList.IsEmpty() == (BOOL)true)
   {
      TK_LOG0("List Empty, add at tail");
      this->ivParagraphList.AddTail(pNextParagraph);
      return;
   }

   TK_LOG0("List not empty, read through it");
   POSITION lPos = NULL;
   POSITION lOldPos = NULL;
   SDiaryParagraph *lPageEntry = this->GetFirstParagraph(lPos);

   bool lInserted = false;
   while (lPageEntry != NULL)
   {
      TK_LOG0("Get next entry");
      if (lPageEntry == NULL)
      {
         TK_LOG0("Next Paragraph NULL, leave loop");
         break;
      }

	  // TODO: Instead of a total match, devise a compare that
	  //       yields a matching percentage. That way we can 
	  //       ask the user to choose if they are largely equal. 

	  if (*pNextParagraph == *lPageEntry)
      {
         TK_LOG0("Entries identical, return without adding");
         return;
      }
      if (pNextParagraph->comesBefore(lPageEntry))
      {
         TK_LOG0("Next entry higher, insert before");
         if (pNextParagraph->hasExplicitTimeStamp() == false)
         {
            TK_LOG0("No explicit Timestamp, did previous record have same timestamp?");
            SDiaryParagraph *lPrev = this->GetPreviousParagraph(lPos);
            if (lPrev != NULL && lPrev->getTime() == pNextParagraph->getTime())
            {
               pNextParagraph->SetSeqNo(lPrev->getSequenceNumber() + 1);
            }
         }
         this->ivParagraphList.InsertBefore(lOldPos, pNextParagraph);
         lInserted = true;
         break;
      }
      lOldPos = lPos;
      lPageEntry = this->GetNextParagraph(lPos);
   }
   if (lInserted == false)
   {
      TK_LOG0("All entries lower, append at end");
      this->ivParagraphList.AddTail(pNextParagraph);
   }
}

// This method is a remnant of the time when I thought I had to sort the paragraphs
// Now I know that clever inserting is also a way to do it...

void SDiaryPage::SortParagraphs(void)
{
   TK_TRACER("SDiaryPage::SortParagraphs(void)");
   TK_LOG0("check first if there are paragraphs in this page. If not, return");
   if (this->ivParagraphList.GetCount() <= 1)
   {
      return;
   }

   TK_LOG0("check first if the paragraphs are in sequence. If so, return");
   bool lSorted = true;
   POSITION lPos = NULL;
   SDiaryParagraph *lParagraph = this->GetFirstParagraph(lPos);
   SDiaryParagraph *lPreviousParagraph = NULL;
   while (lParagraph != NULL && lSorted == true)
   {
      if (lParagraph->comesBefore(lPreviousParagraph))
      {
         lSorted = false;
         break;
      }
      lPreviousParagraph = lParagraph;
      lParagraph = this->GetNextParagraph(lPos);
   }
   if (lSorted == true)
   {
      return;
   }

   TK_LOG0("Now we do have to sort it...");

   // TODO: Really sort them this time...
}

SDiaryParagraph *SDiaryPage::GetFirstParagraph(POSITION &pPos)
{
   TK_TRACER("SDiaryPage::GetFirstParagraph()");
   if (this->ivParagraphList.IsEmpty() == (BOOL)true)
   {
      TK_LOG0("List empty, returning NULL");
      pPos = NULL;
      return NULL;
   }
   pPos = this->ivParagraphList.GetHeadPosition();
   return this->ivParagraphList.GetNext(pPos);
}

SDiaryParagraph *SDiaryPage::GetLastParagraph(POSITION &pPos)
{
   TK_TRACER("SDiaryPage::GetLastParagraph()");
   if (this->ivParagraphList.IsEmpty() == (BOOL)true)
   {
      TK_LOG0("List empty, returning NULL");
      pPos = NULL;
      return NULL;
   }
   pPos = this->ivParagraphList.GetTailPosition();
   return this->ivParagraphList.GetPrev(pPos);
}
SDiaryParagraph *SDiaryPage::GetNextParagraph(POSITION &pPos)
{
   TK_TRACER("SDiaryPage::GetNextParagraph()");
   if (this->ivParagraphList.IsEmpty() == (BOOL)true)
   {
      return NULL;
   }
   if (pPos == NULL)
   {
      return NULL;
   }
   return this->ivParagraphList.GetNext(pPos);
}

SDiaryParagraph *SDiaryPage::GetPreviousParagraph(POSITION &pPos)
{
   TK_TRACER("SDiaryPage::GetPreviousParagraph()");
   if (this->ivParagraphList.IsEmpty() == (BOOL)true)
   {
      return NULL;
   }
   if (pPos == NULL)
   {
      return NULL;
   }
   return this->ivParagraphList.GetPrev(pPos);
}

void SDiaryPage::UnlinkFirstParagraph(void)
{
   POSITION lPos;
   lPos = this->ivParagraphList.GetHeadPosition();
   this->ivParagraphList.RemoveAt(lPos);
}

void SDiaryPage::DisplayDump(void)
{
   CString lMsg;
   lMsg.Format("ivDate: %08d\nivCryptKey: %s\nivFileSpec: %s\nList Count: %d", 
               this->ivDate.asDecimal(),
               this->ivCryptKey,
               this->ivFileSpec,
               this->ivParagraphList.GetCount());
   AfxMessageBox(lMsg);
   POSITION lPos = this->ivParagraphList.GetHeadPosition();
   while (lPos != NULL)
   {
      SDiaryParagraph *lPar = this->ivParagraphList.GetNext(lPos);
      if (lPar != NULL)
      {
         lPar->DisplayDump();
      }
      else
      {
         AfxMessageBox("NULL Entry found");
      }
   }
}

void SDiaryPage::Export(FILE *pFile, CString pDirectory, OutputType pType)
{
	switch (pType)
	{
	case enSimpleTextNoImages:
		fprintf(pFile, "%02d/%02d/%04d\n\n", 
			    this->ivDate.getDays(), 
				this->ivDate.getMonth(), 
				this->ivDate.getYear());
		break;

	case enSimpleHTMLNoImages:
	case enSimpleHTMLInlineImages:
	case enSimpleHTMLLinkedImages:
	    fprintf(pFile, "<h3>%02d/%02d/%04d</h3>\n",
			    this->ivDate.getDays(), 
				this->ivDate.getMonth(), 
				this->ivDate.getYear());
		break;

	case enFramedHTML:
		break;
	}

    POSITION lPos = this->ivParagraphList.GetHeadPosition();
    while (lPos != NULL)
    {
        SDiaryParagraph *lPar = this->ivParagraphList.GetNext(lPos);
        if (lPar != NULL)
        {
            lPar->Export(pFile, pDirectory, pType);
        }
        else
        {
            AfxMessageBox("NULL Entry found");
        }
    }
}