#include "stdafx.h"
#include <time.h>        // PLATFORM_SPECIFIC for time routines
#include <sys/timeb.h>   // PLATFORM_SPECIFIC for time routines

#include "sdate.h"
#include "toolkit.h"

SDate::SDate(void)
{
   time_t          lTime;
   struct tm       lToday;
   char lTimeStamp[50];

   time(&lTime);
   localtime_s( &lToday, &lTime );
   strftime(lTimeStamp, MAX_STRING, "%Y-%m-%d %H:%M:%S", &lToday);
}

SDate::SDate(long pDecimalDate)
{
   int lYear    = 0;
   int lMonth = JANUARY;
   int lDay     = 0;
   this->DecimalDateAsParts(pDecimalDate, 
	                         &lYear, 
		  		                &lMonth, 
                      		 &lDay);
   if (!this->isValidDate(lYear, lMonth, lDay))
   {
      this->Set(lYear, lMonth, lDay);
   }
}

SDate::SDate(int pYear, int pMonth, int pDay)
{
   int lYear    = (int)pYear;
   int lMonth = (int)pMonth;
   int lDay     = (int)pDay; 

   if (this->isValidDate(lYear, lMonth, lDay))
   {
      this->Set(lYear, lMonth, lDay);
   }
}

bool SDate::operator != (SDate &pOtherDate)
{
   return (ivYear  != pOtherDate.ivYear  ||
           ivMonth != pOtherDate.ivMonth ||
           ivDay   != pOtherDate.ivDay);
}

SDate &SDate::operator = (SYSTEMTIME pOtherDate)
{
	this->ivYear = pOtherDate.wYear;
	this->ivMonth = pOtherDate.wMonth;
	this->ivDay = pOtherDate.wDay;

    return *this;
}

SDate &SDate::operator = (SDate &pOtherDate)
{
   this->ivYear = pOtherDate.getYear();
   this->ivMonth = pOtherDate.getMonth();
   this->ivDay = pOtherDate.getDays();

   return *this;
}

bool SDate::operator >= (SDate &pOtherDate)
{
   return (this->asDecimal() >= pOtherDate.asDecimal());
}

bool SDate::operator <= (SDate &pOtherDate)
{
   return (this->asDecimal() <= pOtherDate.asDecimal());
}

bool SDate::operator >(SDate &pOtherDate)
{
   return (this->asDecimal() > pOtherDate.asDecimal());
}

bool SDate::operator < (SDate &pOtherDate)
{
   return (this->asDecimal() < pOtherDate.asDecimal());
}

SDate::~SDate(void)
{
}

int SDate::findDoomsDate(int pYear, int pMonth)
{
	switch(pMonth)
	{
		case JANUARY: 
			if (isLeapYear(pYear))
			{
				return 4;
			}
			else
			{
				return 31;
			}
			break;

		case FEBRUARY:
			if (isLeapYear(pYear))
			{
				return 1;
			}
			else
			{
				return 7;
			}
			break;

		case MARCH:
			return 7;

		case APRIL:
			return 4;

		case MAY:
			return 9;

		case JUNE:
			return 6;

		case JULY:
			return 11;

		case AUGUST:
			return 8;

		case SEPTEMBER:
			return 5;

		case OCTOBER:
			return 10;

		case NOVEMBER:
			return 7;

		case DECEMBER:
			return 12;

		default:
			printf("%s: illegal month (%d)\n", __FILE__, pMonth);
			return 0;
	}
};

enWeekday SDate::findDoomsDay(int pYear, int pMonth, int pDay)
{
	int lCentury = pYear / 100;
	int	lFactorQ = (pYear % 100) / 12;
	int	lFactorR = (pYear % 100) % 12;
	int	lFactorS = lFactorR / 4;
	int	lWeekDay;

	if (asDecimal(pYear, pMonth, pDay) <= 15821004)
	{
		/*****************************************************************/
		/* separate treatment for julian years                           */
		/*****************************************************************/

		lWeekDay = SUNDAY - lCentury;
		while (lWeekDay < MONDAY)
		{
			lWeekDay += 7;
		}
	}		
    else if (asDecimal(pYear, pMonth, pDay) >= 15821015)
	{
		/******************************************************************/
		/* otherwise the calculation for gregorian dates                  */
		/******************************************************************/

		switch (lCentury % 4)
		{
			case 0:
				lWeekDay = TUESDAY;
				break;

			case 1:
				lWeekDay = SUNDAY;
				break;

			case 2:
				lWeekDay = FRIDAY;
				break;

			case 3:
				lWeekDay = WEDNESDAY;
				break;
	    }
	}
	else 
	{
		TK_LOG0("Date is inbetween the Julian and the Gregorian calendar");
		return INVALID_DAY;
	}

	lWeekDay = (lWeekDay - 1 + lFactorQ + lFactorR + lFactorS) % 7 + 1;

	return (enWeekday)lWeekDay;
};


enWeekday SDate::calculateWeekDay (int pYear, int pMonth, int pDay)
{
	int       lDoomsDate;
	enWeekday lDoomsDay;
	int       lDaysFromDoomsDay;
	enWeekday lWeekDay;

	if (!isValidDate(pYear, pMonth, pDay))
	{
		TK_LOG0("Date given is invalid");
		return INVALID_DAY;
	}

	lDoomsDate = findDoomsDate(pYear, pMonth);
	lDoomsDay  = findDoomsDay(pYear, pMonth, pDay);

	lDaysFromDoomsDay = pDay - lDoomsDate;

	/**********************************************************************/
	/* we add 5 weeks here, to avoid having to count backwards            */
	/**********************************************************************/
	lWeekDay = (enWeekday)((35 + lDoomsDay - 1 + lDaysFromDoomsDay) % 7 + 1);

	return lWeekDay;
};

void SDate::Set(int pYear, int pMonth, int pDays, bool pCanCorrect)
{
   if (!this->isValidDate(pYear, (int)pMonth, pDays))
   {
      if (pCanCorrect == true)
      {
         TK_LOG0("Trying to set an invalid date, correcting...");
         this->FindFirstValidDate(&pYear, &pMonth, &pDays);
      }
      else
      {
         pYear = 1900;
         pMonth = 1;
         pDays = 1;
      }
   }
   this->ivYear  = pYear;
   this->ivMonth = pMonth;
   this->ivDay   = pDays;
}

void SDate::Set(long pDecimalDate, bool pCanCorrect)
{
   int   lYear = 0;
   int   lMonth = INVALID_MONTH;
   int   lDays = 0;
   this->DecimalDateAsParts(pDecimalDate, 
                            &lYear, 
				            &lMonth, 
				            &lDays);
   this->Set((int)lYear,(int)lMonth, (int)lDays, pCanCorrect);
}

void SDate::Set(char *pStringDate, bool pCanCorrect)
{
   TK_LOG0("Assuming 8 digit sortable date to be in first 8 positions of string");
   CString lRawInput(pStringDate);
   CString lDate = lRawInput.Left(8);
   long lNumDate = atoi(lDate.GetBuffer());
   this->Set(lNumDate, pCanCorrect);
}

void SDate::DecimalDateAsParts(long     pDate, 
	                           int     *pYear, 
				               int     *pMonth, 
				               int     *pDay)
{
	*pYear   =  (int)(pDate / 10000);
	pDate   -=  *pYear * 10000;
	*pMonth  =  (int)(pDate / 100);
	pDate   -=  *pMonth * 100;
	*pDay    =  (int)pDate;
}

bool SDate::isValidDate(int pYear, int pMonth, int pDay)
{
	if (asDecimal(pYear, pMonth, pDay) <= 15821004 ||
     	asDecimal(pYear, pMonth, pDay) >= 15821015)
	{
		if (pMonth >= JANUARY || pMonth <= DECEMBER)
		{
		    if (pDay > 0 && pDay <= maxDaysInMonth(pYear, pMonth))
		    {
		        return true;
		    }
		}
	}
	return false;
};

long SDate::asDecimal(int pYear, int pMonth, int pDay)
{
	long lDecimalDate;

	lDecimalDate  = pYear * 10000;
	lDecimalDate += pMonth * 100;
	lDecimalDate += pDay;

	return lDecimalDate;
};

long SDate::asDecimal(void)
{
   return asDecimal(this->ivYear, (int)this->ivMonth, this->ivDay);
}

bool SDate::isLeapYear(int pYear)
{
	if (pYear > 1582)
	{
		/******************************************************************/
		/* for the gregorian calendar, leapyears are dividable by four,   */
		/* but not by hundred, or dividable by fourhundred                */
		/******************************************************************/
 		if ((pYear %   4 == 0) &&
 			(pYear % 100 != 0) ||
			(pYear % 400 == 0))
 		{
 			return true;
 		}
	}
	else
	{
		/******************************************************************/
		/* for the Julian calendar, leapyears are dividable by four       */
		/******************************************************************/
 		if (pYear % 4 == 0)
 		{
 			return true;
 		}
	}
	return false;
};

int SDate::maxDaysInMonth(int pYear, int pMonth)
{
	switch (pMonth)
	{
    	case JANUARY:
		case MARCH:
		case MAY:
		case JULY:
		case AUGUST:
		case OCTOBER:
		case DECEMBER:
			return 31;

		case APRIL:
		case JUNE:
		case SEPTEMBER:
		case NOVEMBER:
			return 30;

		case FEBRUARY:
			if (isLeapYear(pYear))
			{
				return 29;
			}
			else
			{
				return 28;
			}

		default:
			return 0;
	}
}

char *SDate::dayAsLongString(enWeekday pDay)
{
	switch (pDay)
	{
        case MONDAY:
		    return STR_MONDAY;

		case TUESDAY:
			return STR_TUESDAY;

		case WEDNESDAY:
			return STR_WEDNESDAY;

		case THURSDAY:
			return STR_THURSDAY;

		case FRIDAY:
			return STR_FRIDAY;

		case SATURDAY:
			return STR_SATURDAY;

		case SUNDAY:
			return STR_SUNDAY;

		default:
			return STR_NO_DAY;
	}
};

char *SDate::dayAsShortString(enWeekday pDay)
{
	switch (pDay)
	{
        case MONDAY:
		    return STR_MON;

		case TUESDAY:
			return STR_TUE;

		case WEDNESDAY:
			return STR_WED;

		case THURSDAY:
			return STR_THU;

		case FRIDAY:
			return STR_FRI;

		case SATURDAY:
			return STR_SAT;

		case SUNDAY:
			return STR_SUN;

		default:
			return STR_NO_DAY;
	}
};

char *SDate::monthAsLongString(int pMonth)
{
	switch(pMonth)
	{
    	case JANUARY:
			return STR_JANUARY;

		case FEBRUARY:
			return STR_FEBRUARY;

		case MARCH:
			return STR_MARCH;

		case APRIL:
			return STR_APRIL;

		case MAY:
			return STR_MAY;

		case JUNE:
			return STR_JUNE;

		case JULY:
			return STR_JULY;

		case AUGUST:
			return STR_AUGUST;

		case SEPTEMBER:
			return STR_SEPTEMBER;

		case OCTOBER:
			return STR_OCTOBER;

		case NOVEMBER:
			return STR_NOVEMBER;

		case DECEMBER:
			return STR_DECEMBER;

		default:
			return STR_NO_MONTH;
	}
}

char *SDate::monthAsShortString(int pMonth)
{
	switch(pMonth)
	{
    	case JANUARY:
			return STR_JAN;

		case FEBRUARY:
			return STR_FEB;

		case MARCH:
			return STR_MAR;

		case APRIL:
			return STR_APR;

		case MAY:
			return STR_MAY;

		case JUNE:
			return STR_JUN;

 		case JULY:
			return STR_JUL;

		case AUGUST:
			return STR_AUG;

		case SEPTEMBER:
			return STR_SEP;

		case OCTOBER:
			return STR_OCT;

		case NOVEMBER:
			return STR_NOV;

		case DECEMBER:
			return STR_DEC;

		default:
			return STR_NO_MONTH;
	}
}

void SDate::FindFirstValidDate(int *pYear, int *pMonth, int *pDays)
{
	if (asDecimal(*pYear, (int)*pMonth, *pDays) <= 15821004 ||
    	asDecimal(*pYear, (int)*pMonth, *pDays) >= 15821015)
	{
		while (*pMonth < JANUARY)
		{
            TK_LOG0("Month below border value, going back one year to increase months");
			*pMonth += 12;
            *pYear--;
		}
        while (*pMonth > DECEMBER)
		{
			TK_LOG0("Month above border value, carrying to next year");
            *pMonth -= 12;
            *pYear++;
		}
		while (*pDays < 1)
        {
            TK_LOG0("Negative days value, borrow from months");
            *pMonth--;
            *pDays += maxDaysInMonth(*pYear, (int)(*pMonth - 1)); 
            if (*pMonth < 1)
            {
                *pYear--;
                *pMonth = 12;
                *pDays = 31;
            }
        }
        while (*pDays > maxDaysInMonth(*pYear, (int)*pMonth))
		{
			*pMonth++;
            *pDays -= maxDaysInMonth(*pYear, (int)*pMonth);
            if (*pMonth > 12)
            {
                *pYear++;
                *pMonth = 1;
            }
		}
	}
	else
	{
        TK_LOG0("Date inbetween calendars, reverting to old calendar..."); 
		*pYear  = 1582;
        *pMonth = 10;
        *pDays  = 10;
	}
}

char *SDate::dayAsLongString(void)
{
   enWeekday lDay = this->calculateWeekDay ((int)this->ivYear, 
                                            (int)this->ivMonth, 
                                            (int)this->ivDay);
   return this->dayAsLongString(lDay);
}

char *SDate::getDateSuffix(int pDate)
{
   switch (pDate)
   {
   case 1:
   case 21:
   case 31:
      return "st";
   case 2:
   case 22:
      return "nd";
   case 3:
   case 23:
      return "rd";
   default:
      return "th"; 
   }
}

void SDate::Add(int pYears, int pMonths, int pDays)
{
   this->ivDay += pDays;
   while (this->ivDay > this->maxDaysInMonth(this->ivYear, this->ivMonth))
   {
      this->ivMonth++;
	  if (ivMonth > 12)
	  {
		  ivMonth = 1;
		  ivYear++;
	  }
      this->ivDay -= this->maxDaysInMonth(this->ivYear, this->ivMonth - 1);
   }
   this->ivMonth += pMonths;
   while (this->ivMonth > 12)
   {
      this->ivYear++;
      this->ivMonth -= 12;
   }
   this->ivYear += pYears;
   this->FindFirstValidDate(&(this->ivYear), &(this->ivMonth), &(this->ivDay));
}

void SDate::Subtract(int pYears, int pMonths, int pDays)
{
   this->ivDay -= pDays;
   while (this->ivDay < 1)
   {
      this->ivMonth--;
	  if (ivMonth == 0)
	  {
		  ivMonth = 12;
		  ivYear--;
	  }
      this->ivDay += this->maxDaysInMonth(this->ivYear, this->ivMonth);
   }
   this->ivMonth -= pMonths;
   while (this->ivMonth < 1)
   {
      this->ivYear--;
      this->ivMonth += 12;
   }
   this->ivYear -= pYears;
   this->FindFirstValidDate(&(this->ivYear), &(this->ivMonth), &(this->ivDay));
}

int  SDate::daysFrom(SDate theOtherDate)
{
	int theCount = 0;
	SDate theOther = theOtherDate;
	SDate theCopy = *this;

	// determine if we should count at all...
	if (theCopy != theOther)
	{
		// determine which way to go...
		if (theCopy < theOther)
		{
			// count forwards...
			while (theCopy < theOther)
			{
				theOther.Subtract(0, 0, 1);
				theCount++;
			}				
		}
		else
		{
			// count backwards....
			while (theCopy > theOther)
			{
				theCopy.Subtract(0, 0, 1);
				theCount--;
			}
		}
	}

	return theCount;
}
